@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine.h.in,v 1.162 2006/12/21 00:09:18 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * public xine-lib (libxine) interface and documentation
  *
@@ -91,8 +89,8 @@ typedef struct xine_video_port_s xine_video_port_t;
 
 #ifndef XINE_DISABLE_DEPRECATED_FEATURES
 /* convenience types: simple player UIs might want to call ports drivers */
-typedef xine_audio_port_t xine_ao_driver_t;
-typedef xine_video_port_t xine_vo_driver_t;
+typedef xine_audio_port_t xine_ao_driver_t XINE_DEPRECATED;
+typedef xine_video_port_t xine_vo_driver_t XINE_DEPRECATED;
 #endif
 
 
@@ -115,8 +113,8 @@ int  xine_check_version (int major, int minor, int sub) XINE_PROTECTED;
 /* static info - which libxine release this header came from */
 #define XINE_MAJOR_VERSION 1
 #define XINE_MINOR_VERSION 1
-#define XINE_SUB_VERSION   4
-#define XINE_VERSION       "1.1.4"
+#define XINE_SUB_VERSION   14
+#define XINE_VERSION       "1.1.14"
 
 /*
  * pre-init the xine engine
@@ -176,6 +174,8 @@ void xine_close_video_driver (xine_t *self, xine_video_port_t  *driver) XINE_PRO
 #define XINE_VISUAL_TYPE_DIRECTX           7 /* used by the win32/msvc port */
 #define XINE_VISUAL_TYPE_CACA              8
 #define XINE_VISUAL_TYPE_MACOSX            9
+#define XINE_VISUAL_TYPE_XCB              11
+#define XINE_VISUAL_TYPE_RAW              12
 
 /*
  * free all resources, close all plugins, close engine.
@@ -260,7 +260,7 @@ int  xine_play (xine_stream_t *stream, int start_pos, int start_time) XINE_PROTE
  *
  * returns 1 if OK, 0 on error (use xine_get_error for details)
  */
-int  xine_trick_mode (xine_stream_t *stream, int mode, int value) XINE_PROTECTED;
+int  xine_trick_mode (xine_stream_t *stream, int mode, int value) XINE_PROTECTED XINE_DEPRECATED;
 
 /* trick modes */
 #define XINE_TRICK_MODE_OFF                0
@@ -419,14 +419,64 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
  * image format can be YUV 4:2:0 or 4:2:2
  * will copy the image data into memory that <img> points to
  * (interleaved for yuv 4:2:2 or planary for 4:2:0)
+ * 
+ * xine_get_current_frame() requires that <img> must be able
+ * to hold the image data. Use a NULL pointer to retrieve the
+ * necessary parameters for calculating the buffer size. Be
+ * aware that the image can change between two successive calls
+ * so you better pause the stream.
  *
- * returns 1 on success, 0 failure.
+ * xine_get_current_frame_s() requires to specify the buffer
+ * size and it returns the needed / used size. It won't copy
+ * image data into a too small buffer.
+ *
+ * xine_get_current_frame_alloc() takes care of allocating
+ * a buffer on its own, so image data can be retrieved by
+ * a single call without the need to pause the stream.
+ * 
+ * xine_get_current_frame_data() passes the parameters of the
+ * previously mentioned functions plus further information in
+ * a structure and can work like the _s or _alloc function
+ * respectively depending on the passed flags.
+ *
+ * all functions return 1 on success, 0 failure.
  */
 int  xine_get_current_frame (xine_stream_t *stream,
 			     int *width, int *height,
 			     int *ratio_code, int *format,
 			     uint8_t *img) XINE_PROTECTED;
 
+int  xine_get_current_frame_s (xine_stream_t *stream,
+			     int *width, int *height,
+			     int *ratio_code, int *format,
+			     uint8_t *img, int *img_size) XINE_PROTECTED;
+
+int  xine_get_current_frame_alloc (xine_stream_t *stream,
+			     int *width, int *height,
+			     int *ratio_code, int *format,
+			     uint8_t **img, int *img_size) XINE_PROTECTED;
+
+typedef struct {
+
+  int      width;
+  int      height;
+  int      crop_left;
+  int      crop_right;
+  int      crop_top;
+  int      crop_bottom;
+  int      ratio_code;
+  int      interlaced;
+  int      format;
+  int      img_size;
+  uint8_t *img;
+} xine_current_frame_data_t;
+
+#define XINE_FRAME_DATA_ALLOCATE_IMG (1<<0)
+
+int  xine_get_current_frame_data (xine_stream_t *stream,
+			     xine_current_frame_data_t *data,
+			     int flags) XINE_PROTECTED;
+
 /* xine image formats */
 #define XINE_IMGFMT_YV12 (('2'<<24)|('1'<<16)|('V'<<8)|'Y')
 #define XINE_IMGFMT_YUY2 (('2'<<24)|('Y'<<16)|('U'<<8)|'Y')
@@ -515,7 +565,7 @@ int xine_get_video_frame (xine_stream_t *stream,
 			  int *ratio_code, 
 			  int *duration, /* msec */
 			  int *format,
-			  uint8_t *img) XINE_PROTECTED;
+			  uint8_t *img) XINE_PROTECTED XINE_DEPRECATED;
 
 /* TODO: xine_get_audio_frame */
 
@@ -791,20 +841,13 @@ int    xine_get_log_section_count(xine_t *self) XINE_PROTECTED;
 const char *const *xine_get_log_names(xine_t *self) XINE_PROTECTED;
 
 /* print some log information to <buf> section */
-#ifndef __GNUC__
 void   xine_log (xine_t *self, int buf,
-		 const char *format, ...) XINE_PROTECTED;
+		 const char *format, ...) XINE_FORMAT_PRINTF(3, 4) XINE_PROTECTED;
 void   xine_vlog(xine_t *self, int buf,
-		  const char *format, va_list args) XINE_PROTECTED;
-#else
-void   xine_log (xine_t *self, int buf,
-		 const char *format, ...) __attribute__ ((__format__(printf, 3, 4))) XINE_PROTECTED;
-void   xine_vlog(xine_t *self, int buf,
-		  const char *format, va_list args) __attribute__ ((__format__(printf, 3, 0))) XINE_PROTECTED;
-#endif
+		  const char *format, va_list args) XINE_FORMAT_PRINTF(3, 0) XINE_PROTECTED;
 
 /* get log messages of specified section */
-const char *const *xine_get_log (xine_t *self, int buf) XINE_PROTECTED;
+char *const *xine_get_log (xine_t *self, int buf) XINE_PROTECTED;
 
 /* log callback will be called whenever something is logged */
 typedef void (*xine_log_cb_t) (void *user_data, int section);
@@ -843,6 +886,8 @@ int  xine_get_status (xine_stream_t *stream) XINE_PROTECTED;
  * try to find out audio/spu language of given channel
  * (use -1 for current channel)
  *
+ * lang must point to a buffer of at least XINE_LANG_MAX bytes
+ *
  * returns 1 on success, 0 on failure
  */
 int xine_get_audio_lang (xine_stream_t *stream, int channel,
@@ -1052,6 +1097,8 @@ const char *xine_get_post_plugin_description  (xine_t *self,
 /* get lists of available audio and video output plugins */
 const char *const *xine_list_audio_output_plugins (xine_t *self) XINE_PROTECTED;
 const char *const *xine_list_video_output_plugins (xine_t *self) XINE_PROTECTED;
+/* typemask is (1ULL << XINE_VISUAL_TYPE_FOO) | ... */
+const char *const *xine_list_video_output_plugins_typed (xine_t *self, uint64_t typemask) XINE_PROTECTED;
 
 /* get list of available demultiplexor plugins */
 const char *const *xine_list_demuxer_plugins(xine_t *self) XINE_PROTECTED;
@@ -1077,7 +1124,7 @@ void xine_plugins_garbage_collector(xine_t *self) XINE_PROTECTED;
 #ifndef XINE_DISABLE_DEPRECATED_FEATURES
 /* talk to video output driver - old method */
 int    xine_gui_send_vo_data (xine_stream_t *self,
-			      int type, void *data) XINE_PROTECTED;
+			      int type, void *data) XINE_PROTECTED XINE_DEPRECATED;
 #endif
 
 /* new (preferred) method to talk to video driver. */
@@ -1112,7 +1159,7 @@ typedef struct {
    *
    * this will be called by the video driver to find out
    * how big the video output area size will be for a
-   * given video size. The ui should _not_ adjust it's
+   * given video size. The ui should _not_ adjust its
    * video out area, just do some calculations and return
    * the size. This will be called for every frame, ui
    * implementation should be fast.
@@ -1136,12 +1183,12 @@ typedef struct {
    * frame output callback
    *
    * this will be called by the video driver for every frame
-   * it's about to draw. ui can adapt it's size if necessary
+   * it's about to draw. ui can adapt its size if necessary
    * here.
    * note: the ui doesn't have to adjust itself to this
    * size, this is just to be taken as a hint.
    * ui must return the actual size of the video output
-   * area and the video output driver will do it's best
+   * area and the video output driver will do its best
    * to adjust the video frames to that size (while
    * preserving aspect ratio and stuff).
    *    dest_x, dest_y: offset inside window
@@ -1191,6 +1238,149 @@ typedef struct {
 
 } x11_visual_t;
 
+/*
+ * this is the visual data struct any xcb gui
+ * must supply to the xine_open_video_driver call
+ * ("data" parameter)
+ */
+typedef struct {
+
+  /* some information about the display */
+  void        *connection; /* xcb_connection_t */
+  void        *screen;     /* xcb_screen_t     */
+
+  /* window to display the video in / on */
+  unsigned int window; /* xcb_window_t */
+
+  void        *user_data;
+
+  /*
+   * dest size callback
+   *
+   * this will be called by the video driver to find out
+   * how big the video output area size will be for a
+   * given video size. The ui should _not_ adjust its
+   * video out area, just do some calculations and return
+   * the size. This will be called for every frame, ui
+   * implementation should be fast.
+   * dest_pixel_aspect should be set to the used display pixel aspect.
+   * NOTE: Semantics has changed: video_width and video_height
+   * are no longer pixel aspect corrected. Get the old semantics
+   * in the UI with
+   *   *dest_pixel_aspect = display_pixel_aspect;
+   *   if (video_pixel_aspect >= display_pixel_aspect)
+   *     video_width  = video_width * video_pixel_aspect / display_pixel_aspect + .5;
+   *   else
+   *     video_height = video_height * display_pixel_aspect / video_pixel_aspect + .5;
+   */
+  void (*dest_size_cb) (void *user_data,
+			int video_width, int video_height,
+			double video_pixel_aspect,
+			int *dest_width, int *dest_height,
+			double *dest_pixel_aspect);
+
+  /*
+   * frame output callback
+   *
+   * this will be called by the video driver for every frame
+   * it's about to draw. ui can adapt its size if necessary
+   * here.
+   * note: the ui doesn't have to adjust itself to this
+   * size, this is just to be taken as a hint.
+   * ui must return the actual size of the video output
+   * area and the video output driver will do its best
+   * to adjust the video frames to that size (while
+   * preserving aspect ratio and stuff).
+   *    dest_x, dest_y: offset inside window
+   *    dest_width, dest_height: available drawing space
+   *    dest_pixel_aspect: display pixel aspect
+   *    win_x, win_y: window absolute screen position
+   * NOTE: Semantics has changed: video_width and video_height
+   * are no longer pixel aspect corrected. Get the old semantics
+   * in the UI with
+   *   *dest_pixel_aspect = display_pixel_aspect;
+   *   if (video_pixel_aspect >= display_pixel_aspect)
+   *     video_width  = video_width * video_pixel_aspect / display_pixel_aspect + .5;
+   *   else
+   *     video_height = video_height * display_pixel_aspect / video_pixel_aspect + .5;
+   */
+  void (*frame_output_cb) (void *user_data,
+			   int video_width, int video_height,
+			   double video_pixel_aspect,
+			   int *dest_x, int *dest_y,
+			   int *dest_width, int *dest_height,
+			   double *dest_pixel_aspect,
+			   int *win_x, int *win_y);
+
+} xcb_visual_t;
+
+/**************************************************
+ * XINE_VO_RAW struct definitions
+ *************************************************/
+/*  frame_format definitions */
+#define XINE_VORAW_YV12 1
+#define XINE_VORAW_YUY2 2
+#define XINE_VORAW_RGB 4
+
+/*  maximum number of overlays the raw driver can handle */
+#define XINE_VORAW_MAX_OVL 16
+
+/* raw_overlay_t struct used in raw_overlay_cb callback */
+typedef struct {
+  uint8_t *ovl_rgba;
+  int ovl_w, ovl_h; /* overlay's width and height */
+  int ovl_x, ovl_y; /* overlay's top-left display position */
+} raw_overlay_t;
+
+/* this is the visual data struct any raw gui
+ * must supply to the xine_open_video_driver call
+ * ("data" parameter)
+ */
+typedef struct {
+  void *user_data;
+
+  /* OR'ed frame_format
+   * Unsupported frame formats are converted to rgb.
+   * XINE_VORAW_RGB is always assumed by the driver, even if not set.
+   * So a frontend must at least support rgb.
+   * Be aware that rgb requires more cpu than yuv,
+   * so avoid its usage for video playback.
+   * However, it's useful for single frame capture (e.g. thumbs)
+   */
+  int supported_formats;
+
+  /* raw output callback
+   * this will be called by the video driver for every frame
+   *
+   * If frame_format==XINE_VORAW_YV12, data0 points to frame_width*frame_height Y values
+   *                                   data1 points to (frame_width/2)*(frame_height/2) U values
+   *                                   data2 points to (frame_width/2)*(frame_height/2) V values
+   *
+   * If frame_format==XINE_VORAW_YUY2, data0 points to frame_width*frame_height*2 YU/Y²V values
+   *                                   data1 is NULL
+   *                                   data2 is NULL
+   *
+   * If frame_format==XINE_VORAW_RGB, data0 points to frame_width*frame_height*3 RGB values
+   *                                  data1 is NULL
+   *                                  data2 is NULL
+   */
+  void (*raw_output_cb) (void *user_data, int frame_format,
+                        int frame_width, int frame_height,
+			double frame_aspect,
+			void *data0, void *data1, void *data2);
+
+  /* raw overlay callback
+   * this will be called by the video driver for every new overlay state
+   * overlays_array points to an array of num_ovl raw_overlay_t
+   * Note that num_ovl can be 0, meaning "end of overlay display"
+   * num_ovl is at most XINE_VORAW_MAX_OVL */
+  void (*raw_overlay_cb) (void *user_data, int num_ovl,
+                         raw_overlay_t *overlays_array);
+} raw_visual_t;
+/**********************************************
+ * end of vo_raw defs
+ *********************************************/
+
 /*
  * this is the visual data struct any fb gui
  * may supply to the xine_open_video_driver call
@@ -1454,12 +1644,21 @@ int  xine_config_lookup_entry (xine_t *self, const char *key,
 /*
  * update a config entry (which was returned from lookup_entry() )
  *
- * xine will make a deep copy of the data in the entry into it's internal
+ * xine will make a deep copy of the data in the entry into its internal
  * config database.
  */
 void xine_config_update_entry (xine_t *self,
 			       const xine_cfg_entry_t *entry) XINE_PROTECTED;
 
+/*
+ * translation of old configuration entry names
+ */
+typedef struct {
+  const char *old_name, *new_name;
+} xine_config_entry_translation_t;
+
+void xine_config_set_translation_user (const xine_config_entry_translation_t *) XINE_PROTECTED;
+
 /*
  * load/save config data from/to afile (e.g. $HOME/.xine/config)
  */
@@ -1503,6 +1702,7 @@ void xine_config_reset (xine_t *self) XINE_PROTECTED;
 #define XINE_EVENT_SPU_BUTTON            11 /* the mouse pointer enter/leave a button */
 #define XINE_EVENT_DROPPED_FRAMES        12 /* number of dropped frames is too high */
 #define XINE_EVENT_MRL_REFERENCE_EXT     13 /* demuxer->frontend: MRL reference(s) for the real stream */
+#define XINE_EVENT_AUDIO_AMP_LEVEL       14 /* report current audio amp level (l/r/mute) */
 
 
 /* input events coming from frontend */
@@ -1581,6 +1781,7 @@ void xine_config_reset (xine_t *self) XINE_PROTECTED;
 #define XINE_EVENT_VDR_AUDIO            332
 #define XINE_EVENT_VDR_INFO             333
 #define XINE_EVENT_VDR_CHANNELPREVIOUS  334
+#define XINE_EVENT_VDR_SUBTITLES        335
 /* some space for further keys */
 #define XINE_EVENT_VDR_SETVIDEOWINDOW   350
 #define XINE_EVENT_VDR_FRAMESIZECHANGED 351
@@ -1838,8 +2039,8 @@ typedef struct {
  * This is the mechanism to report async errors from engine.
  *
  * If frontend knows about the XINE_MSG_xxx type it may safely 
- * ignore the 'explanation' field and provide it's own custom
- * dialog to the 'parameters'.
+ * ignore the 'explanation' field and provide its own custom
+ * dialogue to the 'parameters'.
  *
  * right column specifies the usual parameters.
  */
@@ -2017,16 +2218,16 @@ void        xine_osd_free          (xine_osd_t *self) XINE_PROTECTED;
 typedef enum {
 	XINE_TVSYSTEM_PAL = 0,
 	XINE_TVSYSTEM_NTSC
-} xine_tvsystem;
+} xine_tvsystem XINE_DEPRECATED;
 
 /* connect to nvtvd server and save current TV and X settings */
-int xine_tvmode_init (xine_t *self) XINE_PROTECTED;
+int xine_tvmode_init (xine_t *self) XINE_PROTECTED XINE_DEPRECATED;
 
 /* Turn tvmode on/off (1/0)*/
-int xine_tvmode_use(xine_t *self, int use_tvmode) XINE_PROTECTED;
+int xine_tvmode_use(xine_t *self, int use_tvmode) XINE_PROTECTED XINE_DEPRECATED;
 
 /* Set which tv system to use: XINE_TVSYSTEM_PAL or XINE_TVSYSTEM_NTSC */
-void xine_tvmode_set_tvsystem(xine_t *self, xine_tvsystem system) XINE_PROTECTED;
+void xine_tvmode_set_tvsystem(xine_t *self, xine_tvsystem system) XINE_PROTECTED XINE_DEPRECATED;
 
 /* try to change TV state if enabled
  *   type select 'regular' (0) or 'TV' (1) state
@@ -2035,16 +2236,16 @@ void xine_tvmode_set_tvsystem(xine_t *self, xine_tvsystem system) XINE_PROTECTED
  *   fps frame rate the mode should match best or 0 if unknown
  * returns: finally selected state
  */
-int xine_tvmode_switch (xine_t *self, int type, int width, int height, double fps) XINE_PROTECTED;
+int xine_tvmode_switch (xine_t *self, int type, int width, int height, double fps) XINE_PROTECTED XINE_DEPRECATED;
 
 /* adapt (maximum) output size to visible area if necessary and return pixel
  * aspect and real frame rate if available
  */
 void xine_tvmode_size (xine_t *self, int *width, int *height,
-		       double *pixelratio, double *fps) XINE_PROTECTED;
+		       double *pixelratio, double *fps) XINE_PROTECTED XINE_DEPRECATED;
 
 /* restore old TV and X settings and close nvtvd connection */
-void xine_tvmode_exit (xine_t *self) XINE_PROTECTED;
+void xine_tvmode_exit (xine_t *self) XINE_PROTECTED XINE_DEPRECATED;
 
 #endif
 
