@@ -79,6 +79,7 @@ static const ff_codec_t ff_audio_lookup[] = {
   {BUF_AUDIO_14_4,       CODEC_ID_RA_144,         "Real 14.4 (ffmpeg)"},
   {BUF_AUDIO_28_8,       CODEC_ID_RA_288,         "Real 28.8 (ffmpeg)"},
   {BUF_AUDIO_MPEG,       CODEC_ID_MP3,            "MP3 (ffmpeg)"},
+  {BUF_AUDIO_MP3ADU,     CODEC_ID_MP3ADU,         "MPEG-3 adu (ffmpeg)"},
   {BUF_AUDIO_MSADPCM,    CODEC_ID_ADPCM_MS,       "MS ADPCM (ffmpeg)"},
   {BUF_AUDIO_QTIMAADPCM, CODEC_ID_ADPCM_IMA_QT,   "QT IMA ADPCM (ffmpeg)"},
   {BUF_AUDIO_MSIMAADPCM, CODEC_ID_ADPCM_IMA_WAV,  "MS IMA ADPCM (ffmpeg)"},
@@ -177,7 +178,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
           this->context->bit_rate    = audio_header->nAvgBytesPerSec * 8;
       
           if(audio_header->cbSize > 0) {
-            this->context->extradata = xine_xmalloc(audio_header->cbSize);
+            this->context->extradata = malloc(audio_header->cbSize);
             this->context->extradata_size = audio_header->cbSize;
             memcpy( this->context->extradata, 
                     (uint8_t *)audio_header + sizeof(xine_waveformatex),
@@ -199,10 +200,10 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
             this->audio_channels    = this->buf[0x37];
             /* this->audio_bits = buf->content[0x35] */
   
-            this->context->block_align = _X_BE_16(&this->buf[0x2A]);
-  
+            this->context->block_align = _X_BE_32(&this->buf[0x18]);
+
             this->context->extradata_size = 5*sizeof(short);
-            this->context->extradata      = xine_xmalloc(this->context->extradata_size);
+            this->context->extradata      = malloc(this->context->extradata_size);
   
             ptr = (short *) this->context->extradata;
   
@@ -211,7 +212,49 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
             ptr[2] = _X_BE_16(&this->buf[0x16]); /* subpacket flavour */
             ptr[3] = _X_BE_32(&this->buf[0x18]); /* coded frame size */
             ptr[4] = 0;                          /* codec's data length  */
+
+	    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                    "ffmpeg_audio_dec: 28_8 audio channels %d bits %d sample rate %d block align %d\n",
+		     this->audio_channels, this->audio_bits, this->audio_sample_rate,
+		     this->context->block_align);
             break;
+	  case BUF_AUDIO_COOK:
+	    {
+	      int version;
+	      int data_len;
+	      int extradata;
+
+	      version = _X_BE_16 (this->buf+4);
+	      if (version == 4) {
+		this->audio_sample_rate = _X_BE_16 (this->buf+48);
+		this->audio_bits = _X_BE_16 (this->buf+52);
+		this->audio_channels = _X_BE_16 (this->buf+54);
+		data_len = _X_BE_32 (this->buf+67);
+		extradata = 71;
+	      } else {
+		this->audio_sample_rate = _X_BE_16 (this->buf+54);
+		this->audio_bits = _X_BE_16 (this->buf+58);
+		this->audio_channels = _X_BE_16 (this->buf+60);
+		data_len = _X_BE_32 (this->buf+74);
+		extradata = 78;
+	      }
+	      this->context->block_align = _X_BE_16 (this->buf+44);
+
+              xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+	              "ffmpeg_audio_dec: cook audio channels %d bits %d sample rate %d block align %d\n",
+		      this->audio_channels, this->audio_bits, this->audio_sample_rate,
+		      this->context->block_align);
+
+              if (extradata + data_len > this->size)
+                break; /* abort early - extradata length is bad */
+
+	      this->context->extradata_size = data_len;
+	      this->context->extradata      = malloc(this->context->extradata_size +
+						     FF_INPUT_BUFFER_PADDING_SIZE);
+	      xine_fast_memcpy (this->context->extradata, this->buf + extradata,
+				this->context->extradata_size);
+	      break;
+	    }
           default:
             xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
                     "ffmpeg_audio_dec: unknown header with buf type 0x%X\n", codec_type);
@@ -228,11 +271,12 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
       this->context->sample_rate = this->audio_sample_rate;
       this->context->channels    = this->audio_channels;
       this->context->codec_id    = this->codec->id;
+      this->context->codec_type  = this->codec->type;
       this->context->codec_tag   = _x_stream_info_get(this->stream, XINE_STREAM_INFO_AUDIO_FOURCC);
   
       this->size = 0;
   
-      this->decode_buffer = xine_xmalloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
+      this->decode_buffer = calloc(1, AVCODEC_MAX_AUDIO_FRAME_SIZE);
   
       return;
     }
@@ -240,8 +284,8 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
              (buf->decoder_info[1] == BUF_SPECIAL_STSD_ATOM)) {
 
     this->context->extradata_size = buf->decoder_info[2];
-    this->context->extradata = xine_xmalloc(buf->decoder_info[2] +
-      FF_INPUT_BUFFER_PADDING_SIZE);
+    this->context->extradata = malloc(buf->decoder_info[2] +
+				      FF_INPUT_BUFFER_PADDING_SIZE);
     memcpy(this->context->extradata, buf->decoder_info_ptr[2],
       buf->decoder_info[2]);
 
@@ -267,7 +311,26 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
       this->decoder_ok = 1;
     }
 
+    if( buf->decoder_flags & BUF_FLAG_PREVIEW )
+      return;
+
+    ff_audio_ensure_buffer_size(this, this->size + buf->size);
+    xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
+    this->size += buf->size;
+
     if (!this->output_open) {
+      if (!this->audio_bits || !this->audio_sample_rate || !this->audio_channels) {
+        avcodec_decode_audio (this->context,
+                              (int16_t *)this->decode_buffer,
+                              &decode_buffer_size,
+                              &this->buf[0],
+                              this->size);
+	this->audio_bits = this->context->bits_per_sample;
+	this->audio_sample_rate = this->context->sample_rate;
+	this->audio_channels = this->context->channels;
+	if (!this->audio_bits || !this->audio_sample_rate || !this->audio_channels)
+	  return;
+      }
       this->output_open = (this->stream->audio_out->open) (this->stream->audio_out,
         this->stream, this->audio_bits, this->audio_sample_rate,
         _x_ao_channels2mode(this->audio_channels));
@@ -277,13 +340,6 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
     if (!this->output_open)
       return;
 
-    if( buf->decoder_flags & BUF_FLAG_PREVIEW )
-      return;
-
-    ff_audio_ensure_buffer_size(this, this->size + buf->size);
-    xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
-    this->size += buf->size;
-
     if (buf->decoder_flags & BUF_FLAG_FRAME_END)  { /* time to decode a frame */
 
       offset = 0;
@@ -324,7 +380,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 
           /* fill up this buffer */
           xine_fast_memcpy(audio_buffer->mem, &this->decode_buffer[out],
-            bytes_to_send);
+			   bytes_to_send);
           /* byte count / 2 (bytes / sample) / channels */
           audio_buffer->num_frames = bytes_to_send / 2 / this->audio_channels;
 
@@ -355,7 +411,8 @@ static void ff_audio_reset (audio_decoder_t *this_gen) {
   if( this->context && this->decoder_ok ) {
     pthread_mutex_lock (&ffmpeg_lock);
     avcodec_close (this->context);
-    avcodec_open (this->context, this->codec);
+    if (avcodec_open (this->context, this->codec) < 0)
+      this->decoder_ok = 0;
     pthread_mutex_unlock (&ffmpeg_lock);
   }
 }
@@ -393,7 +450,7 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
 
   ff_audio_decoder_t *this ;
 
-  this = (ff_audio_decoder_t *) xine_xmalloc (sizeof (ff_audio_decoder_t));
+  this = calloc(1, sizeof (ff_audio_decoder_t));
 
   this->audio_decoder.decode_data         = ff_audio_decode_data;
   this->audio_decoder.reset               = ff_audio_reset;
@@ -429,7 +486,7 @@ void *init_audio_plugin (xine_t *xine, void *data) {
 
   ff_audio_class_t *this ;
 
-  this = (ff_audio_class_t *) xine_xmalloc (sizeof (ff_audio_class_t));
+  this = calloc(1, sizeof (ff_audio_class_t));
 
   this->decoder_class.open_plugin     = ff_audio_open_plugin;
   this->decoder_class.get_identifier  = ff_audio_get_identifier;
@@ -547,5 +604,5 @@ static uint32_t supported_audio_types[] = {
 
 decoder_info_t dec_info_ffmpeg_audio = {
   supported_audio_types,   /* supported types */
-  6                        /* priority        */
+  7                        /* priority        */
 };
