@@ -15,13 +15,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * FILM (CPK) File Demuxer by Mike Melanson (melanson@pcisys.net)
  * For more information on the FILM file format, visit:
  *   http://www.pcisys.net/~melanson/codecs/
- *
- * $Id: demux_film.c,v 1.77.2.1 2005/02/09 19:39:05 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -152,8 +150,8 @@ static int open_film_file(demux_film_t *film) {
   film->input->seek(film->input, 16, SEEK_SET);
 
   /* header size = header size - 16-byte FILM signature */
-  film_header_size = BE_32(&scratch[4]) - 16;
-  film_header = xine_xmalloc(film_header_size);
+  film_header_size = _X_BE_32(&scratch[4]) - 16;
+  film_header = malloc(film_header_size);
   if (!film_header)
     return 0;
   strncpy(film->version, &scratch[8], 4);
@@ -180,8 +178,8 @@ static int open_film_file(demux_film_t *film) {
   /* traverse the FILM header */
   i = 0;
   while (i < film_header_size) {
-    chunk_type = BE_32(&film_header[i]);
-    chunk_size = BE_32(&film_header[i + 4]);
+    chunk_type = _X_BE_32(&film_header[i]);
+    chunk_size = _X_BE_32(&film_header[i + 4]);
 
     /* sanity check the chunk size */
     if (i + chunk_size > film_header_size) {
@@ -197,8 +195,8 @@ static int open_film_file(demux_film_t *film) {
       llprintf(DEBUG_FILM_LOAD, "parsing FDSC chunk\n");
 
       /* always fetch the video information */
-      film->bih.biWidth = BE_32(&film_header[i + 16]);
-      film->bih.biHeight = BE_32(&film_header[i + 12]);
+      film->bih.biWidth = _X_BE_32(&film_header[i + 16]);
+      film->bih.biHeight = _X_BE_32(&film_header[i + 12]);
       film->video_codec = *(uint32_t *)&film_header[i + 8];
       film->video_type = _x_fourcc_to_buf_video(*(uint32_t *)&film_header[i + 8]);
 
@@ -209,7 +207,7 @@ static int open_film_file(demux_film_t *film) {
       if (chunk_size == 32) {
         film->audio_channels = film_header[21];
         film->audio_bits = film_header[22];
-        film->sample_rate = BE_16(&film_header[24]);
+        film->sample_rate = _X_BE_16(&film_header[24]);
       } else {
         /* If the FDSC chunk is not 32 bytes long, this is an early FILM
          * file. Make a few assumptions about the audio parms based on the
@@ -255,21 +253,23 @@ static int open_film_file(demux_film_t *film) {
       /* load the sample table */
       if (film->sample_table)
         free(film->sample_table);
-      film->frequency = BE_32(&film_header[i + 8]);
-      film->sample_count = BE_32(&film_header[i + 12]);
+      film->frequency = _X_BE_32(&film_header[i + 8]);
+      film->sample_count = _X_BE_32(&film_header[i + 12]);
       film->sample_table =
-        xine_xmalloc(film->sample_count * sizeof(film_sample_t));
+        calloc(film->sample_count, sizeof(film_sample_t));
+      if (!film->sample_table)
+        goto film_abort;
       for (j = 0; j < film->sample_count; j++) {
 
         film->sample_table[j].sample_offset =
-          BE_32(&film_header[(i + 16) + j * 16 + 0])
+          _X_BE_32(&film_header[(i + 16) + j * 16 + 0])
           + film_header_size + 16;
         film->sample_table[j].sample_size =
-          BE_32(&film_header[(i + 16) + j * 16 + 4]);
+          _X_BE_32(&film_header[(i + 16) + j * 16 + 4]);
         pts =
-          BE_32(&film_header[(i + 16) + j * 16 + 8]);
+          _X_BE_32(&film_header[(i + 16) + j * 16 + 8]);
         film->sample_table[j].duration =
-          BE_32(&film_header[(i + 16) + j * 16 + 12]);
+          _X_BE_32(&film_header[(i + 16) + j * 16 + 12]);
 
         if (pts == 0xFFFFFFFF) {
 
@@ -311,7 +311,7 @@ static int open_film_file(demux_film_t *film) {
         if (film->sample_table[j].pts > largest_pts)
           largest_pts = film->sample_table[j].pts;
 
-        llprintf(DEBUG_FILM_LOAD, "sample %4d @ %8" PRIxMAX ", %8X bytes, %s, pts %lld, duration %lld%s\n",
+        llprintf(DEBUG_FILM_LOAD, "sample %4d @ %8" PRIxMAX ", %8X bytes, %s, pts %" PRId64 ", duration %" PRId64 "%s\n",
           j,
           (intmax_t)film->sample_table[j].sample_offset,
           film->sample_table[j].sample_size,
@@ -331,15 +331,16 @@ static int open_film_file(demux_film_t *film) {
       /* allocate enough space in the interleave preload buffer for the
        * first chunk (which will be more than enough for successive chunks) */
       if (film->audio_type) {
-        if (film->interleave_buffer)
-          free(film->interleave_buffer);
-        film->interleave_buffer =
-          xine_xmalloc(film->sample_table[0].sample_size);
+	free(film->interleave_buffer);
+        film->interleave_buffer = calloc(1, film->sample_table[0].sample_size);
+        if (!film->interleave_buffer)
+          goto film_abort;
       }
       break;
 
     default:
       xine_log(film->stream->xine, XINE_LOG_MSG, _("unrecognized FILM chunk\n"));
+    film_abort:
       free (film->interleave_buffer);
       free (film->sample_table);
       free (film_header);
@@ -473,7 +474,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-      llprintf(DEBUG_FILM_DEMUX, "sending video buf with %d bytes, %lld pts, %d duration\n",
+      llprintf(DEBUG_FILM_DEMUX, "sending video buf with %" PRId32 " bytes, %" PRId64 " pts, %" PRId32 " duration\n",
         buf->size, buf->pts, buf->decoder_info[0]);
       this->video_fifo->put(this->video_fifo, buf);
     }
@@ -517,7 +518,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-      llprintf(DEBUG_FILM_DEMUX, "sending video buf with %d bytes, %lld pts, %d duration\n",
+      llprintf(DEBUG_FILM_DEMUX, "sending video buf with %" PRId32 " bytes, %" PRId64 " pts, %" PRId32 " duration\n",
         buf->size, buf->pts, buf->decoder_info[0]);
       this->video_fifo->put(this->video_fifo, buf);
     }
@@ -577,7 +578,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-      llprintf(DEBUG_FILM_DEMUX, "sending mono audio buf with %d bytes, %lld pts, %d duration\n",
+      llprintf(DEBUG_FILM_DEMUX, "sending mono audio buf with %" PRId32 " bytes, %" PRId64 " pts, %" PRId32 " duration\n",
         buf->size, buf->pts, buf->decoder_info[0]);
       this->audio_fifo->put(this->audio_fifo, buf);
 
@@ -635,12 +636,12 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
         interleave_index += buf->size / 2;
       } else {
         for (j = 0, k = interleave_index; j < buf->size; j += 2, k += 1) {
-          buf->content[j] = this->interleave_buffer[k];
+          buf->content[j] = this->interleave_buffer[k] += 0x80;
         }
         for (j = 1, 
              k = interleave_index + this->sample_table[i].sample_size / 2; 
              j < buf->size; j += 2, k += 1) {
-          buf->content[j] = this->interleave_buffer[k];
+          buf->content[j] = this->interleave_buffer[k] += 0x80;
         }
         interleave_index += buf->size / 2;
       }
@@ -648,7 +649,7 @@ static int demux_film_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-      llprintf(DEBUG_FILM_DEMUX, "sending stereo audio buf with %d bytes, %lld pts, %d duration\n",
+      llprintf(DEBUG_FILM_DEMUX, "sending stereo audio buf with %" PRId32 " bytes, %" PRId64 " pts, %" PRId32 " duration\n",
         buf->size, buf->pts, buf->decoder_info[0]);
       this->audio_fifo->put(this->audio_fifo, buf);
     }
@@ -855,7 +856,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_film_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_film_t));
+  this         = calloc(1, sizeof(demux_film_t));
   this->stream = stream;
   this->input  = input;
 
@@ -874,7 +875,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
   case METHOD_BY_EXTENSION: {
-    char *extensions, *mrl;
+    const char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
     extensions = class_gen->get_extensions (class_gen);
@@ -904,19 +905,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "FILM (CPK) demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "FILM (CPK)";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "cpk cak film";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
@@ -929,7 +930,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_film_init_plugin (xine_t *xine, void *data) {
   demux_film_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_film_class_t));
+  this = calloc(1, sizeof(demux_film_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
