@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_matroska.c,v 1.46 2006/06/02 22:18:56 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * demultiplexer for matroska streams
  *
@@ -25,7 +23,6 @@
  *   more decoders init
  *   metadata
  *   non seekable input plugins support
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -61,20 +58,6 @@
 
 #define WRAP_THRESHOLD        90000
 
-#if !defined(MIN)
-#define MIN(a, b)	((a)<(b)?(a):(b))
-#endif
-#if !defined(MAX)
-#define MAX(a, b)	((a)>(b)?(a):(b))
-#endif
-
-/* FOURCC will be manipulated using machine endian */
-#ifdef WORDS_BIGENDIAN
-#define meFOURCC BE_FOURCC
-#else
-#define meFOURCC LE_FOURCC
-#endif
-
 typedef struct {
   int                  track_num;
   off_t               *pos;
@@ -362,6 +345,140 @@ static int parse_audio_track (demux_matroska_t *this, matroska_audio_track_t *at
 }
 
 
+static int parse_content_compression (demux_matroska_t *this, matroska_track_t *track) {
+  ebml_parser_t *ebml = this->ebml;
+  int next_level = 6;
+
+  while (next_level == 6) {
+    ebml_elem_t elem;
+    uint64_t    val;
+
+    if (!ebml_read_elem_head(ebml, &elem))
+      return 0;
+
+    switch (elem.id) {
+      case MATROSKA_ID_CE_COMPALGO:
+        lprintf("ContentCompAlgo\n");
+        if (!ebml_read_uint(ebml, &elem, &val))
+          return 0;
+        switch (val)
+        {
+          case MATROSKA_COMPRESS_ZLIB:
+          case MATROSKA_COMPRESS_BZLIB:
+          case MATROSKA_COMPRESS_LZO1X:
+          case MATROSKA_COMPRESS_HEADER_STRIP:
+            track->compress_algo = val;
+            break;
+          default:
+            track->compress_algo = MATROSKA_COMPRESS_UNKNOWN;
+            break;
+        }
+        break;
+      case MATROSKA_ID_CE_COMPSETTINGS:
+        lprintf("ContentCompSettings (UNSUPPORTED)\n");
+        if (!ebml_skip(ebml, &elem))
+          return 0;
+        break;
+      default:
+        lprintf("Unhandled ID: 0x%x\n", elem.id);
+        if (!ebml_skip(ebml, &elem))
+          return 0;
+    }
+    next_level = ebml_get_next_level(ebml, &elem);
+  }
+  return 1;
+}
+
+
+static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *track) {
+  ebml_parser_t *ebml = this->ebml;
+  int next_level = 5;
+
+  while (next_level == 5) {
+    ebml_elem_t elem;
+    uint64_t    val;
+
+    if (!ebml_read_elem_head(ebml, &elem))
+      return 0;
+
+    switch (elem.id) {
+      case MATROSKA_ID_CE_ORDER:
+        lprintf("ContentEncodingOrder\n");
+        if (!ebml_read_uint(ebml, &elem, &val))
+          return 0;
+        if (val != 0) {  // multiple content encoding isn't supported
+          lprintf("   warning: a non-zero encoding order is UNSUPPORTED\n");
+          return 0;
+        }
+        break;
+      case MATROSKA_ID_CE_SCOPE:
+        lprintf("ContentEncodingScope\n");
+        if (!ebml_read_uint(ebml, &elem, &val))
+          return 0;
+        if (val != 1) {  // 1 (all frame contents) is the only supported option
+          lprintf("   warning: UNSUPPORTED encoding scope (%" PRId64 ")\n", val);
+          return 0;
+        }
+        break;
+      case MATROSKA_ID_CE_TYPE:
+        lprintf("ContentEncodingType\n");
+        if (!ebml_read_uint(ebml, &elem, &val))
+          return 0;
+        if (val != 0)  // only compression (0) is supported
+          return 0;
+        break;
+      case MATROSKA_ID_CE_COMPRESSION:
+        lprintf("ContentCompression\n");
+        if (!ebml_read_master (ebml, &elem))
+          return 0;
+        if ((elem.len > 0) && !parse_content_compression(this, track))
+          return 0;
+        break;
+      case MATROSKA_ID_CE_ENCRYPTION:
+        lprintf("ContentEncryption (UNSUPPORTED)\n");
+        if (!ebml_skip(ebml, &elem))
+          return 0;
+        break;
+      default:
+        lprintf("Unhandled ID: 0x%x\n", elem.id);
+        if (!ebml_skip(ebml, &elem))
+          return 0;
+    }
+    next_level = ebml_get_next_level(ebml, &elem);
+  }
+  return 1;
+}
+
+
+static int parse_content_encodings (demux_matroska_t *this, matroska_track_t *track) {
+  ebml_parser_t *ebml = this->ebml;
+  int next_level = 4;
+
+  while (next_level == 4) {
+    ebml_elem_t elem;
+
+    if (!ebml_read_elem_head(ebml, &elem))
+      return 0;
+
+    switch (elem.id) {
+      case MATROSKA_ID_CONTENTENCODING:
+        lprintf("ContentEncoding\n");
+        if (!ebml_read_master (ebml, &elem))
+          return 0;
+        if ((elem.len > 0) && !parse_content_encoding(this, track))
+          return 0;
+        break;
+      default:
+        lprintf("Unhandled ID: 0x%x\n", elem.id);
+        if (!ebml_skip(ebml, &elem))
+          return 0;
+    }
+    next_level = ebml_get_next_level(ebml, &elem);
+  }
+  return 1;
+}
+
+
 static void init_codec_video(demux_matroska_t *this, matroska_track_t *track) {
   buf_element_t *buf;
   
@@ -483,8 +600,7 @@ static void init_codec_real(demux_matroska_t *this, matroska_track_t * track) {
   track->fifo->put (track->fifo, buf);
 }
 
-
-static void init_codec_vorbis(demux_matroska_t *this, matroska_track_t *track) {
+static void init_codec_xiph(demux_matroska_t *this, matroska_track_t *track) {
   buf_element_t *buf;
   uint8_t nb_lace;
   int frame[3];
@@ -512,9 +628,7 @@ static void init_codec_vorbis(demux_matroska_t *this, matroska_track_t *track) {
     }
     buf->size = frame[i];
     
-    buf->decoder_flags = BUF_FLAG_HEADER;
-    if (i == 2)
-      buf->decoder_flags |= BUF_FLAG_FRAME_END;
+    buf->decoder_flags = BUF_FLAG_HEADER | BUF_FLAG_FRAME_START | BUF_FLAG_FRAME_END;
     buf->type          = track->buf_type;
     buf->pts           = 0;
 
@@ -656,7 +770,7 @@ static int vobsub_parse_custom_colors(matroska_track_t *t, const char *start) {
     use_custom_colors = 1;
   else if (!strncasecmp(start, "OFF", 3) || (*start == '0'))
     use_custom_colors = 0;
-  lprintf("VobSub custom colors: %s\n", use_custom_colors ? "ON" : "OFF");
+  lprintf("VobSub custom colours: %s\n", use_custom_colors ? "ON" : "OFF");
   if ((start = strstr(start, "colors:")) != NULL) {
     start += 7;
     while (isspace(*start))
@@ -670,7 +784,7 @@ static int vobsub_parse_custom_colors(matroska_track_t *t, const char *start) {
     }
     if (i == 4) {
       t->sub_track->custom_colors = 4;
-      lprintf("VobSub colors: %06x,%06x,%06x,%06x\n", t->sub_track->colors[0],
+      lprintf("VobSub colours: %06x,%06x,%06x,%06x\n", t->sub_track->colors[0],
               t->sub_track->colors[1], t->sub_track->colors[2],
               t->sub_track->colors[3]);
     }
@@ -730,7 +844,7 @@ static void init_codec_vobsub(demux_matroska_t *this,
         things_found |= vobsub_parse_size(track, start);
       else if (!strncasecmp(start, "palette:", 8))
         things_found |= vobsub_parse_palette(track, start);
-      else if (!strncasecmp(start, "custom colors:", 14))
+      else if (!strncasecmp(start, "custom colours:", 14))
         things_found |= vobsub_parse_custom_colors(track, start);
       else if (!strncasecmp(start, "forced subs:", 12))
         things_found |= vobsub_parse_forced_subs(track, start);
@@ -937,45 +1051,67 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
                            int input_normpos, int input_time) {
   demux_matroska_t *this = (demux_matroska_t *) this_gen;
   buf_element_t *buf;
-  z_stream zstream;
-  uint8_t *dest;
-  int old_data_len, result;
 
-  old_data_len = data_len;
-  zstream.zalloc = (alloc_func) 0;
-  zstream.zfree = (free_func) 0;
-  zstream.opaque = (voidpf) 0;
-  if (inflateInit (&zstream) != Z_OK) {
-    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            "demux_matroska: VobSub: zlib inflateInit failed.\n");
-    return;
-  }
-  zstream.next_in = (Bytef *)data;
-  zstream.avail_in = data_len;
+  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB ||
+      track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+    z_stream zstream;
+    uint8_t *dest;
+    int old_data_len, result;
 
-  dest = (uint8_t *)malloc(data_len);
-  zstream.avail_out = data_len;
-  do {
-    data_len += 4000;
-    dest = (uint8_t *)realloc(dest, data_len);
-    zstream.next_out = (Bytef *)(dest + zstream.total_out);
-    result = inflate (&zstream, Z_NO_FLUSH);
-    if ((result != Z_OK) && (result != Z_STREAM_END)) {
+    old_data_len = data_len;
+    zstream.zalloc = (alloc_func) 0;
+    zstream.zfree = (free_func) 0;
+    zstream.opaque = (voidpf) 0;
+    if (inflateInit (&zstream) != Z_OK) {
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-              "demux_matroska: VobSub: zlib decompression failed.\n");
-      free(dest);
-      inflateEnd(&zstream);
+              "demux_matroska: VobSub: zlib inflateInit failed.\n");
       return;
     }
-    zstream.avail_out += 4000;
-  } while ((zstream.avail_out == 4000) &&
-           (zstream.avail_in != 0) && (result != Z_STREAM_END));
+    zstream.next_in = (Bytef *)data;
+    zstream.avail_in = data_len;
 
-  data_len = zstream.total_out;
-  inflateEnd(&zstream);
-
-  lprintf("VobSub: decompression for track %d from %d to %d\n",
-          (int)track->track_num, old_data_len, data_len);
+    dest = (uint8_t *)malloc(data_len);
+    zstream.avail_out = data_len;
+    do {
+      data_len += 4000;
+      dest = (uint8_t *)realloc(dest, data_len);
+      zstream.next_out = (Bytef *)(dest + zstream.total_out);
+      result = inflate (&zstream, Z_NO_FLUSH);
+      if ((result != Z_OK) && (result != Z_STREAM_END)) {
+        xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                "demux_matroska: VobSub: zlib decompression failed for track %d (result = %d).\n",
+                (int)track->track_num, result);
+        free(dest);
+        inflateEnd(&zstream);
+       
+        if (result == Z_DATA_ERROR && track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+          track->compress_algo = MATROSKA_COMPRESS_NONE;
+          data_len = old_data_len;
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  "demux_matroska: VobSub: falling back to uncompressed mode.\n");
+          break;
+        }
+        return;
+      }
+      zstream.avail_out += 4000;
+    } while ((zstream.avail_out == 4000) &&
+            (zstream.avail_in != 0) && (result != Z_STREAM_END));
+ 
+    if (track->compress_algo != MATROSKA_COMPRESS_NONE) {
+      data_len = zstream.total_out;
+      inflateEnd(&zstream);
+ 
+      data = dest;
+      track->compress_algo = MATROSKA_COMPRESS_ZLIB;
+      lprintf("VobSub: decompression for track %d from %d to %d\n",
+              (int)track->track_num, old_data_len, data_len);
+    }
+  }
+  else
+  {
+    lprintf("VobSub: track %d isn't compressed (%d bytes)\n",
+            (int)track->track_num, data_len);
+  }
 
   buf = track->fifo->buffer_pool_alloc(track->fifo);
 
@@ -986,8 +1122,8 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
     buf->decoder_info[2] = SPU_DVD_SUBTYPE_VOBSUB_PACKAGE;
     buf->type = track->buf_type;
 
-    xine_fast_memcpy(buf->content, dest, data_len);
-    
+    xine_fast_memcpy(buf->content, data, data_len);
+
     buf->extra_info->input_normpos = input_normpos;
     buf->extra_info->input_time    = input_time;
 
@@ -1000,7 +1136,8 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
     buf->free_buffer(buf);
   }
 
-  free(dest);
+  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB)
+    free(data);
 }
 
 static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
@@ -1033,11 +1170,10 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       break;
       
       case MATROSKA_ID_TR_CODECID: {
-        char *codec_id = malloc (elem.len + 1);
+        char *codec_id = ebml_alloc_read_ascii (ebml, &elem);
         lprintf("CodecID\n");
-        if (!ebml_read_ascii(ebml, &elem, codec_id))
+        if (!codec_id)
           return 0;
-        codec_id[elem.len] = '\0';
         track->codec_id = codec_id;
       }
       break;
@@ -1045,19 +1181,20 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       case MATROSKA_ID_TR_CODECPRIVATE: {
         char *codec_private = malloc (elem.len);
         lprintf("CodecPrivate\n");
-        if (!ebml_read_binary(ebml, &elem, codec_private))
+        if (!ebml_read_binary(ebml, &elem, codec_private)) {
+	  free(codec_private);
           return 0;
+	}
         track->codec_private = codec_private;
         track->codec_private_len = elem.len;
       }
       break;
         
       case MATROSKA_ID_TR_LANGUAGE: {
-        char *language = malloc (elem.len + 1);
+        char *language = ebml_alloc_read_ascii (ebml, &elem);
         lprintf("Language\n");
-        if (!ebml_read_ascii(ebml, &elem, language))
+        if (!language)
           return 0;
-        language[elem.len] = '\0';
         track->language = language;
       }
       break;
@@ -1066,7 +1203,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
         lprintf("Video\n");
         if (track->video_track)
           return 1;
-        track->video_track = (matroska_video_track_t *)xine_xmalloc(sizeof(matroska_video_track_t));
+        track->video_track = (matroska_video_track_t *)calloc(1, sizeof(matroska_video_track_t));
         if (!ebml_read_master (ebml, &elem))
           return 0;
         if ((elem.len > 0) && !parse_video_track(this, track->video_track))
@@ -1077,7 +1214,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
         lprintf("Audio\n");
         if (track->audio_track)
           return 1;
-        track->audio_track = (matroska_audio_track_t *)xine_xmalloc(sizeof(matroska_audio_track_t));
+        track->audio_track = (matroska_audio_track_t *)calloc(1, sizeof(matroska_audio_track_t));
         if (!ebml_read_master (ebml, &elem))
           return 0;
         if ((elem.len > 0) && !parse_audio_track(this, track->audio_track))
@@ -1100,7 +1237,16 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
         if (!ebml_read_uint(ebml, &elem, &val))
           return 0;
         track->default_duration = val;
-        lprintf("Default Duration: %lld\n", track->default_duration);
+        lprintf("Default Duration: %"PRIu64"\n", track->default_duration);
+      }
+      break;
+
+      case MATROSKA_ID_CONTENTENCODINGS: {
+        lprintf("ContentEncodings\n");
+        if (!ebml_read_master (ebml, &elem))
+          return 0;
+        if ((elem.len > 0) && !parse_content_encodings(this, track))
+          return 0;
       }
       break;
 
@@ -1154,7 +1300,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       /* create a bitmap info header struct for MPEG 4 */
       bih = malloc(sizeof(xine_bmiheader) + track->codec_private_len);
       bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
-      bih->biCompression = meFOURCC('M', 'P', '4', 'S');
+      bih->biCompression = ME_FOURCC('M', 'P', '4', 'S');
       bih->biWidth = track->video_track->pixel_width;
       bih->biHeight = track->video_track->pixel_height;
       _x_bmiheader_le2me(bih);
@@ -1175,7 +1321,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       /* create a bitmap info header struct for h264 */
       bih = malloc(sizeof(xine_bmiheader) + track->codec_private_len);
       bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
-      bih->biCompression = meFOURCC('a', 'v', 'c', '1');
+      bih->biCompression = ME_FOURCC('a', 'v', 'c', '1');
       bih->biWidth = track->video_track->pixel_width;
       bih->biHeight = track->video_track->pixel_height;
       _x_bmiheader_le2me(bih);
@@ -1214,6 +1360,10 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_real;
 
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MJPEG)) {
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_THEORA)) {
+      lprintf("MATROSKA_CODEC_ID_V_THEORA\n");
+      track->buf_type = BUF_VIDEO_THEORA_RAW;
+      init_codec = init_codec_xiph;
     } else if ((!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_MPEG1_L1)) ||
                (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_MPEG1_L2)) ||
                (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_MPEG1_L3))) {
@@ -1230,13 +1380,25 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_audio;
       
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_DTS)) {
+      lprintf("MATROSKA_CODEC_ID_A_DTS\n");
+      track->buf_type = BUF_AUDIO_DTS;
+      init_codec = init_codec_audio;
+
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_VORBIS)) {
 
       lprintf("MATROSKA_CODEC_ID_A_VORBIS\n");
       track->buf_type = BUF_AUDIO_VORBIS;
-      init_codec = init_codec_vorbis;
+      init_codec = init_codec_xiph;
 
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_ACM)) {
+      xine_waveformatex *wfh;
+      lprintf("MATROSKA_CODEC_ID_A_ACM\n");
+
+      wfh = (xine_waveformatex*)track->codec_private;
+      _x_waveformatex_le2me(wfh);
+
+      track->buf_type = _x_formattag_to_buf_audio(wfh->wFormatTag);
+      init_codec = init_codec_audio;
     } else if (!strncmp(track->codec_id, MATROSKA_CODEC_ID_A_AAC,
                         sizeof(MATROSKA_CODEC_ID_A_AAC) - 1)) {
       lprintf("MATROSKA_CODEC_ID_A_AAC\n");
@@ -1281,6 +1443,14 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       track->buf_type = BUF_SPU_DVD;
       track->handle_content = handle_vobsub;
       init_codec = init_codec_vobsub;
+      
+      /* Enable autodetection of the zlib compression, unless it was
+       * explicitely set. Most vobsubs are compressed with zlib but
+       * are not declared as such.
+       */
+      if (track->compress_algo == MATROSKA_COMPRESS_NONE) {
+        track->compress_algo = MATROSKA_COMPRESS_UNKNOWN;
+      }
     } else {
       lprintf("unknown codec\n");
     }
@@ -1331,12 +1501,17 @@ static int parse_tracks(demux_matroska_t *this) {
 
     switch (elem.id) {
       case MATROSKA_ID_TR_ENTRY: {
+        matroska_track_t *track;
+
         /* alloc and initialize a track with 0 */
-        this->tracks[this->num_tracks] = xine_xmalloc(sizeof(matroska_track_t));
+        track = calloc(1, sizeof(matroska_track_t));
+        track->compress_algo = MATROSKA_COMPRESS_NONE;
+        this->tracks[this->num_tracks] = track;
+
         lprintf("TrackEntry\n");
         if (!ebml_read_master (ebml, &elem))
           return 0;
-        if ((elem.len > 0) && !parse_track_entry(this, this->tracks[this->num_tracks]))
+        if ((elem.len > 0) && !parse_track_entry(this, track))
           return 0;
         this->num_tracks++;
       }
@@ -1642,11 +1817,20 @@ static int read_block_data (demux_matroska_t *this, int len) {
   return 1;
 }
 
+static int parse_int16(uint8_t *data) {
+  int value = (int)_X_BE_16(data);
+  if (value & 1<<15)
+  {
+    value -= 1<<16;
+  }
+  return value;
+}
+
 static int parse_block (demux_matroska_t *this, uint64_t block_size,
                         uint64_t cluster_timecode, uint64_t block_duration,
                         int normpos, int is_key) {
   matroska_track_t *track;
-  int64_t           track_num;
+  uint64_t          track_num;
   uint8_t          *data;
   uint8_t           flags;
   int               gap, lacing, num_len;
@@ -1658,21 +1842,23 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
   if (!(num_len = parse_ebml_uint(this, data, &track_num)))
     return 0;
   data += num_len;
-    
-  timecode_diff = (int)BE_16(data);
+
+  /* timecode_diff is signed */
+  timecode_diff = parse_int16(data);
   data += 2;
 
   flags = *data;
   data += 1;
   
-  lprintf("track_num: %" PRId64 ", timecode_diff: %d, flags: 0x%x\n", track_num, timecode_diff, flags);
+  lprintf("track_num: %" PRIu64 ", timecode_diff: %d, flags: 0x%x\n", track_num, timecode_diff, flags);
 
   gap = flags & 1;
   lacing = (flags >> 1) & 0x3;
+/*fprintf(stderr, "lacing: %x\n", lacing);*/
 
   if (!find_track_by_id(this, (int)track_num, &track)) {
      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-             "demux_matroska: invalid track id: %" PRId64 "\n", track_num);
+             "demux_matroska: invalid track id: %" PRIu64 "\n", track_num);
      return 0;
   }
 
@@ -1781,24 +1967,51 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
       break;
 
       case MATROSKA_EBML_LACING: {
-        int64_t tmp;
+        uint64_t first_frame_size;
 
         lprintf("ebml lacing\n");
 
         /* size of each frame */
-        if (!(num_len = parse_ebml_uint(this, data, &tmp)))
+        if (!(num_len = parse_ebml_uint(this, data, &first_frame_size)))
           return 0;
+        if (num_len > block_size_left) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  "demux_matroska: block too small\n");
+          return 0;
+        }
+        if (first_frame_size > INT_MAX) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                  "demux_matroska: invalid first frame size (%" PRId64 ")\n",
+                  first_frame_size);
+          return 0;
+        }
         data += num_len; block_size_left -= num_len;
-        frame[0] = (int) tmp;
+        frame[0] = (int) first_frame_size;
         lprintf("first frame len: %d\n", frame[0]);
         block_size_left -= frame[0];
 
         for (i = 1; i < lace_num; i++) {
-          if (!(num_len = parse_ebml_sint(this, data, &tmp)))
+          int64_t frame_size_diff;
+          int64_t frame_size;
+
+          if (!(num_len = parse_ebml_sint(this, data, &frame_size_diff)))
             return 0;
 
+          if (num_len > block_size_left) {
+            xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                    "demux_matroska: block too small\n");
+            return 0;
+          }
           data += num_len; block_size_left -= num_len;
-          frame[i] = frame[i-1] + tmp;
+
+          frame_size = frame[i-1] + frame_size_diff;
+          if (frame_size > INT_MAX || frame_size < 0) {
+            xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                    "demux_matroska: invalid frame size (%" PRId64 ")\n",
+                    frame_size);
+            return 0;
+          }
+          frame[i] = frame_size;
           block_size_left -= frame[i];
         }
 
@@ -2615,13 +2828,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   break;
 
   case METHOD_BY_EXTENSION: {
-    char *mrl = input->get_mrl(input);
-    char *extensions;
+    const char *const mrl = input->get_mrl(input);
+    const char *const extensions = class_gen->get_extensions (class_gen);;
 
     lprintf ("stage by extension %s\n", mrl);
 
-    extensions = class_gen->get_extensions (class_gen);
-
     if (!_x_demux_check_extension (mrl, extensions))
       return NULL;
 
@@ -2635,7 +2846,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     return NULL;
   }
 
-  this = xine_xmalloc (sizeof (demux_matroska_t));
+  this = calloc(1, sizeof(demux_matroska_t));
 
   this->demux_plugin.send_headers      = demux_matroska_send_headers;
   this->demux_plugin.send_chunk        = demux_matroska_send_chunk;
@@ -2669,8 +2880,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 
 error:
-  if (ebml)
-    dispose_ebml_parser(ebml);
+  dispose_ebml_parser(ebml);
   free(this);
   return NULL;
 }
@@ -2680,23 +2890,24 @@ error:
  * demux matroska class
  */
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "matroska demux plugin";
 }
 
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "matroska";
 }
 
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "mkv";
 }
 
 
-static char *get_mimetypes (demux_class_t *this_gen) {
-  return "video/mkv: mkv: matroska;";
+static const char *get_mimetypes (demux_class_t *this_gen) {
+  return "video/mkv: mkv: matroska;"
+        "video/x-matroska: mkv: matroska;";
 }
 
 
@@ -2711,7 +2922,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   demux_matroska_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_matroska_class_t));
+  this         = calloc(1, sizeof(demux_matroska_class_t));
   this->xine   = xine;
 
   this->demux_class.open_plugin     = open_plugin;
@@ -2731,7 +2942,7 @@ static const demuxer_info_t demux_info_matroska = {
   10                       /* priority */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "matroska", XINE_VERSION_CODE, &demux_info_matroska, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
