@@ -260,7 +260,7 @@ static demux_plugin_t* open_plugin(demux_class_t *class_gen, xine_stream_t *stre
 
   demux_mng_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_mng_t));
+  this         = calloc(1, sizeof(demux_mng_t));
   this->stream = stream;
   this->input  = input;
 
@@ -360,7 +360,7 @@ static void class_dispose(demux_class_t *this){
 static void *init_plugin(xine_t *xine, void *data){
   demux_mng_class_t     *this;
 
-  this  = xine_xmalloc (sizeof (demux_mng_class_t));
+  this  = calloc(1, sizeof(demux_mng_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
