@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
@@ -29,8 +29,6 @@
  *
  * For more information regarding the NSF format, visit:
  *   http://www.tripoint.org/kevtris/nes/nsfspec.txt
- *
- * $Id: demux_nsf.c,v 1.23 2007/01/19 00:26:40 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -71,12 +69,14 @@ typedef struct {
   input_plugin_t      *input;
   int                  status;
 
-  char                *title;
-  char                *artist;
-  char                *copyright;
   int                  total_songs;
   int                  current_song;
   int                  new_song;  /* indicates song change */
+
+  char                *title;
+  char                *artist;
+  char                *copyright;
+
   off_t                filesize;
 
   int64_t              current_pts;
@@ -106,9 +106,9 @@ static int open_nsf_file(demux_nsf_t *this) {
 
   this->total_songs = header[6];
   this->current_song = header[7];
-  this->title = strdup(&header[0x0E]);
-  this->artist = strdup(&header[0x2E]);
-  this->copyright = strdup(&header[0x4E]);
+  this->title = strndup((char*)&header[0x0E], 0x20);
+  this->artist = strndup((char*)&header[0x2E], 0x20);
+  this->copyright = strndup((char*)&header[0x4E], 0x20);
 
   this->filesize = this->input->get_length(this->input);
 
@@ -303,7 +303,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     return NULL;
   }
 
-  this         = xine_xmalloc (sizeof (demux_nsf_t));
+  this         = calloc(1, sizeof(demux_nsf_t));
   this->stream = stream;
   this->input  = input;
 
@@ -377,7 +377,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_nsf_init_plugin (xine_t *xine, void *data) {
   demux_nsf_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_nsf_class_t));
+  this = calloc(1, sizeof(demux_nsf_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
