@@ -15,15 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
  * Nullsoft Video (NSV) file demuxer by Mike Melanson (melanson@pcisys.net)
  * For more information regarding the NSV file format, visit:
  *   http://www.pcisys.net/~melanson/codecs/
- *
- * $Id: demux_nsv.c,v 1.20.2.1 2005/04/20 17:21:05 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -172,7 +170,7 @@ static off_t nsv_read(demux_nsv_t *this, uint8_t *buffer, off_t len) {
 	  return -1;
 	}
 	/* read packet payload len */
-	this->ultravox_size = BE_16(&ultravox_buf[5]);
+	this->ultravox_size = _X_BE_16(&ultravox_buf[5]);
 	this->ultravox_pos = 0;
 	lprintf("ultravox_size: %d\n", this->ultravox_size);
       } else {
@@ -287,11 +285,11 @@ static int open_nsv_file(demux_nsv_t *this) {
 	  return 0;
 
 	lprintf("found NSVf chunk\n");
-	/*	this->data_size = LE_32(&preview[8]);*/
+	/*	this->data_size = _X_LE_32(&preview[8]);*/
 	/*lprintf("data_size: %lld\n", this->data_size);*/
 	
 	/* skip the rest of the data */
-	chunk_size = LE_32(&preview[4]);
+	chunk_size = _X_LE_32(&preview[4]);
 	nsv_seek(this, chunk_size - 28, SEEK_CUR);
       }
       break;
@@ -303,21 +301,21 @@ static int open_nsv_file(demux_nsv_t *this) {
       if (nsv_read(this, &preview[4], 15) != 15)
 	return 0;
       
-      this->video_fourcc = ME_32(&preview[4]);
-      if (BE_32(&preview[4]) == NONE_TAG)
+      this->video_fourcc = _X_ME_32(&preview[4]);
+      if (_X_BE_32(&preview[4]) == NONE_TAG)
 	this->video_type = 0;
       else
 	this->video_type = _x_fourcc_to_buf_video(this->video_fourcc);
       
-      this->audio_fourcc = ME_32(&preview[8]);
-      if (BE_32(&preview[8]) == NONE_TAG)
+      this->audio_fourcc = _X_ME_32(&preview[8]);
+      if (_X_BE_32(&preview[8]) == NONE_TAG)
 	this->audio_type = 0;
       else
 	this->audio_type = _x_formattag_to_buf_audio(this->audio_fourcc);
       
       this->bih.biSize = sizeof(this->bih);
-      this->bih.biWidth = LE_16(&preview[12]);
-      this->bih.biHeight = LE_16(&preview[14]);
+      this->bih.biWidth = _X_LE_16(&preview[12]);
+      this->bih.biHeight = _X_LE_16(&preview[14]);
       this->bih.biCompression = this->video_fourcc;
       this->video_pts = 0;
       
@@ -493,10 +491,10 @@ static int demux_nsv_send_chunk(demux_plugin_t *this_gen) {
       this->status = DEMUX_FINISHED;
       return this->status;
     }
-    video_size = LE_32(&buffer[0]);
+    video_size = _X_LE_32(&buffer[0]);
     video_size >>= 4;
     video_size &= 0xFFFFF;
-    audio_size = LE_16(&buffer[3]);
+    audio_size = _X_LE_16(&buffer[3]);
     
     nsv_parse_payload(this, video_size, audio_size);
     break;
@@ -596,7 +594,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_nsv_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_nsv_t));
+  this         = calloc(1, sizeof(demux_nsv_t));
   this->stream = stream;
   this->input  = input;
 
@@ -615,7 +613,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
   case METHOD_BY_EXTENSION: {
-    char *extensions, *mrl;
+    const char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
     extensions = class_gen->get_extensions (class_gen);
@@ -645,19 +643,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "Nullsoft Video demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "Nullsoft NSV";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "nsv";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
@@ -670,7 +668,7 @@ static void class_dispose (demux_class_t *this_gen) {
 static void *demux_nsv_init_plugin (xine_t *xine, void *data) {
   demux_nsv_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_nsv_class_t));
+  this = calloc(1, sizeof(demux_nsv_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -685,11 +683,11 @@ static void *demux_nsv_init_plugin (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_nsv = {
+static const demuxer_info_t demux_info_nsv = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "nsv", XINE_VERSION_CODE, &demux_info_nsv, demux_nsv_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
