@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_ts.c,v 1.124 2007/01/19 00:26:40 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Demultiplexer for MPEG2 Transport Streams.
  *
@@ -180,14 +178,13 @@
 
 #define MAX_PES_BUF_SIZE 2048
 
+#define CORRUPT_PES_THRESHOLD 10
+
 #define NULL_PID 0x1fff
 #define INVALID_PID ((unsigned int)(-1))
 #define INVALID_PROGRAM ((unsigned int)(-1))
 #define INVALID_CC ((unsigned int)(-1))
 
-#define	MIN(a,b)   (((a)<(b))?(a):(b))
-#define	MAX(a,b)   (((a)>(b))?(a):(b))
-
 #define PROG_STREAM_MAP  0xBC
 #define PRIVATE_STREAM1  0xBD
 #define PADDING_STREAM   0xBE
@@ -220,7 +217,9 @@
       ISO_13818_PART7_AUDIO = 0x0f,     /* ISO/IEC 13818-7 Audio with ADTS transport sytax */
       ISO_14496_PART2_VIDEO = 0x10,     /* ISO/IEC 14496-2 Visual (MPEG-4) */
       ISO_14496_PART3_AUDIO = 0x11,     /* ISO/IEC 14496-3 Audio with LATM transport syntax */
-      ISO_14496_PART10_VIDEO = 0x1b     /* ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264) */
+      ISO_14496_PART10_VIDEO = 0x1b,    /* ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264) */
+      STREAM_VIDEO_MPEG = 0x80,
+      STREAM_AUDIO_AC3 = 0x81,
     } streamType;
 
 #define WRAP_THRESHOLD       270000
@@ -246,7 +245,7 @@ typedef struct {
   int64_t          pts;
   buf_element_t   *buf;
   unsigned int     counter;
-  uint8_t          descriptor_tag;
+  uint16_t         descriptor_tag; /* +0x100 for PES stream IDs (no available TS descriptor tag?) */
   int64_t          packet_count;
   int              corrupted_pes;
   uint32_t         buffered_bytes;
@@ -254,7 +253,7 @@ typedef struct {
 } demux_ts_media;
 
 /* DVBSUB */
-#define MAX_NO_SPU_LANGS 16
+#define MAX_SPU_LANGS 16
 
 typedef struct {
   spu_dvb_descriptor_t desc;
@@ -262,6 +261,14 @@ typedef struct {
   int media_index;
 } demux_ts_spu_lang;
   
+/* Audio Channels */
+#define MAX_AUDIO_TRACKS 16
+
+typedef struct {
+    int pid;
+    int media_index;
+    char lang[4];
+} demux_ts_audio_track;
 
 typedef struct {
   /*
@@ -289,6 +296,7 @@ typedef struct {
   uint8_t         *pmt[MAX_PMTS];
   uint8_t         *pmt_write_ptr[MAX_PMTS];
   uint32_t         crc32_table[256];
+  uint32_t         last_pmt_crc;
   /*
    * Stuff to do with the transport header. As well as the video
    * and audio PIDs, we keep the index of the corresponding entry
@@ -299,10 +307,10 @@ typedef struct {
   unsigned int     pid;
   unsigned int     pid_count;
   unsigned int     videoPid;
-  unsigned int     audioPid;
   unsigned int     videoMedia;
-  unsigned int     audioMedia;
-  char             audioLang[4];
+  
+  demux_ts_audio_track audio_tracks[MAX_AUDIO_TRACKS];
+  int              audio_tracks_count;
   
   int              send_end_buffers;
   int64_t          last_pts[2];
@@ -319,8 +327,8 @@ typedef struct {
   /* DVBSUB */
   unsigned int      spu_pid;
   unsigned int      spu_media;
-  demux_ts_spu_lang spu_langs[MAX_NO_SPU_LANGS];
-  int               no_spu_langs;
+  demux_ts_spu_lang spu_langs[MAX_SPU_LANGS];
+  int               spu_langs_count;
   int               current_spu_channel;
 
   /* dvb */
@@ -329,9 +337,10 @@ typedef struct {
   int32_t packet_number;
   /* NEW: var to keep track of number of last read packets */
   int32_t npkt_read;
-  int32_t read_zero;
 
   uint8_t buf[BUF_SIZE]; /* == PKT_SIZE * NPKT_PER_READ */
+  
+  int numPreview;
 
 } demux_ts_t;
 
@@ -359,8 +368,8 @@ static void demux_ts_build_crc32_table(demux_ts_t*this) {
 }
 
 static uint32_t demux_ts_compute_crc32(demux_ts_t*this, uint8_t *data, 
-				       uint32_t length, uint32_t crc32) {
-  uint32_t i;
+				       int32_t length, uint32_t crc32) {
+  int32_t i;
 
   for(i = 0; i < length; i++) {
     crc32 = (crc32 << 8) ^ this->crc32_table[(crc32 >> 24) ^ data[i]];
@@ -435,6 +444,17 @@ static void check_newpts( demux_ts_t *this, int64_t pts, int video )
   }
 }
 
+/* Send a BUF_SPU_DVB to let xine know of that channel. */
+static void demux_send_special_spu_buf( demux_ts_t *this, int spu_channel )
+{
+  buf_element_t *buf;
+
+  buf = this->video_fifo->buffer_pool_alloc( this->video_fifo );
+  buf->type = BUF_SPU_DVB|spu_channel;
+  buf->content = buf->mem;
+  buf->size = 0;
+  this->video_fifo->put( this->video_fifo, buf );
+}
 
 /*
  * demux_ts_update_spu_channel
@@ -444,7 +464,6 @@ static void check_newpts( demux_ts_t *this, int64_t pts, int video )
  */
 static void demux_ts_update_spu_channel(demux_ts_t *this)
 {
-  xine_event_t ui_event;
   buf_element_t *buf;
   
   this->current_spu_channel = this->stream->spu_channel;
@@ -457,12 +476,13 @@ static void demux_ts_update_spu_channel(demux_ts_t *this)
   buf->size = 0;
     
   if (this->current_spu_channel >= 0
-      && this->current_spu_channel < this->no_spu_langs)
+      && this->current_spu_channel < this->spu_langs_count)
     {
       demux_ts_spu_lang *lang = &this->spu_langs[this->current_spu_channel];
 
       buf->decoder_info[2] = sizeof(lang->desc);
       buf->decoder_info_ptr[2] = &(lang->desc);
+      buf->type |= this->current_spu_channel;
 
       this->spu_pid = lang->pid;
       this->spu_media = lang->media_index;
@@ -484,11 +504,6 @@ static void demux_ts_update_spu_channel(demux_ts_t *this)
     }
 
   this->video_fifo->put(this->video_fifo, buf);
-
-  /* Inform UI of SPU channel changes */
-  ui_event.type = XINE_EVENT_UI_CHANNELS_CHANGED;
-  ui_event.data_length = 0;
-  xine_event_send(this->stream, &ui_event);
 }
 
 /*
@@ -505,7 +520,7 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
                                 unsigned char *pkt, unsigned int pusi) {
   uint32_t       table_id;
   uint32_t       section_syntax_indicator;
-  uint32_t       section_length;
+  int32_t        section_length;
   uint32_t       transport_stream_id;
   uint32_t       version_number;
   uint32_t       current_next_indicator;
@@ -627,7 +642,8 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
     /* force PMT reparsing when pmt_pid changes */
     if (this->pmt_pid[program_count] != pmt_pid) {
       this->pmt_pid[program_count] = pmt_pid;
-      this->audioPid = INVALID_PID;
+      this->audio_tracks_count = 0;
+      this->last_pmt_crc = 0;
       this->videoPid = INVALID_PID;
       this->spu_pid = INVALID_PID;
     }
@@ -675,7 +691,11 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
   stream_id  = p[3];
 
   if (packet_len==0)
+  {
+    xprintf (xine, XINE_VERBOSITY_DEBUG,
+             "demux_ts: error pes length 0\n");
     return 0;
+  }
 
 #ifdef TS_LOG
   printf ("demux_ts: packet stream id: %.2x len: %d (%x)\n",
@@ -722,32 +742,32 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 
   if (stream_id == 0xbd) {
 
-    int track, spu_id;
+    int spu_id;
       
     lprintf ("audio buf = %02X %02X %02X %02X %02X %02X %02X %02X\n",
 	     p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7]);
 
-    track = p[0] & 0x0F; /* hack : ac3 track */
     /*
      * we check the descriptor tag first because some stations
      * do not include any of the ac3 header info in their audio tracks
      * these "raw" streams may begin with a byte that looks like a stream type.
+     * For audio streams, m->type already contains the stream no.
      */
-    if((m->descriptor_tag == 0x81) ||    /* ac3 - raw */ 
+    if((m->descriptor_tag == STREAM_AUDIO_AC3) ||    /* ac3 - raw */ 
        (p[0] == 0x0B && p[1] == 0x77)) { /* ac3 - syncword */
       m->content   = p;
       m->size = packet_len;
-      m->type = BUF_AUDIO_A52;
+      m->type |= BUF_AUDIO_A52;
       return 1;
 
-    } else if (m->descriptor_tag == 0x06
+    } else if (m->descriptor_tag == ISO_13818_PES_PRIVATE
 	     && p[0] == 0x20 && p[1] == 0x00) {
       /* DVBSUB */
       long payload_len = ((buf[4] << 8) | buf[5]) - header_len - 3;
 
       m->content = p;
       m->size = packet_len;
-      m->type = BUF_SPU_DVB;
+      m->type |= BUF_SPU_DVB;
       m->buf->decoder_info[2] = payload_len;
       return 1;
     } else if ((p[0] & 0xE0) == 0x20) {
@@ -761,7 +781,7 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 
       m->content   = p+4;
       m->size      = packet_len - 4;
-      m->type      = BUF_AUDIO_A52 + track;
+      m->type      |= BUF_AUDIO_A52;
       return 1;
 
     } else if ((p[0]&0xf0) == 0xa0) {
@@ -777,7 +797,7 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 
       m->content   = p+pcm_offset;
       m->size      = packet_len-pcm_offset;
-      m->type      = BUF_AUDIO_LPCM_BE + track;
+      m->type      |= BUF_AUDIO_LPCM_BE;
       return 1;
     }
 
@@ -788,6 +808,7 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
     switch (m->descriptor_tag) {
     case ISO_11172_VIDEO:
     case ISO_13818_VIDEO:
+    case STREAM_VIDEO_MPEG:
       lprintf ("demux_ts: found MPEG video type.\n");
       m->type      = BUF_VIDEO_MPEG;
       break;
@@ -808,26 +829,22 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 
   } else if ((stream_id & 0xe0) == 0xc0) {
 
-    int track;
-
-    track = stream_id & 0x1f;
-
     m->content   = p;
     m->size      = packet_len;
     switch (m->descriptor_tag) {
     case  ISO_11172_AUDIO: 
     case  ISO_13818_AUDIO:
       lprintf ("demux_ts: found MPEG audio track.\n");
-      m->type      = BUF_AUDIO_MPEG + track;
+      m->type      |= BUF_AUDIO_MPEG;
       break;
     case  ISO_13818_PART7_AUDIO:
     case  ISO_14496_PART3_AUDIO:
       lprintf ("demux_ts: found AAC audio track.\n");
-      m->type      = BUF_AUDIO_AAC + track;
+      m->type      |= BUF_AUDIO_AAC;
       break;
     default:
       lprintf ("demux_ts: unknown audio type: %d, defaulting to MPEG.\n", m->descriptor_tag);
-      m->type      = BUF_AUDIO_MPEG + track;
+      m->type      |= BUF_AUDIO_MPEG;
       break;
     }
     return 1;
@@ -882,6 +899,16 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
         m->buf->decoder_info[1] = BUF_SPECIAL_SPU_DVD_SUBTYPE;
         m->buf->decoder_info[2] = SPU_DVD_SUBTYPE_PACKAGE;
       }
+      else {
+        if (this->numPreview<5)
+	  ++this->numPreview;
+	if ( this->numPreview==1 )
+	  m->buf->decoder_flags=BUF_FLAG_HEADER | BUF_FLAG_FRAME_END;
+	else if ( this->numPreview<5 )
+	  m->buf->decoder_flags=BUF_FLAG_PREVIEW;
+	else
+	  m->buf->decoder_flags=BUF_FLAG_FRAME_END;
+      }
       m->buf->pts = m->pts;
       m->buf->decoder_info[0] = 1;
       
@@ -902,12 +929,17 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
      m->buf = m->fifo->buffer_pool_alloc(m->fifo);
 
     if (!demux_ts_parse_pes_header(this->stream->xine, m, ts, len, this->stream)) {
-      m->corrupted_pes = 1;
       m->buf->free_buffer(m->buf);
       m->buf = NULL;
-      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+      
+      if (m->corrupted_pes > CORRUPT_PES_THRESHOLD) {
+        if (this->videoPid == m->pid)
+          this->videoPid = INVALID_PID;
+      } else {
+        m->corrupted_pes++;
+        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
 	      "demux_ts: PID 0x%.4x: corrupted pes encountered\n", m->pid);
-
+      }
     } else {
 
       m->corrupted_pes = 0;
@@ -951,7 +983,7 @@ static void demux_ts_pes_new(demux_ts_t*this,
                              unsigned int mediaIndex,
                              unsigned int pid,
                              fifo_buffer_t *fifo,
-			     uint8_t descriptor) {
+			     uint16_t descriptor) {
 
   demux_ts_media *m = &this->media[mediaIndex];
 
@@ -1162,6 +1194,15 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     return;
   }
 
+  if (!section_length) {
+    free (this->pmt[program_count]);
+    this->pmt[program_count] = NULL;
+#ifdef TS_PMT_LOG
+    printf ("ts_demux: eek, zero-length section?\n");
+#endif
+    return;
+  }
+
 #ifdef TS_PMT_LOG
   printf ("ts_demux: have all TS packets for the PMT section\n");
 #endif
@@ -1181,11 +1222,33 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 	     crc32,calc_crc32); 
     return;
   }
-#ifdef TS_PMT_LOG
   else {
+#ifdef TS_PMT_LOG
     printf ("demux_ts: PMT CRC32 ok.\n");
-  }
 #endif
+    if ( crc32==this->last_pmt_crc ) {
+#ifdef TS_PMT_LOG
+      printf("demux_ts: PMT with CRC32=%d already parsed. Skipping.\n", crc32);
+#endif
+      return;
+    }
+    else {
+#ifdef TS_PMT_LOG
+      printf("demux_ts: new PMT, parsing...\n");
+#endif
+      this->last_pmt_crc = crc32;
+    }
+  }
+
+  /*
+   * Forget the current video, audio and subtitle PIDs; if the PMT has not
+   * changed, we'll pick them up again when we parse this PMT, while if the
+   * PMT has changed (e.g. an IPTV streamer that's just changed its source),
+   * we'll get new PIDs that we should follow.
+   */
+  this->audio_tracks_count = 0;
+  this->videoPid = INVALID_PID;
+  this->spu_pid = INVALID_PID;
 
   /*
    * ES definitions start here...we are going to learn upto one video
@@ -1212,7 +1275,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
   /*
    * Extract the elementary streams.
    */
-  this->no_spu_langs = 0;
+  this->spu_langs_count = 0;
   while (section_length > 0) {
     unsigned int stream_info_length;
 
@@ -1236,7 +1299,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     case ISO_14496_PART10_VIDEO:
       if (this->videoPid == INVALID_PID) {
 #ifdef TS_PMT_LOG
-        printf ("demux_ts: PMT video pid 0x%.4x\n", pid);
+        printf ("demux_ts: PMT video pid 0x%.4x type %2.2x\n", pid, stream[0]);
 #endif
         demux_ts_pes_new(this, this->media_num, pid, this->video_fifo,stream[0]);
 	this->videoMedia = this->media_num;
@@ -1248,20 +1311,31 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     case ISO_13818_AUDIO:
     case ISO_13818_PART7_AUDIO:
     case ISO_14496_PART3_AUDIO:
-      if (this->audioPid == INVALID_PID) {
+      if (this->audio_tracks_count < MAX_AUDIO_TRACKS) {
+        int i, found = 0;
+        for(i = 0; i < this->audio_tracks_count; i++) {
+            if(this->audio_tracks[i].pid == pid) {
+                found = 1;
+                break;
+            }
+        }
+        if(!found) {
 #ifdef TS_PMT_LOG
-        printf ("demux_ts: PMT audio pid 0x%.4x\n", pid);
+            printf ("demux_ts: PMT audio pid 0x%.4x type %2.2x\n", pid, stream[0]);
 #endif
-        demux_ts_pes_new(this, this->media_num, pid, this->audio_fifo,stream[0]);
-        this->audioPid = pid;
-        this->audioMedia = this->media_num;
-	demux_ts_get_lang_desc(this, this->audioLang,
+            demux_ts_pes_new(this, this->media_num, pid, this->audio_fifo,stream[0]);
+            this->audio_tracks[this->audio_tracks_count].pid = pid;
+            this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
+            this->media[this->media_num].type = this->audio_tracks_count;
+            demux_ts_get_lang_desc(this, this->audio_tracks[this->audio_tracks_count].lang,
 			       stream + 5, stream_info_length);
+            this->audio_tracks_count++;
+        }
       }
       break;
     case ISO_13818_PRIVATE:
 #ifdef TS_PMT_LOG
-      printf ("demux_ts: PMT streamtype 13818_PRIVATE, pid: 0x%.4x\n", pid);
+      printf ("demux_ts: PMT streamtype 13818_PRIVATE, pid: 0x%.4x type %2.2x\n", pid, stream[0]);
 
       for (i = 5; i < coded_length; i++)
         printf ("%.2x ", stream[i]);
@@ -1270,40 +1344,40 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
       break;
     case  ISO_13818_TYPE_C: /* data carousel */
 #ifdef TS_PMT_LOG
-      printf ("demux_ts: PMT streamtype 13818_TYPE_C, pid: 0x%.4x\n", pid);
+      printf ("demux_ts: PMT streamtype 13818_TYPE_C, pid: 0x%.4x type %2.2x\n", pid, stream[0]);
 #endif
       break;
     case ISO_13818_PES_PRIVATE:
       for (i = 5; i < coded_length; i += stream[i+1] + 2) {
-        if ((stream[i] == 0x6a) && (this->audioPid == INVALID_PID)) {
-          uint32_t format_identifier=0;
+          if ((stream[i] == 0x6a) && (this->audio_tracks_count < MAX_AUDIO_TRACKS)) {
+          int i, found = 0;
+          for(i = 0; i < this->audio_tracks_count; i++) {
+            if(this->audio_tracks[i].pid == pid) {
+              found = 1;
+              break;
+             }
+	  }
+          if(!found) {
 #ifdef TS_PMT_LOG
-          printf ("demux_ts: PMT AC3 audio pid 0x%.4x\n", pid);
+            printf ("demux_ts: PMT AC3 audio pid 0x%.4x type %2.2x\n", pid, stream[0]);
 #endif
-          demux_ts_get_reg_desc(this, &format_identifier,
-                                stream + 5, stream_info_length);
-          if (format_identifier == 0x41432d33) /* AC-3 */
-            demux_ts_pes_new(this, this->media_num, pid,
-                             this->audio_fifo, 0x81);
-          else if((stream[i+1]>1) && (stream[i+2] & 0x80 ) && (stream[3] & 0x40)) /* AC3 Full Service */
-            demux_ts_pes_new(this, this->media_num, pid,
-                             this->audio_fifo, 0x81);
-          else
-            demux_ts_pes_new(this, this->media_num, pid,
-                             this->audio_fifo,stream[0]);
+          demux_ts_pes_new(this, this->media_num, pid,
+                           this->audio_fifo, STREAM_AUDIO_AC3);
 
-          this->audioPid = pid;
-          this->audioMedia = this->media_num;
-	  demux_ts_get_lang_desc(this, this->audioLang,
-				 stream + 5, stream_info_length);
+          this->audio_tracks[this->audio_tracks_count].pid = pid;
+          this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
+          this->media[this->media_num].type = this->audio_tracks_count;
+          demux_ts_get_lang_desc(this, this->audio_tracks[this->audio_tracks_count].lang,
+                                 stream + 5, stream_info_length);
+          this->audio_tracks_count++;
           break;
+          }
         }
-
         /* Teletext */
         else if (stream[i] == 0x56)
           {
 #ifdef TS_PMT_LOG
-            printf ("demux_ts: PMT Teletext, pid: 0x%.4x\n", pid);
+            printf ("demux_ts: PMT Teletext, pid: 0x%.4x type %2.2x\n", pid, stream[0]);
 
             for (i = 5; i < coded_length; i++)
               printf ("%.2x ", stream[i]);
@@ -1318,13 +1392,13 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 	    int pos;
             for (pos = i + 2;
 		 pos + 8 <= i + 2 + stream[i + 1]
-		   && this->no_spu_langs < MAX_NO_SPU_LANGS;
+		   && this->spu_langs_count < MAX_SPU_LANGS;
 		 pos += 8)
 	      {
-		int no = this->no_spu_langs;
+		int no = this->spu_langs_count;
 		demux_ts_spu_lang *lang = &this->spu_langs[no];
 		
-		this->no_spu_langs++;
+		this->spu_langs_count++;
 	
 		memcpy(lang->desc.lang, &stream[pos], 3);
 		lang->desc.lang[3] = 0;
@@ -1334,14 +1408,15 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 		  (stream[pos + 6] << 8) | stream[pos + 7];
 		lang->pid = pid;
 		lang->media_index = this->media_num;
-		demux_ts_pes_new(this, this->media_num,
-				 pid, this->video_fifo,
-				 stream[0]);
+		this->media[this->media_num].type = no;
+		demux_ts_pes_new(this, this->media_num, pid, this->video_fifo, stream[0]);
+		demux_send_special_spu_buf( this, no );
 #ifdef TS_LOG
-		printf("demux_ts: DVBSUB: pid 0x%.4x: %s  page %ld %ld\n",
+		printf("demux_ts: DVBSUB: pid 0x%.4x: %s  page %ld %ld type %2.2x\n",
 		       pid, lang->desc.lang,
 		       lang->desc.comp_page_id,
-		       lang->desc.aux_page_id);
+		       lang->desc.aux_page_id,
+                       stream[0]);
 #endif
 	      }
 	  }
@@ -1356,18 +1431,29 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
  * if is does, we tag this as an audio stream.
  * FIXME: This will need expanding if we ever see a DTS or other media format here.
  */ 
-      if (this->audioPid == INVALID_PID && (stream[0] >= 0x80) ) {
-        uint32_t format_identifier=0;
-        demux_ts_get_reg_desc(this, &format_identifier,
-			       stream + 5, stream_info_length);
-        /* If no format identifier, assume A52 */
-        if ((format_identifier == 0x41432d33) || (format_identifier == 0)) {
-	  demux_ts_pes_new(this, this->media_num, pid, this->audio_fifo, stream[0]);
-          this->audioPid = pid;
-          this->audioMedia = this->media_num;
-	  demux_ts_get_lang_desc(this, this->audioLang,
-		  	       stream + 5, stream_info_length);
-          break;
+        if ((this->audio_tracks_count < MAX_AUDIO_TRACKS) && (stream[0] >= 0x80) ) {
+        int i, found = 0;
+        for(i = 0; i < this->audio_tracks_count; i++) {
+            if(this->audio_tracks[i].pid == pid) {
+                found = 1;
+                break;
+            }
+        }
+        if(!found) {
+            uint32_t format_identifier=0;
+            demux_ts_get_reg_desc(this, &format_identifier,
+                    stream + 5, stream_info_length);
+            /* If no format identifier, assume A52 */
+            if ((format_identifier == 0x41432d33) || (format_identifier == 0)) {
+                demux_ts_pes_new(this, this->media_num, pid, this->audio_fifo, stream[0]);
+                this->audio_tracks[this->audio_tracks_count].pid = pid;
+                this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
+                this->media[this->media_num].type = this->audio_tracks_count;
+                demux_ts_get_lang_desc(this, this->audio_tracks[this->audio_tracks_count].lang,
+                                       stream + 5, stream_info_length);
+                this->audio_tracks_count++;
+                break;
+            }
         }
       } else {
 #ifdef TS_PMT_LOG
@@ -1402,8 +1488,14 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     this->pcrPid = pid;
   }
 
-  /* DVBSUB: update spu decoder */
-  demux_ts_update_spu_channel(this);
+  if ( this->stream->spu_channel>=0 && this->spu_langs_count>0 )
+    demux_ts_update_spu_channel( this );
+
+  /* Inform UI of channels changes */
+  xine_event_t ui_event;
+  ui_event.type = XINE_EVENT_UI_CHANNELS_CHANGED;
+  ui_event.data_length = 0;
+  xine_event_send( this->stream, &ui_event );
 }
 
 static int sync_correct(demux_ts_t*this, uint8_t *buf, int32_t npkt_read) {
@@ -1500,12 +1592,7 @@ static unsigned char * demux_synchronise(demux_ts_t* this) {
        */
 
       if (this->npkt_read == 0) {
-	/* printf ("demux_ts: read 0 packets! (%d)\n", this->read_zero); */
-	this->read_zero++;
-      } else this->read_zero = 0;
-
-      if (this->read_zero > 200) {
-	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_ts: read 0 packets too many times!\n");
+	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_ts: read 0 packets\n");
 	this->status = DEMUX_FINISHED;
 	return NULL;
       }
@@ -1612,6 +1699,16 @@ static int64_t demux_ts_adaptation_field_parse(uint8_t *data,
   return PCR;
 }
 
+/* check if an apid is in the list of known apids */
+static int apid_check(demux_ts_t*this, unsigned int pid) {
+  int i;
+  for(i=0; i<this->audio_tracks_count; i++) {
+    if(this->audio_tracks[i].pid == pid)
+      return i;
+  }
+  return -1;
+}
+
 /* transport stream packet layer */
 static void demux_ts_parse_packet (demux_ts_t*this) {
 
@@ -1744,24 +1841,51 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
     
     if ( (pes_stream_id >= VIDEO_STREAM_S) && (pes_stream_id <= VIDEO_STREAM_E) ) {
       if ( this->videoPid == INVALID_PID) {
-
-	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		 "demux_ts: auto-detected video pid 0x%.4x\n", pid);
-
-	this->videoPid = pid;
-	this->videoMedia = this->media_num;
-	demux_ts_pes_new(this, this->media_num++, pid, this->video_fifo, pes_stream_id);
+        int i, found = 0;
+        for(i = 0; i < this->media_num; i++) {
+          if (this->media[i].pid == pid) {
+            found = 1;
+            break;
+          }
+        }
+        
+        if (found && (this->media[i].corrupted_pes == 0)) {
+          this->videoPid = pid;
+	  this->videoMedia = i;
+        } else if (!found) {
+	  this->videoPid = pid;
+	  this->videoMedia = this->media_num;
+	  demux_ts_pes_new(this, this->media_num++, pid, this->video_fifo, 0x100 + pes_stream_id);
+        }
+        
+        if (this->videoPid != INVALID_PID) {
+          xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
+                   "demux_ts: auto-detected video pid 0x%.4x\n", pid);
+        } 
       }
     } else if ( (pes_stream_id >= AUDIO_STREAM_S) && (pes_stream_id <= AUDIO_STREAM_E) ) {
-      if ( this->audioPid == INVALID_PID) {
-
-	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		 "demux_ts: auto-detected audio pid 0x%.4x\n", pid);
-
-	this->audioPid = pid;
-	this->audioMedia = this->media_num;
-	demux_ts_pes_new(this, this->media_num++, pid, this->audio_fifo, pes_stream_id);
-      }
+       if (this->audio_tracks_count < MAX_AUDIO_TRACKS) {
+           int i, found = 0;
+           for(i = 0; i < this->audio_tracks_count; i++) {
+               if(this->audio_tracks[i].pid == pid) {
+                   found = 1;
+                   break;
+               }
+           }
+           if(!found) {
+#ifdef TS_PMT_LOG
+               xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+                        "demux_ts: auto-detected audio pid 0x%.4x\n", pid);
+#endif
+               /* store PID, index and stream no. */
+               this->audio_tracks[this->audio_tracks_count].pid = pid;
+               this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
+               this->media[this->media_num].type = this->audio_tracks_count;
+               demux_ts_pes_new(this, this->media_num++, pid,
+                                this->audio_fifo, 0x100 + pes_stream_id);
+               this->audio_tracks_count++;
+	   }
+       }
     }
   }
   
@@ -1775,6 +1899,7 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
     /*
      * Do the demuxing in descending order of packet frequency!
      */
+    int index;
     if (pid == this->videoPid) {
 #ifdef TS_LOG
       printf ("demux_ts: Video pid: 0x%.4x\n", pid);
@@ -1785,12 +1910,13 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 			   data_len);
       return;
     }
-    else if (pid == this->audioPid) {
+    else if ((index = apid_check(this, pid)) > -1) {
 #ifdef TS_LOG
       printf ("demux_ts: Audio pid: 0x%.4x\n", pid);
 #endif
-      check_newpts(this, this->media[this->audioMedia].pts, PTS_AUDIO);
-      demux_ts_buffer_pes (this, originalPkt+data_offset, this->audioMedia,
+      check_newpts(this, this->media[this->audio_tracks[index].media_index].pts, PTS_AUDIO);
+      demux_ts_buffer_pes (this, originalPkt+data_offset,
+               this->audio_tracks[index].media_index,
 			   payload_unit_start_indicator, continuity_counter,
 			   data_len);
       return;
@@ -1830,12 +1956,13 @@ static void demux_ts_event_handler (demux_ts_t *this) {
     case XINE_EVENT_PIDS_CHANGE:
 
       this->videoPid    = INVALID_PID;
-      this->audioPid    = INVALID_PID;
+      this->audio_tracks_count = 0;
       this->media_num   = 0;
       this->send_newpts = 1;
       this->spu_pid     = INVALID_PID;
       this->spu_media   = 0;
-      this->no_spu_langs= 0;
+      this->spu_langs_count= 0;
+      this->last_pmt_crc = 0;
       _x_demux_control_start (this->stream);
       break;
       
@@ -1909,8 +2036,9 @@ static void demux_ts_send_headers (demux_plugin_t *this_gen) {
    */
 
   this->videoPid = INVALID_PID;
-  this->audioPid = INVALID_PID;
+  this->audio_tracks_count = 0;
   this->media_num= 0;
+  this->last_pmt_crc = 0;
 
   _x_demux_control_start (this->stream);
   
@@ -1927,8 +2055,8 @@ static void demux_ts_send_headers (demux_plugin_t *this_gen) {
   
   /* DVBSUB */
   this->spu_pid = INVALID_PID;
-  this->no_spu_langs = 0;
-  this->current_spu_channel = this->stream->spu_channel;
+  this->spu_langs_count = 0;
+  this->current_spu_channel = -1;
   
   /* FIXME ? */
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 1);
@@ -2005,6 +2133,7 @@ static int demux_ts_get_optional_data(demux_plugin_t *this_gen,
 {
   demux_ts_t *this = (demux_ts_t *) this_gen;
   char *str = data;
+  int channel = *((int *)data);
 
   /* be a bit paranoid */
   if (this == NULL || this->stream == NULL)
@@ -2013,31 +2142,23 @@ static int demux_ts_get_optional_data(demux_plugin_t *this_gen,
   switch (data_type)
     {
     case DEMUX_OPTIONAL_DATA_AUDIOLANG:
-      if (this->audioLang[0])
-	{
-	  strcpy(str, this->audioLang);
-	}
-      else
-	{
-	  sprintf(str, "%3i", _x_get_audio_channel(this->stream));
-	}
+      if ((channel >= 0) && (channel < this->audio_tracks_count)) {
+        if(this->audio_tracks[channel].lang)
+          strcpy(str, this->audio_tracks[channel].lang);
+        else
+          sprintf(str, "%3i", _x_get_audio_channel(this->stream));
+      }
+      else {
+        snprintf(str, XINE_LANG_MAX, "%3i", _x_get_audio_channel(this->stream));
+      }
       return DEMUX_OPTIONAL_SUCCESS;
 
     case DEMUX_OPTIONAL_DATA_SPULANG:
-      if (this->current_spu_channel >= 0
-	  && this->current_spu_channel < this->no_spu_langs)
-	{
-	  memcpy(str, this->spu_langs[this->current_spu_channel].desc.lang, 3);
-	  str[4] = 0;
-	}
-      else if (this->current_spu_channel == -1)
-	{
-	  strcpy(str, "none");
-	}
-      else
-	{
-	  sprintf(str, "%3i", this->current_spu_channel);
-	}
+      if (channel>=0 && channel<this->spu_langs_count) {
+        memcpy(str, this->spu_langs[channel].desc.lang, 3);
+	str[3] = 0;}
+      else 
+        strcpy(str, "none");
       return DEMUX_OPTIONAL_SUCCESS;
 
     default:
@@ -2127,7 +2248,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
    * if we reach this point, the input has been accepted.
    */
 
-  this            = xine_xmalloc(sizeof(*this));
+  this            = calloc(1, sizeof(*this));
   this->stream    = stream;
   this->input     = input;
   this->blockSize = PKT_SIZE;
@@ -2161,26 +2282,23 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this->pcrPid = INVALID_PID;
   this->scrambled_npids = 0;
   this->videoPid = INVALID_PID;
-  this->audioPid = INVALID_PID;
+  this->audio_tracks_count = 0;
+  this->last_pmt_crc = 0;
 
   this->rate = 16000; /* FIXME */
   
   this->status = DEMUX_FINISHED;
 
-#ifdef TS_READ_STATS
-  for (i=0; i<=NPKT_PER_READ; i++) {
-    this->rstat[i] = 0;
-  }
-#endif
-
   /* DVBSUB */
   this->spu_pid = INVALID_PID;
-  this->no_spu_langs = 0;
-  this->current_spu_channel = this->stream->spu_channel;
+  this->spu_langs_count = 0;
+  this->current_spu_channel = -1;
 
   /* dvb */
   this->event_queue = xine_event_new_queue (this->stream);
   
+  this->numPreview=0;
+  
   return &this->demux_plugin;
 }
 
@@ -2215,7 +2333,7 @@ static void *init_class (xine_t *xine, void *data) {
   
   demux_ts_class_t     *this;
   
-  this         = xine_xmalloc (sizeof (demux_ts_class_t));
+  this         = calloc(1, sizeof(demux_ts_class_t));
   this->config = xine->config;
   this->xine   = xine;
 
@@ -2242,3 +2360,4 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   { PLUGIN_DEMUX, 26, "mpeg-ts", XINE_VERSION_CODE, &demux_info_ts, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
+
