@@ -203,7 +203,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
    * if we reach this point, the input has been accepted.
    */
 
-  this         = xine_xmalloc (sizeof (demux_yuv_frames_t));
+  this         = calloc(1, sizeof(demux_yuv_frames_t));
   this->stream = stream;
   this->input  = input;
 
@@ -253,7 +253,7 @@ static void class_dispose (demux_class_t *this_gen) {
 static void *init_class (xine_t *xine, void *data) {
   demux_yuv_frames_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_yuv_frames_class_t));
+  this = calloc(1, sizeof(demux_yuv_frames_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
