@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: group_audio.h,v 1.8 2005/05/29 19:20:48 jstembridge Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #ifndef HAVE_GROUP_AUDIO_H
@@ -34,14 +32,18 @@ void *demux_dts_init_plugin (xine_t *xine, void *data);
 void *demux_flac_init_plugin (xine_t *xine, void *data);
 void *demux_mpgaudio_init_class (xine_t *xine, void *data);
 void *demux_mpc_init_plugin (xine_t *xine, void *data);
-void *demux_nsf_init_plugin (xine_t *xine, void *data);
 void *demux_realaudio_init_plugin (xine_t *xine, void *data);
 void *demux_shn_init_plugin (xine_t *xine, void *data);
 void *demux_snd_init_plugin (xine_t *xine, void *data);
+void *demux_tta_init_plugin (xine_t *xine, void *data);
 void *demux_voc_init_plugin (xine_t *xine, void *data);
 void *demux_vox_init_plugin (xine_t *xine, void *data);
 void *demux_wav_init_plugin (xine_t *xine, void *data);
 
+#ifdef HAVE_NOSEFART
+void *demux_nsf_init_plugin (xine_t *xine, void *data);
+#endif
+
 #ifdef HAVE_MODPLUG
 void *demux_mod_init_plugin (xine_t *xine, void *data);
 #endif
