@@ -337,9 +337,9 @@ static int id3v22_interp_frame(input_plugin_t *input,
 }
 
 
-int id3v22_parse_tag(input_plugin_t *input,
-                     xine_stream_t *stream,
-                     int8_t *mp3_frame_header) {
+static int id3v22_parse_tag(input_plugin_t *input,
+			    xine_stream_t *stream,
+			    int8_t *mp3_frame_header) {
   id3v2_header_t tag_header;
   id3v22_frame_header_t tag_frame_header;
   int pos = 0;
@@ -530,9 +530,9 @@ static int id3v23_interp_frame(input_plugin_t *input,
   }
 }
 
-int id3v23_parse_tag(input_plugin_t *input,
-                     xine_stream_t *stream,
-                     int8_t *mp3_frame_header) {
+static int id3v23_parse_tag(input_plugin_t *input,
+			    xine_stream_t *stream,
+			    int8_t *mp3_frame_header) {
   id3v2_header_t tag_header;
   id3v23_frame_header_t tag_frame_header;
   id3v23_frame_ext_header_t tag_frame_ext_header;
@@ -783,9 +783,9 @@ static int id3v24_interp_frame(input_plugin_t *input,
   }
 }
 
-int id3v24_parse_tag(input_plugin_t *input,
-                     xine_stream_t *stream,
-                     int8_t *mp3_frame_header) {
+static int id3v24_parse_tag(input_plugin_t *input,
+			    xine_stream_t *stream,
+			    int8_t *mp3_frame_header) {
   id3v2_header_t tag_header;
   id3v24_frame_header_t tag_frame_header;
   id3v24_frame_ext_header_t tag_frame_ext_header;
