@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2002-2004 the xine project
  *
  * Copyright (C) James Courtier-Dutton James@superbug.demon.co.uk - July 2001
  *
@@ -34,9 +35,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; see the file COPYING.  If not, write to
  * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: spu.c,v 1.43 2002/07/20 16:28:14 mroi Exp $
- *
  */
 
 #include <stdio.h>
@@ -45,37 +43,36 @@
 #include <unistd.h>
 #include <string.h>
 #include <inttypes.h>
-#include <malloc.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
 
-#include "video_out/alphablend.h"
-#include "xineutils.h"
 #include "xine_internal.h"
+#include "xineutils.h"
 #include "spu.h"
 #include "buffer.h"
-#include "events.h"
 #include "xine-engine/bswap.h"
-#include "nav_types.h"
-#include "nav_read.h"
-#include "nav_print.h"
+#ifdef HAVE_DVDNAV
+#  include <dvdnav/nav_read.h>
+#  include <dvdnav/nav_print.h>
+#else
+#  include "nav_read.h"
+#  include "nav_print.h"
+#endif
+
 
 /*
 #define LOG_DEBUG 1
-#define LOG_NAV 1
 #define LOG_BUTTON 1
+#define LOG_NAV 1
 */
 
-void spudec_reassembly (spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len);
-void spudec_process( spudec_decoder_t *this, uint32_t stream_id);
-void spudec_decode_nav( spudec_decoder_t *this, buf_element_t *buf);
-void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode,
-                                vo_overlay_t * overlay, vo_overlay_t * base );
-static void spudec_do_commands (spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl);
-static void spudec_draw_picture (spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl);
-static void spudec_discover_clut (spudec_state_t *state, vo_overlay_t *ovl);
+static void spudec_do_commands (xine_t *xine, spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl);
+static void spudec_draw_picture (xine_t *xine, spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl);
+static void spudec_discover_clut (xine_t *xine, spudec_state_t *state, vo_overlay_t *ovl);
+#ifdef LOG_DEBUG
 static void spudec_print_overlay( vo_overlay_t *overlay );
+#endif
 
 void spudec_decode_nav(spudec_decoder_t *this, buf_element_t *buf) {
   uint8_t                  *p;
@@ -84,11 +81,12 @@ void spudec_decode_nav(spudec_decoder_t *this, buf_element_t *buf) {
   uint32_t                  header_len;
   pci_t                     pci;
   dsi_t                     dsi;
-  video_overlay_instance_t *ovl_instance = this->vo_out->get_overlay_instance (this->vo_out);
+  video_overlay_manager_t  *ovl_manager = this->stream->video_out->get_overlay_manager (this->stream->video_out);
 
   p = buf->content;
   if (p[0] || p[1] || (p[2] != 1)) {
-    printf("libspudec:spudec_decode_nav:nav demux error! %02x %02x %02x (should be 0x000001) \n",p[0],p[1],p[2]);
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	    "libspudec:spudec_decode_nav:nav demux error! %02x %02x %02x (should be 0x000001) \n",p[0],p[1],p[2]);
     return;
   }
 
@@ -107,12 +105,11 @@ void spudec_decode_nav(spudec_decoder_t *this, buf_element_t *buf) {
  */
     if(p[0] == 0x00) {
 #ifdef LOG_NAV
-      int btngr_ns = 0, btn_ns = 0;
       printf("libspudec:nav_PCI\n");
 #endif
-      nav_read_pci(&pci, p+1);
+      navRead_PCI(&pci, p+1);
 #ifdef LOG_NAV
-      printf("libspudec:nav:hli_ss=%u, hli_s_ptm=%u, hli_e_ptm=%u, btn_sl_e_ptm=%u pts=%llu\n",
+      printf("libspudec:nav:hli_ss=%u, hli_s_ptm=%u, hli_e_ptm=%u, btn_sl_e_ptm=%u pts=%lli\n",
        pci.hli.hl_gi.hli_ss,
        pci.hli.hl_gi.hli_s_ptm,
        pci.hli.hl_gi.hli_e_ptm,
@@ -126,10 +123,10 @@ void spudec_decode_nav(spudec_decoder_t *this, buf_element_t *buf) {
       printf("btngr%d_dsp_ty    0x%02x\n", 2, pci.hli.hl_gi.btngr2_dsp_ty);
       printf("btngr%d_dsp_ty    0x%02x\n", 3, pci.hli.hl_gi.btngr3_dsp_ty);
       //navPrint_PCI(&pci); 
-      navPrint_PCI_GI(&pci.pci_gi);
-      navPrint_NSML_AGLI(&pci.nsml_agli);
+      //navPrint_PCI_GI(&pci.pci_gi);
+      //navPrint_NSML_AGLI(&pci.nsml_agli);
       //navPrint_HLI(&pci.hli);
-      navPrint_HL_GI(&pci.hli.hl_gi, & btngr_ns, & btn_ns);
+      //navPrint_HL_GI(&pci.hli.hl_gi, & btngr_ns, & btn_ns);
 #endif
     }
 
@@ -143,99 +140,188 @@ void spudec_decode_nav(spudec_decoder_t *this, buf_element_t *buf) {
 #ifdef LOG_NAV
       printf("NAV DSI packet\n");  
 #endif
-      nav_read_dsi(&dsi, p+1);
+      navRead_DSI(&dsi, p+1);
 
 //      self->vobu_start = self->dsi.dsi_gi.nv_pck_lbn;
 //      self->vobu_length = self->dsi.dsi_gi.vobu_ea;
     }
   }
+  
+  /* NAV packets contain start and end presentation timestamps, which tell the
+   * application, when the highlight information in the NAV is supposed to be valid.
+   * We handle these timestamps only in a very stripped-down way: We keep a list
+   * of NAV packets (or better: the PCI part of them), tagged with a VPTS timestamp
+   * telling, when the NAV should be processed. However, we only enqueue a new node
+   * into this list, when we receive new highlight information during an already
+   * showing menu. This happens very rarerly on common DVDs, so it is of low impact.
+   * And we only check for processing of queued entries at some prominent
+   * locations in this SPU decoder. Since presentation timestamps rarely solve a real
+   * purpose on most DVDs, this is ok compared to the full-blown solution, which would
+   * require a separate thread managing the queue all the time. */
   pthread_mutex_lock(&this->nav_pci_lock);
   switch (pci.hli.hl_gi.hli_ss) {
     case 0:
       /* No Highlight information for this VOBU */
-      if ( this->pci.hli.hl_gi.hli_ss == 1) {
+      if ( this->pci_cur.pci.hli.hl_gi.hli_ss == 1) {
         /* Hide menu spu between menus */
 #ifdef LOG_BUTTON
         printf("libspudec:nav:SHOULD HIDE SPU here\n");
 #endif
         if( this->menu_handle < 0 ) {
-          this->menu_handle = ovl_instance->get_handle(ovl_instance,1);
+          this->menu_handle = ovl_manager->get_handle(ovl_manager,1);
         }
         if( this->menu_handle >= 0 ) {
-          int64_t vpts_offset; 
-          metronom_t *metronom = this->xine->metronom;
           this->event.object.handle = this->menu_handle;
-          this->event.event_type = EVENT_HIDE_MENU;
-          /* if !vpts then we are near a discontinuity but video_out havent detected
-             it yet and we cannot provide correct vpts values. use current_time 
-             instead as an aproximation.
-           */
-          vpts_offset = metronom->got_spu_packet(metronom, -1);
-          this->event.vpts = metronom->got_spu_packet(metronom, this->pci.pci_gi.vobu_e_ptm);
-          /* Keep all the events in the correct order. */
-          /* This corrects for errors during estimation around discontinuity */
-          if( this->event.vpts < this->last_event_vpts ) {
-#ifdef LOG_BUTTON
-            fprintf(stdout, "libspudec: add_event estimation correction. vpts was %lld\n", this->event.vpts);
-#endif
-            this->event.vpts = this->last_event_vpts + 1;
-          }
-          this->last_event_vpts = this->event.vpts;
-#ifdef LOG_BUTTON
-          fprintf(stdout, "libspudec: add_event HIDE_MENU type=%d : current time=%lld, spu vpts=%lld, vpts_offset=%lld\n",
-                  this->event.event_type,
-                  this->xine->metronom->get_current_time(this->xine->metronom),
-                  this->event.vpts,
-                  vpts_offset);
-#endif
-          ovl_instance->add_event(ovl_instance, (void *)&this->event);
+          this->event.event_type = OVERLAY_EVENT_HIDE;
+	  /* hide menu right now */
+	  this->event.vpts = 0;
+          ovl_manager->add_event(ovl_manager, (void *)&this->event);
         } else {
-          printf("libspudec: No video_overlay handles left for menu\n");
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "libspudec: No video_overlay handles left for menu\n");
         }
       }
-      xine_fast_memcpy(&this->pci, &pci, sizeof(pci_t));
+      spudec_clear_nav_list(this);
+      xine_fast_memcpy(&this->pci_cur.pci, &pci, sizeof(pci_t));
+      /* incoming SPUs will be plain subtitles */
+      this->event.object.object_type = 0;
+      if (this->button_filter) {
+	/* we possibly had buttons before, so we update the UI info */
+	xine_event_t   event;
+	xine_ui_data_t data;
+	
+	event.type = XINE_EVENT_UI_NUM_BUTTONS;
+	event.data = &data;
+	event.data_length = sizeof(data);
+	data.num_buttons = 0;
+	
+	xine_event_send(this->stream, &event);
+      }
       this->button_filter=0;
 
       break;
     case 1: 
       /* All New Highlight information for this VOBU */
-      xine_fast_memcpy(&this->pci, &pci, sizeof(pci_t));
-      this->button_filter=1;
-      /*******************************
-       * We should do something about fosl_btnn, but
-       * until we can send the info to dvdnav, ignore it.
-       * if( pci.hli.hl_gi.fosl_btnn) {
-       *   this->buttonN = pci.hli.hl_gi.fosl_btnn;
-       * }
-       *******************************/
+      if (this->pci_cur.pci.hli.hl_gi.hli_ss != 0 &&
+	  pci.hli.hl_gi.hli_s_ptm > this->pci_cur.pci.hli.hl_gi.hli_s_ptm) {
+	pci_node_t *node = &this->pci_cur;
+#ifdef LOG_DEBUG
+	printf("libspudec: allocating new PCI node for hli_s_ptm %d\n", pci.hli.hl_gi.hli_s_ptm);
+#endif
+	/* append PCI at the end of the list */
+	while (node->next) node = node->next;
+	node->next = malloc(sizeof(pci_node_t));
+	node->next->vpts = this->stream->metronom->got_spu_packet(this->stream->metronom, pci.hli.hl_gi.hli_s_ptm);
+	node->next->next = NULL;
+	xine_fast_memcpy(&node->next->pci, &pci, sizeof(pci_t));
+      } else {
+        spudec_clear_nav_list(this);
+        /* menu ahead, remember PCI for later use */
+        xine_fast_memcpy(&this->pci_cur.pci, &pci, sizeof(pci_t));
+        spudec_process_nav(this);
+      }
       break;
     case 2:
       /* Use Highlight information from previous VOBU */
-      this->pci.pci_gi.vobu_s_ptm = pci.pci_gi.vobu_s_ptm;
-      this->pci.pci_gi.vobu_e_ptm = pci.pci_gi.vobu_e_ptm;
-      this->pci.pci_gi.vobu_se_e_ptm = pci.pci_gi.vobu_se_e_ptm;
+      if (this->pci_cur.next) {
+	/* apply changes to last enqueued NAV */
+	pci_node_t *node = this->pci_cur.next;
+	while (node->next) node = node->next;
+	node->pci.pci_gi.vobu_s_ptm = pci.pci_gi.vobu_s_ptm;
+	node->pci.pci_gi.vobu_e_ptm = pci.pci_gi.vobu_e_ptm;
+	node->pci.pci_gi.vobu_se_e_ptm = pci.pci_gi.vobu_se_e_ptm;
+	spudec_update_nav(this);
+      } else {
+	this->pci_cur.pci.pci_gi.vobu_s_ptm = pci.pci_gi.vobu_s_ptm;
+	this->pci_cur.pci.pci_gi.vobu_e_ptm = pci.pci_gi.vobu_e_ptm;
+	this->pci_cur.pci.pci_gi.vobu_se_e_ptm = pci.pci_gi.vobu_se_e_ptm;
+      }
       break;
     case 3:
       /* Use Highlight information from previous VOBU except commands, which come from this VOBU */
-      this->pci.pci_gi.vobu_s_ptm = pci.pci_gi.vobu_s_ptm;
-      this->pci.pci_gi.vobu_e_ptm = pci.pci_gi.vobu_e_ptm;
-      this->pci.pci_gi.vobu_se_e_ptm = pci.pci_gi.vobu_se_e_ptm;
-      /* FIXME: Add command copying here */
+      if (this->pci_cur.next) {
+	/* apply changes to last enqueued NAV */
+	pci_node_t *node = this->pci_cur.next;
+	while (node->next) node = node->next;
+	node->pci.pci_gi.vobu_s_ptm = pci.pci_gi.vobu_s_ptm;
+	node->pci.pci_gi.vobu_e_ptm = pci.pci_gi.vobu_e_ptm;
+	node->pci.pci_gi.vobu_se_e_ptm = pci.pci_gi.vobu_se_e_ptm;
+	/* FIXME: Add command copying here */
+	spudec_update_nav(this);
+      } else {
+	this->pci_cur.pci.pci_gi.vobu_s_ptm = pci.pci_gi.vobu_s_ptm;
+	this->pci_cur.pci.pci_gi.vobu_e_ptm = pci.pci_gi.vobu_e_ptm;
+	this->pci_cur.pci.pci_gi.vobu_se_e_ptm = pci.pci_gi.vobu_se_e_ptm;
+	/* FIXME: Add command copying here */
+      }
       break;
    default:
-      assert(0);
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	      "libspudec: unknown pci.hli.hl_gi.hli_ss = %d\n", pci.hli.hl_gi.hli_ss );
       break;
   }
   pthread_mutex_unlock(&this->nav_pci_lock);
   return;
 }
 
-void spudec_reassembly (spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len)
+void spudec_clear_nav_list(spudec_decoder_t *this)
 {
+  while (this->pci_cur.next) {
+    pci_node_t *node = this->pci_cur.next->next;
+    free(this->pci_cur.next);
+    this->pci_cur.next = node;
+  }
+  /* invalidate current timestamp */
+  this->pci_cur.pci.hli.hl_gi.hli_s_ptm = (uint32_t)-1;
+}
+
+void spudec_update_nav(spudec_decoder_t *this)
+{
+  metronom_clock_t *clock = this->stream->xine->clock;
+  
+  if (this->pci_cur.next && this->pci_cur.next->vpts <= clock->get_current_time(clock)) {
+    pci_node_t *node = this->pci_cur.next;
+    xine_fast_memcpy(&this->pci_cur, this->pci_cur.next, sizeof(pci_node_t));
+    spudec_process_nav(this);
+    free(node);
+  }
+}
+
+void spudec_process_nav(spudec_decoder_t *this)
+{
+  /* incoming SPUs will be menus */
+  this->event.object.object_type = 1;
+  if (!this->button_filter) {
+    /* we possibly entered a menu, so we update the UI button info */
+    xine_event_t   event;
+    xine_ui_data_t data;
+    
+    event.type = XINE_EVENT_UI_NUM_BUTTONS;
+    event.data = &data;
+    event.data_length = sizeof(data);
+    data.num_buttons = this->pci_cur.pci.hli.hl_gi.btn_ns;
+    
+    xine_event_send(this->stream, &event);
+  }
+  this->button_filter=1;
+}
+
+void spudec_reassembly (xine_t *xine, spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len)
+{
+#ifdef LOG_DEBUG
+  printf ("libspudec: seq->complete = %d\n", seq->complete);
+  printf("libspudec:1: seq->ra_offs = %d, seq->seq_len = %d, seq->buf_len = %d, seq->buf=%p\n",
+             seq->ra_offs,
+             seq->seq_len,
+             seq->buf_len,
+             seq->buf);
+#endif
   if (seq->complete) {
     seq->seq_len = (((uint32_t)pkt_data[0])<<8) | pkt_data[1];
     seq->cmd_offs = (((uint32_t)pkt_data[2])<<8) | pkt_data[3];
-
+    if (seq->cmd_offs >= seq->seq_len) { 
+      xprintf(xine, XINE_VERBOSITY_DEBUG, "libspudec:faulty stream\n");
+      seq->broken = 1;
+    }
     if (seq->buf_len < seq->seq_len) {
       seq->buf_len = seq->seq_len;
 #ifdef LOG_DEBUG
@@ -259,13 +345,22 @@ void spudec_reassembly (spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len)
 #endif
   }
 
-  if (seq->ra_offs < seq->buf_len) {
+#ifdef LOG_DEBUG
+  printf("libspudec:2: seq->ra_offs = %d, seq->seq_len = %d, seq->buf_len = %d, seq->buf=%p\n",
+             seq->ra_offs,
+             seq->seq_len,
+             seq->buf_len,
+             seq->buf);
+#endif
+  if (seq->ra_offs < seq->seq_len) {
     if (seq->ra_offs + pkt_len > seq->seq_len)
       pkt_len = seq->seq_len - seq->ra_offs;
-      
     memcpy (seq->buf + seq->ra_offs, pkt_data, pkt_len);
     seq->ra_offs += pkt_len;
-  }
+  } else {
+    xprintf(xine, XINE_VERBOSITY_DEBUG, "libspudec:faulty stream\n");
+    seq->broken = 1;
+  } 
 
   if (seq->ra_offs == seq->seq_len) {
     seq->finished = 0;
@@ -276,42 +371,49 @@ void spudec_reassembly (spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len)
   return;
 }
 
-void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
+void spudec_process (spudec_decoder_t *this, int stream_id) {
   spudec_seq_t    *cur_seq;
-  video_overlay_instance_t *ovl_instance = this->vo_out->get_overlay_instance (this->vo_out);
+  video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager (this->stream->video_out);
   int pending = 1;
   cur_seq = &this->spudec_stream_state[stream_id].ra_seq;
 
 #ifdef LOG_DEBUG
-  printf ("spu: Found SPU from stream %d pts=%lld vpts=%lld\n",stream_id, 
+  printf ("spu: Found SPU from stream %d pts=%lli vpts=%lli\n",stream_id, 
           this->spudec_stream_state[stream_id].pts,
           this->spudec_stream_state[stream_id].vpts); 
 #endif
   this->state.cmd_ptr = cur_seq->buf + cur_seq->cmd_offs;
   this->state.modified = 1; /* Only draw picture if = 1 on first event of SPU */
-  this->state.visible = EVENT_SHOW_MENU; /* 0 - No value, 1 - Show, 2 - Hide. */
+  this->state.visible = OVERLAY_EVENT_SHOW;
   this->state.forced_display = 0; /* 0 - No value, 1 - Forced Display. */
   this->state.delay = 0;
   cur_seq->finished=0;
   
   do {
     if (!(cur_seq->finished) ) {
+      pci_node_t *node;
       
-      /* Get do commands to build the event. */
-      spudec_do_commands(&this->state, cur_seq, &this->overlay);
-      /* FIXME: Check for Forced-display or subtitle stream
-       *        For subtitles, open event.
-       *        For menus, store it for later.
-       */
       /* spu_channel is now set based on whether we are in the menu or not. */
       /* Bit 7 is set if only forced display SPUs should be shown */
-      if ( (this->xine->spu_channel & 0x1f) != stream_id  ) { 
+      if ( (this->stream->spu_channel & 0x1f) != stream_id  ) { 
 #ifdef LOG_DEBUG
         printf ("spu: Dropping SPU channel %d. Not selected stream_id\n", stream_id);
 #endif
         return;
       }
-      if ( (this->state.forced_display == 0) && (this->xine->spu_channel & 0x80) ) { 
+      /* parse SPU command sequence, this will update forced_display, so it must come
+       * before the check for it */
+      spudec_do_commands(this->stream->xine, &this->state, cur_seq, &this->overlay);
+      /* FIXME: Check for Forced-display or subtitle stream
+       *        For subtitles, open event.
+       *        For menus, store it for later.
+       */
+      if (cur_seq->broken) {
+        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "libspudec: dropping broken SPU\n");
+	cur_seq->broken = 0;
+	return;
+      }
+      if ( (this->state.forced_display == 0) && (this->stream->spu_channel & 0x80) ) { 
 #ifdef LOG_DEBUG
         printf ("spu: Dropping SPU channel %d. Only allow forced display SPUs\n", stream_id);
 #endif
@@ -319,12 +421,16 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
       }
 
 #ifdef LOG_DEBUG
-      /* spudec_print_overlay( &this->overlay ); */
+      spudec_print_overlay( &this->overlay );
       printf ("spu: forced display:%s\n", this->state.forced_display ? "Yes" : "No" ); 
 #endif
       pthread_mutex_lock(&this->nav_pci_lock);
-      if (this->pci.hli.hl_gi.hli_s_ptm == this->spudec_stream_state[stream_id].pts) {
-        if (this->state.visible == EVENT_HIDE_MENU) {
+      /* search for a PCI that matches this SPU's PTS */
+      for (node = &this->pci_cur; node; node = node->next)
+	if (node->pci.hli.hl_gi.hli_s_ptm == this->spudec_stream_state[stream_id].pts)
+	  break;
+      if (node) {
+        if (this->state.visible == OVERLAY_EVENT_HIDE) {
           /* menus are hidden via nav packet decoding, not here */
 	  /* FIXME: James is not sure about this solution and may want to look this over.
 	   *        I'm commiting it, because I haven't found a disc it breaks, but it fixes
@@ -332,45 +438,80 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
           pthread_mutex_unlock(&this->nav_pci_lock);
           continue;
         }
-        if ( this->pci.hli.hl_gi.fosl_btnn > 0) {
-          spu_button_t spu_button;
-          xine_spu_event_t spu_event;
-          this->buttonN = this->pci.hli.hl_gi.fosl_btnn ;
-          spu_event.event.type = XINE_EVENT_INPUT_BUTTON_FORCE;
-          spu_event.data = &spu_button;
-          spu_button.buttonN  = this->buttonN;
-          xine_send_event(this->xine, &spu_event.event);
+        if (node->pci.hli.hl_gi.fosl_btnn > 0) {
+	  xine_event_t event;
+	  
+          this->buttonN     = node->pci.hli.hl_gi.fosl_btnn;
+          event.type        = XINE_EVENT_INPUT_BUTTON_FORCE;
+	  event.stream      = this->stream;
+	  event.data        = &this->buttonN;
+	  event.data_length = sizeof(this->buttonN);
+          xine_event_send(this->stream, &event);
         }
 #ifdef LOG_BUTTON
         fprintf(stderr, "libspudec:Full Overlay\n");
 #endif
-        spudec_copy_nav_to_overlay(&this->pci, this->state.clut, this->buttonN, 0, &this->overlay,
-                                   &this->overlay);
+        if (!spudec_copy_nav_to_overlay(this->stream->xine, 
+					&node->pci, this->state.clut, 
+					this->buttonN, 0, &this->overlay, &this->overlay)) {
+          /* current button does not exist -> use another one */
+	  xine_event_t event;
+	  
+	  if (this->buttonN > node->pci.hli.hl_gi.btn_ns)
+	    this->buttonN = node->pci.hli.hl_gi.btn_ns;
+	  else
+	    this->buttonN = 1;
+          event.type        = XINE_EVENT_INPUT_BUTTON_FORCE;
+	  event.stream      = this->stream;
+	  event.data        = &this->buttonN;
+	  event.data_length = sizeof(this->buttonN);
+          xine_event_send(this->stream, &event);
+	  spudec_copy_nav_to_overlay(this->stream->xine, 
+				     &node->pci, this->state.clut, 
+				     this->buttonN, 0, &this->overlay, &this->overlay);
+        }
       } else {
       /* Subtitle and not a menu button */
         int i;
         for (i = 0;i < 4; i++) {
-          this->overlay.clip_color[i] = this->overlay.color[i];
-          this->overlay.clip_trans[i] = this->overlay.trans[i];
+          this->overlay.hili_color[i] = this->overlay.color[i];
+          this->overlay.hili_trans[i] = this->overlay.trans[i];
         }
       }
       pthread_mutex_unlock(&this->nav_pci_lock);
 
       if ((this->state.modified) ) { 
-        spudec_draw_picture(&this->state, cur_seq, &this->overlay);
+        spudec_draw_picture(this->stream->xine, &this->state, cur_seq, &this->overlay);
       }
       
       if (this->state.need_clut) {
-        spudec_discover_clut(&this->state, &this->overlay);
+        spudec_discover_clut(this->stream->xine, &this->state, &this->overlay);
+      }
+      
+      if (this->state.vobsub) {
+        int width, height;
+        int64_t duration;
+        
+        /*
+         * vobsubs are usually played with a scaled-down stream (not full DVD
+         * resolution), therefore we should try to realign it.
+         */
+        
+        this->stream->video_out->status(this->stream->video_out, NULL,
+                                        &width, &height, &duration );
+
+        this->overlay.x = (width - this->overlay.width) / 2;
+        this->overlay.y = height - this->overlay.height;
       }
       
       /* Subtitle */
       if( this->menu_handle < 0 ) {
-        this->menu_handle = ovl_instance->get_handle(ovl_instance,1);
+        this->menu_handle = ovl_manager->get_handle(ovl_manager,1);
       }
  
       if( this->menu_handle < 0 ) {
-        printf("libspudec: No video_overlay handles left for menu\n");
+        xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+		"libspudec: No video_overlay handles left for menu\n");
         return;
       }
       this->event.object.handle = this->menu_handle;
@@ -382,28 +523,17 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
       this->overlay.rle=NULL;
       /* For force display menus */
       //if ( !(this->state.visible) ) {
-      //  this->state.visible = EVENT_SHOW_MENU;
+      //  this->state.visible = OVERLAY_EVENT_SHOW;
       //}
      
       this->event.event_type = this->state.visible;
       /*
-      printf("spu event %d handle: %d vpts: %d\n", this->event.event_type,
+      printf("spu event %d handle: %d vpts: %lli\n", this->event.event_type,
          this->event.object.handle, this->event.vpts ); 
       */
       
-      /* if !vpts then we are near a discontinuity but video_out havent detected
-         it yet and we cannot provide correct vpts values. use current_time 
-         instead as an aproximation.
-      */
-      if( this->spudec_stream_state[stream_id].vpts ) {
-        this->event.vpts = this->spudec_stream_state[stream_id].vpts+(this->state.delay*1000); 
-      } else {
-        this->event.vpts = this->xine->metronom->get_current_time(this->xine->metronom)
-                           + (this->state.delay*1000); 
-#ifdef LOG_BUTTON
-        printf("libspudec: vpts current time estimation around discontinuity\n");
-#endif
-      }
+      this->event.vpts = this->spudec_stream_state[stream_id].vpts+(this->state.delay*1000); 
+      
       /* Keep all the events in the correct order. */
       /* This corrects for errors during estimation around discontinuity */
       if( this->event.vpts < this->last_event_vpts ) {
@@ -412,12 +542,12 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
       this->last_event_vpts = this->event.vpts;
 
 #ifdef LOG_BUTTON
-      fprintf(stderr, "libspudec: add_event type=%d : current time=%lld, spu vpts=%lld\n",
+      fprintf(stderr, "libspudec: add_event type=%d : current time=%lld, spu vpts=%lli\n",
         this->event.event_type,
-        this->xine->metronom->get_current_time(this->xine->metronom),
+        this->stream->xine->clock->get_current_time(this->stream->xine->clock),
         this->event.vpts);
 #endif 
-      ovl_instance->add_event(ovl_instance, (void *)&this->event);
+      ovl_manager->add_event(ovl_manager, (void *)&this->event);
     } else {
       pending = 0;
     }
@@ -435,10 +565,11 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
 #define CMD_SPU_WIPE		0x07  /* Not currently implemented */
 #define CMD_SPU_EOF		0xff
 
-static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl)
+static void spudec_do_commands(xine_t *xine, spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl)
 {
   uint8_t *buf = state->cmd_ptr;
   uint8_t *next_seq;
+  int32_t param_length;
 
 #ifdef LOG_DEBUG
   printf ("spu: SPU DO COMMANDS\n");
@@ -467,7 +598,7 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
 #ifdef LOG_DEBUG
       printf ("spu: \tshow subpicture\n");
 #endif
-      state->visible = EVENT_SHOW_MENU;
+      state->visible = OVERLAY_EVENT_SHOW;
       buf++;
       break;
       
@@ -475,7 +606,7 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
 #ifdef LOG_DEBUG
       printf ("spu: \thide subpicture\n");
 #endif
-      state->visible = EVENT_HIDE_MENU;
+      state->visible = OVERLAY_EVENT_HIDE;
       buf++;
       break;
       
@@ -492,12 +623,6 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
       ovl->color[2] = state->clut[clut->entry1];
       ovl->color[1] = state->clut[clut->entry2];
       ovl->color[0] = state->clut[clut->entry3];
-      if ( (clut->entry0 | clut->entry1 | clut->entry2 | clut->entry3) == 0) {
-        ovl->color[3] = 0x108080; 
-        ovl->color[2] = 0x808080;
-        ovl->color[1] = 0xb08080;
-        ovl->color[0] = 0x108080;
-      }
 
 #ifdef LOG_DEBUG
       printf ("spu: \tclut [%x %x %x %x]\n",
@@ -517,14 +642,7 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
       ovl->trans[2] = trans->entry1;
       ovl->trans[1] = trans->entry2;
       ovl->trans[0] = trans->entry3;
-/**************************88
-      if ( (trans->entry0 | trans->entry1 | trans->entry2 | trans->entry3) == 0) {
-        ovl->trans[3] = 15;
-        ovl->trans[2] = 15;
-        ovl->trans[1] = 15;
-        ovl->trans[0] = 0;
-      }
-*************************/
+
 #ifdef LOG_DEBUG
       printf ("spu: \ttrans [%d %d %d %d]\n",
 	       ovl->trans[0], ovl->trans[1], ovl->trans[2], ovl->trans[3]);
@@ -545,10 +663,10 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
       ovl->y      = (buf[4]  << 4) | (buf[5] >> 4);
       ovl->width  = (((buf[2] & 0x0f) << 8) | buf[3]) - ovl->x + 1; 
       ovl->height = (((buf[5] & 0x0f) << 8) | buf[6]) - ovl->y + 1;
-      ovl->clip_top    = 0;
-      ovl->clip_bottom = ovl->height - 1;
-      ovl->clip_left   = 0;
-      ovl->clip_right  = ovl->width - 1;
+      ovl->hili_top    = -1;
+      ovl->hili_bottom = -1;
+      ovl->hili_left   = -1;
+      ovl->hili_right  = -1;
 
 #ifdef LOG_DEBUG
       printf ("spu: \tx = %d y = %d width = %d height = %d\n",
@@ -561,42 +679,50 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
     case CMD_SPU_SET_PXD_OFFSET:	/* image top[0] field / image bottom[1] field*/
       state->field_offs[0] = (((u_int)buf[1]) << 8) | buf[2];
       state->field_offs[1] = (((u_int)buf[3]) << 8) | buf[4];
+
 #ifdef LOG_DEBUG
       printf ("spu: \toffset[0] = %d offset[1] = %d\n",
 	       state->field_offs[0], state->field_offs[1]);
 #endif
+
+      if ((state->field_offs[0] >= seq->seq_len) ||
+          (state->field_offs[1] >= seq->seq_len)) {
+        xprintf(xine, XINE_VERBOSITY_DEBUG, "libspudec:faulty stream\n");
+        seq->broken = 1;
+      }
       state->modified = 1;
       buf += 5;
       break;
       
     case CMD_SPU_WIPE:
 #ifdef LOG_DEBUG
-      printf ("spu: \tSPU_WIPE not implemented yet\n");
+      printf ("libspudec: \tSPU_WIPE not implemented yet\n");
 #endif
+      param_length = (buf[1] << 8) | (buf[2]);
+      buf += 1 + param_length; 
       break;
 
     case CMD_SPU_FORCE_DISPLAY:
 #ifdef LOG_DEBUG
-      printf ("spu: \tForce Display/Menu\n");
+      printf ("libspudec: \tForce Display/Menu\n");
 #endif
       state->forced_display = 1;
       buf++;
       break;
 
     default:
-      printf("libspudec: unknown seqence command (%02x)\n", buf[0]);
+      xprintf(xine, XINE_VERBOSITY_DEBUG, "libspudec: unknown seqence command (%02x)\n", buf[0]);
       /* FIXME: SPU should be dropped, and buffers resynced */
       buf = next_seq;
+      seq->broken = 1;
       break;
     }
   }
+  
   if (next_seq >= seq->buf + seq->seq_len)
     seq->finished = 1;       /* last sub-sequence */
-  
-  state->cmd_ptr = next_seq;
-
-
 }
+
 /* FIXME: Get rid of all these static values */
 static uint8_t *bit_ptr[2];
 static int field;		// which field we are currently decoding
@@ -647,7 +773,7 @@ static int spudec_next_line (vo_overlay_t *spu)
   return 0;
 }
 
-static void spudec_draw_picture (spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl)
+static void spudec_draw_picture (xine_t *xine, spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl)
 {
   rle_elem_t *rle;
   field = 0;
@@ -661,31 +787,32 @@ static void spudec_draw_picture (spudec_state_t *state, spudec_seq_t* seq, vo_ov
  *  ovl->width  = state->o_right - state->o_left + 1;
  *  ovl->height = state->o_bottom - state->o_top + 1;
 
- *  ovl->clip_top    = 0;
- *  ovl->clip_bottom = ovl->height - 1;
- *  ovl->clip_left   = 0;
- *  ovl->clip_right  = ovl->width - 1;
+ *  ovl->hili_top    = 0;
+ *  ovl->hili_bottom = ovl->height - 1;
+ *  ovl->hili_left   = 0;
+ *  ovl->hili_right  = ovl->width - 1;
  */
 
+  /* allocate for the worst case:
+   *  - both fields running to the very end
+   *  - 2 RLE elements per byte meaning single pixel RLE
+   */
+  ovl->data_size = ((seq->cmd_offs - state->field_offs[0]) + 
+                    (seq->cmd_offs - state->field_offs[1])) * 2 * sizeof(rle_elem_t);
 
-  ovl->data_size = seq->cmd_offs * 2 * sizeof(rle_elem_t);
-#ifdef LOG_DEBUG
-  printf ("spu: MALLOC1: ovl->rle %p, len=%d\n", ovl->rle,ovl->data_size);
-#endif
   if (ovl->rle) {
-    printf ("libspudec: spudec_draw_picture: ovl->rle is not empty!!!! It should be!!! You should never see this message.\n");
+    xprintf (xine, XINE_VERBOSITY_DEBUG, 
+	     "libspudec: spudec_draw_picture: ovl->rle is not empty!!!! It should be!!! "
+	     "You should never see this message.\n");
     free(ovl->rle);
     ovl->rle=NULL;
   }
   ovl->rle = malloc(ovl->data_size);
-#ifdef LOG_DEBUG
-  printf ("spu: MALLOC2: ovl->rle %p, len=%d\n", ovl->rle,ovl->data_size);
-#endif
 
   state->modified = 0; /* mark as already processed */
   rle = ovl->rle;
 #ifdef LOG_DEBUG
-  printf ("spu: Draw RLE=%p\n",rle);
+  printf ("libspudec: Draw RLE=%p\n",rle);
 #endif
 
   while (bit_ptr[1] < seq->buf + seq->cmd_offs) {
@@ -722,6 +849,7 @@ static void spudec_draw_picture (spudec_state_t *state, spudec_seq_t* seq, vo_ov
 
   ovl->num_rle = rle - ovl->rle;
   ovl->rgb_clut = 0;
+  ovl->unscaled = 0;
 #ifdef LOG_DEBUG
   printf ("spu: Num RLE=%d\n",ovl->num_rle);
   printf ("spu: Date size=%d\n",ovl->data_size);
@@ -736,7 +864,7 @@ static void spudec_draw_picture (spudec_state_t *state, spudec_seq_t* seq, vo_ov
    MINFOUND is the number of ocurrences threshold.
 */
 #define MINFOUND 20
-static void spudec_discover_clut(spudec_state_t *state, vo_overlay_t *ovl)
+static void spudec_discover_clut(xine_t *xine, spudec_state_t *state, vo_overlay_t *ovl)
 {
   int bg,c;
   int seqcolor[10];
@@ -817,6 +945,7 @@ static void spudec_discover_clut(spudec_state_t *state, vo_overlay_t *ovl)
   }
 }
 
+#ifdef LOG_DEBUG
 static void spudec_print_overlay( vo_overlay_t *ovl ) {
   printf ("spu: OVERLAY to show\n");
   printf ("spu: \tx = %d y = %d width = %d height = %d\n",
@@ -826,47 +955,54 @@ static void spudec_print_overlay( vo_overlay_t *ovl ) {
   printf ("spu: \ttrans [%d %d %d %d]\n",
 	  ovl->trans[0], ovl->trans[1], ovl->trans[2], ovl->trans[3]);
   printf ("spu: \tclip top=%d bottom=%d left=%d right=%d\n",
-	  ovl->clip_top, ovl->clip_bottom, ovl->clip_left, ovl->clip_right);
+	  ovl->hili_top, ovl->hili_bottom, ovl->hili_left, ovl->hili_right);
   printf ("spu: \tclip_clut [%x %x %x %x]\n",
-	  ovl->clip_color[0], ovl->clip_color[1], ovl->clip_color[2], ovl->clip_color[3]);
-  printf ("spu: \tclip_trans [%d %d %d %d]\n",
-	  ovl->clip_trans[0], ovl->clip_trans[1], ovl->clip_trans[2], ovl->clip_trans[3]);
+	  ovl->hili_color[0], ovl->hili_color[1], ovl->hili_color[2], ovl->hili_color[3]);
+  printf ("spu: \thili_trans [%d %d %d %d]\n",
+	  ovl->hili_trans[0], ovl->hili_trans[1], ovl->hili_trans[2], ovl->hili_trans[3]);
   return;
-} 
-void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode,
-                                vo_overlay_t * overlay, vo_overlay_t * base ) {
-  btni_t *button_ptr;
+}
+#endif
+
+int spudec_copy_nav_to_overlay(xine_t *xine, pci_t* nav_pci, uint32_t* clut,
+			       int32_t button, int32_t mode, vo_overlay_t * overlay, vo_overlay_t * base ) {
+  btni_t *button_ptr = NULL;
+  unsigned int btns_per_group;
   int i;
 
-  /* FIXME: Need to communicate with dvdnav vm to get/set 
-    "self->vm->state.HL_BTNN_REG" info. 
-    now done via button events from dvdnav.
-   *
-   * if ( this->pci.hli.hl_gi.fosl_btnn > 0) {
-   *   button = this->pci.hli.hl_gi.fosl_btnn ;
-   * }
-   */
-  if((button <= 0) || (button > nav_pci->hli.hl_gi.btn_ns)) {
-    printf("libspudec:xine_decoder.c:Unable to select button number %i as it doesn't exist. Forcing button 1\n",
-	      button);
-    button = 1;
+  if((button <= 0) || (button > nav_pci->hli.hl_gi.btn_ns))
+    return 0;
+  
+  btns_per_group = 36 / nav_pci->hli.hl_gi.btngr_ns;
+  
+  /* choose button group: we can always use a normal 4:3 or widescreen button group
+   * as long as xine blends the overlay before scaling the image to its aspect */
+  if (!button_ptr && nav_pci->hli.hl_gi.btngr_ns >= 1 && !(nav_pci->hli.hl_gi.btngr1_dsp_ty & 6))
+    button_ptr = &nav_pci->hli.btnit[0 * btns_per_group + button - 1];
+  if (!button_ptr && nav_pci->hli.hl_gi.btngr_ns >= 2 && !(nav_pci->hli.hl_gi.btngr2_dsp_ty & 6))
+    button_ptr = &nav_pci->hli.btnit[1 * btns_per_group + button - 1];
+  if (!button_ptr && nav_pci->hli.hl_gi.btngr_ns >= 3 && !(nav_pci->hli.hl_gi.btngr3_dsp_ty & 6))
+    button_ptr = &nav_pci->hli.btnit[2 * btns_per_group + button - 1];
+  if (!button_ptr) {
+    xprintf(xine, XINE_VERBOSITY_DEBUG, 
+	    "libspudec: No suitable menu button group found, using group 1.\n");
+    button_ptr = &nav_pci->hli.btnit[button - 1];
   }
-  /* FIXME:Only the first grouping of buttons are used at the moment */
-  button_ptr = &nav_pci->hli.btnit[button-1];
+  
   /* button areas in the nav packet are in screen coordinates,
    * overlay clipping areas are in overlay coordinates;
    * therefore we must subtract the display coordinates of the underlying overlay */
-  overlay->clip_left   = (button_ptr->x_start > base->x) ? (button_ptr->x_start - base->x) : 0;
-  overlay->clip_top    = (button_ptr->y_start > base->y) ? (button_ptr->y_start - base->y) : 0;
-  overlay->clip_right  = (button_ptr->x_end   > base->x) ? (button_ptr->x_end   - base->x) : 0;
-  overlay->clip_bottom = (button_ptr->y_end   > base->y) ? (button_ptr->y_end   - base->y) : 0;
+  overlay->hili_left   = (button_ptr->x_start > base->x) ? (button_ptr->x_start - base->x) : 0;
+  overlay->hili_top    = (button_ptr->y_start > base->y) ? (button_ptr->y_start - base->y) : 0;
+  overlay->hili_right  = (button_ptr->x_end   > base->x) ? (button_ptr->x_end   - base->x) : 0;
+  overlay->hili_bottom = (button_ptr->y_end   > base->y) ? (button_ptr->y_end   - base->y) : 0;
   if(button_ptr->btn_coln != 0) {
 #ifdef LOG_BUTTON
     fprintf(stderr, "libspudec: normal button clut\n");
 #endif
     for (i = 0;i < 4; i++) {
-      overlay->clip_color[i] = clut[0xf & (nav_pci->hli.btn_colit.btn_coli[button_ptr->btn_coln-1][mode] >> (16 + 4*i))];
-      overlay->clip_trans[i] = 0xf & (nav_pci->hli.btn_colit.btn_coli[button_ptr->btn_coln-1][mode] >> (4*i));
+      overlay->hili_color[i] = clut[0xf & (nav_pci->hli.btn_colit.btn_coli[button_ptr->btn_coln-1][mode] >> (16 + 4*i))];
+      overlay->hili_trans[i] = 0xf & (nav_pci->hli.btn_colit.btn_coli[button_ptr->btn_coln-1][mode] >> (4*i));
     }
   } else {
 #ifdef LOG_BUTTON
@@ -874,10 +1010,10 @@ void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button,
 #endif
     for (i = 0;i < 4; i++) {
 #ifdef LOG_BUTTON
-      printf("libspudec:btn_coln = 0, clip_color = color\n");
+      printf("libspudec:btn_coln = 0, hili_color = color\n");
 #endif
-      overlay->clip_color[i] = overlay->color[i];
-      overlay->clip_trans[i] = overlay->trans[i];
+      overlay->hili_color[i] = overlay->color[i];
+      overlay->hili_trans[i] = overlay->trans[i];
     }
   }
 
@@ -886,5 +1022,5 @@ void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button,
   printf("libspudec:xine_decoder.c:NAV to SPU pts match!\n");
 #endif
   
+  return 1;
 }
-
