@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_sputext.c,v 1.48 2006/05/03 19:46:08 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * code based on old libsputext/xine_decoder.c
  *
@@ -131,7 +129,6 @@ static int eol(char p) {
 }
 
 static inline void trail_space(char *s) {
-  int i;
   while (isspace(*s)) {
     char *copy = s;
     do {
@@ -139,7 +136,7 @@ static inline void trail_space(char *s) {
       copy++;
     } while(*copy);
   }
-  i = strlen(s) - 1;
+  size_t i = strlen(s) - 1;
   while (i > 0 && isspace(s[i])) 
     s[i--] = '\0';
 }
@@ -264,12 +261,7 @@ static char *sub_readtext(char *source, char **dest) {
     p++,len++;
   }
   
-  *dest= (char *)xine_xmalloc (len+1);
-  if (!dest) 
-    return ERR;
-  
-  strncpy(*dest, source, len);
-  (*dest)[len]=0;
+  *dest = strndup(source, len);
   
   while (*p=='\r' || *p=='\n' || *p=='|')
     p++;
@@ -335,10 +327,8 @@ static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *cu
     p=q=line;
     for (current->lines=1; current->lines <= SUB_MAX_TEXT; current->lines++) {
       for (q=p,len=0; *p && *p!='\r' && *p!='\n' && *p!='|' && strncasecmp(p,"[br]",4); p++,len++);
-      current->text[current->lines-1]=(char *)xine_xmalloc (len+1);
+      current->text[current->lines-1] = strndup(q, len);
       if (!current->text[current->lines-1]) return ERR;
-      strncpy (current->text[current->lines-1], q, len);
-      current->text[current->lines-1][len]='\0';
       if (!*p || *p=='\r' || *p=='\n') break;
       if (*p=='[') while (*p++!=']');
       if (*p=='|') p++;
@@ -384,31 +374,6 @@ static subtitle_t *sub_read_line_subrip(demux_sputext_t *this,subtitle_t *curren
           } else
             temp_line[temp_index++]=*p;
           break;
-        case '{':
-#if 0 /* italic not implemented in renderer, ignore them for now */
-          if(!strncmp(p,"{\\i1}",5) && temp_index+3<SUB_BUFSIZE) {
-            temp_line[temp_index++]='<';
-            temp_line[temp_index++]='i';
-            temp_line[temp_index++]='>';
-#else
-          if(!strncmp(p,"{\\i1}",5)) {
-#endif
-            p+=4;
-          }
-#if 0 /* italic not implemented in renderer, ignore them for now */
-          else if(!strncmp(p,"{\\i0}",5) && temp_index+4<SUB_BUFSIZE) {
-            temp_line[temp_index++]='<';
-            temp_line[temp_index++]='/';
-            temp_line[temp_index++]='i';
-            temp_line[temp_index++]='>';
-#else
-          else if(!strncmp(p,"{\\i0}",5)) {
-#endif
-            p+=4;
-          }
-          else
-            temp_line[temp_index++]=*p;
-          break;
         case '\r': /* just ignore '\r's */
           break;
         case '\n':
@@ -423,10 +388,10 @@ static subtitle_t *sub_read_line_subrip(demux_sputext_t *this,subtitle_t *curren
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "Too many characters in a subtitle line\n");
         if(temp_line[temp_index-1]=='\0' || temp_index==SUB_BUFSIZE) {
           if(temp_index>1) { /* more than 1 char (including '\0') -> that is a valid one */
-            current->text[i]=(char *)xine_xmalloc(temp_index);
+	    /* temp_index<=SUB_BUFSIZE is always true here */
+            current->text[i] = strndup(temp_line, temp_index);
             if(!current->text[i])
               return ERR;
-            strncpy(current->text[i],temp_line,temp_index); /* temp_index<=SUB_BUFSIZE is always true here */
             i++;
             temp_index=0;
           } else
@@ -571,6 +536,8 @@ static subtitle_t *sub_read_line_ssa(demux_sputext_t *this,subtitle_t *current)
 		   line3) < 9	    );
   
   line2=strchr(line3, ',');
+  if (!line2)
+    return NULL;
   
   for (comma = 4; comma < max_comma; comma ++)
     {
@@ -590,9 +557,7 @@ static subtitle_t *sub_read_line_ssa(demux_sputext_t *this,subtitle_t *current)
   current->end   = 360000*hour2 + 6000*min2 + 100*sec2 + hunsec2;
   
   while (((tmp=strstr(line2, "\\n")) != NULL) || ((tmp=strstr(line2, "\\N")) != NULL) ){
-    current->text[num]=(char *)malloc(tmp-line2+1);
-    strncpy (current->text[num], line2, tmp-line2);
-    current->text[num][tmp-line2]='\0';
+    current->text[num] = strndup(line2, tmp-line2);
     line2=tmp+2;
     num++;
     current->lines++;
@@ -727,7 +692,7 @@ static subtitle_t *sub_read_line_aqt (demux_sputext_t *this, subtitle_t *current
   sub_readtext((char *) &line,&current->text[1]);
   current->lines = 2;
   
-  if ((current->text[0]=="") && (current->text[1]=="")) {
+  if ((current->text[0][0]==0) && (current->text[1][0]==0)) {
     return NULL;
   }
   
@@ -827,26 +792,19 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 	    ++p;
 	}
 	if (isalpha(*p)||*p == '[') {
-	    int cont, jLength;
-
 	    if (sscanf(p, "%s %" LINE_LEN_QUOT "[^\n\r]", directive, line1) < 2)
 		return ERR;
-	    jLength = strlen(directive);
-	    for (cont = 0; cont < jLength; ++cont) {
-		if (isalpha(*(directive + cont)))
-		    *(directive + cont) = toupper(*(directive + cont));
-	    }
-	    if ((strstr(directive, "RDB") != NULL)
-		|| (strstr(directive, "RDC") != NULL)
-		|| (strstr(directive, "RLB") != NULL)
-		|| (strstr(directive, "RLG") != NULL)) {
+	    if ((strcasestr(directive, "RDB") != NULL)
+		|| (strcasestr(directive, "RDC") != NULL)
+		|| (strcasestr(directive, "RLB") != NULL)
+		|| (strcasestr(directive, "RLG") != NULL)) {
 		continue;
 	    }
 	    /* no alignment */
 #if 0
-	    if (strstr(directive, "JL") != NULL) {
+	    if (strcasestr(directive, "JL") != NULL) {
 		current->alignment = SUB_ALIGNMENT_HLEFT;
-	    } else if (strstr(directive, "JR") != NULL) {
+	    } else if (strcasestr(directive, "JR") != NULL) {
 		current->alignment = SUB_ALIGNMENT_HRIGHT;
 	    } else {
 		current->alignment = SUB_ALIGNMENT_HCENTER;
@@ -915,7 +873,8 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 			return NULL;
 		    trail_space(directive);
 		    strncat(line2, directive,
-			    (LINE_LEN > 511) ? LINE_LEN : 511);
+			    ((LINE_LEN > 511) ? LINE_LEN-1 : 511)
+			    - strlen(line2));
 		    break;
 		}
 	    default:
@@ -926,7 +885,10 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 	    }
 	}
 	*q = '\0';
-	current->text[current->lines] = strdup(line1);
+        if (current->lines < SUB_MAX_TEXT)
+	    current->text[current->lines] = strdup(line1);
+        else
+            xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "Too many lines in a subtitle\n");
     }
     current->lines++;
     return current;
@@ -951,10 +913,9 @@ static subtitle_t *sub_read_line_subviewer2(demux_sputext_t *this, subtitle_t *c
             len=0;
             for (p=line; *p!='\n' && *p!='\r' && *p; ++p,++len);
             if (len) {
-                current->text[i]=(char *)malloc (len+1);
-                if (!current->text[i]) return ERR;
-                strncpy (current->text[i], line, len); current->text[i][len]='\0';
-                ++i;
+	      current->text[i] = strndup(line, len);
+	      if (!current->text[i]) return ERR;
+	      ++i;
             } else {
                 break;
             }
@@ -1186,7 +1147,7 @@ static subtitle_t *sub_read_file (demux_sputext_t *this) {
   this->buflen = 0;
 
   this->num=0;n_max=32;
-  first = (subtitle_t *) xine_xmalloc(n_max*sizeof(subtitle_t));
+  first = calloc(n_max, sizeof(subtitle_t));
   if(!first) return NULL;
   timeout = ((demux_sputext_class_t *)
              (this->demux_plugin.demux_class))->max_timeout;
@@ -1271,9 +1232,8 @@ static int demux_sputext_next (demux_sputext_t *this_gen) {
   *val++ = (this->uses_time) ? sub->end * 10 : sub->end;
   str = (char *)val;
   for (line = 0; line < sub->lines; line++, str+=strlen(str)+1) {
-    if( strlen(sub->text[line]) > SUB_BUFSIZE )
-      sub->text[line][SUB_BUFSIZE] = '\0';
-    strcpy(str, sub->text[line]);
+    strncpy(str, sub->text[line], SUB_BUFSIZE-1);
+    str[SUB_BUFSIZE-1] = '\0';
   }
   
   this->stream->video_fifo->put(this->stream->video_fifo, buf);
@@ -1384,7 +1344,7 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
 
   lprintf("open_plugin() called\n");
   
-  this = xine_xmalloc (sizeof (demux_sputext_t));
+  this = calloc(1, sizeof (demux_sputext_t));
   this->stream = stream;
   this->input  = input;
 
@@ -1403,10 +1363,8 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
   switch (stream->content_detection_method) {
   case METHOD_BY_EXTENSION:
     {
-      char *mrl, *ending;
-      
-      mrl = input->get_mrl(input);
-      ending = strrchr(mrl, '.');
+      const char *const mrl = input->get_mrl(input);
+      const char *const ending = strrchr(mrl, '.');
 
       if (!ending || (
 	  (strncasecmp(ending, ".asc", 4) != 0) &&
@@ -1451,19 +1409,19 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
   return NULL;
 }
   
-static char *get_demux_description (demux_class_t *this_gen) {
+static const char *get_demux_description (demux_class_t *this_gen) {
   return "sputext demuxer plugin";
 }
 
-static char *get_demux_identifier (demux_class_t *this_gen) {
+static const char *get_demux_identifier (demux_class_t *this_gen) {
   return "sputext";
 }
 
-static char *get_demux_extensions (demux_class_t *this_gen) {
+static const char *get_demux_extensions (demux_class_t *this_gen) {
   return "asc txt sub srt smi ssa";
 }
 
-static char *get_demux_mimetypes (demux_class_t *this_gen) {
+static const char *get_demux_mimetypes (demux_class_t *this_gen) {
   return NULL;
 
   /* do not report this mimetype, it might confuse browsers. */
@@ -1488,7 +1446,7 @@ static void *init_sputext_demux_class (xine_t *xine, void *data) {
 
   lprintf("initializing\n");
 
-  this = xine_xmalloc (sizeof (demux_sputext_class_t));
+  this = calloc(1, sizeof (demux_sputext_class_t));
 
   this->demux_class.open_plugin     = open_demux_plugin;
   this->demux_class.get_description = get_demux_description;
@@ -1513,7 +1471,7 @@ static void *init_sputext_demux_class (xine_t *xine, void *data) {
   return this;
 }
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_DEMUX, 26, "sputext", XINE_VERSION_CODE, NULL, &init_sputext_demux_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
