@@ -15,14 +15,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: w32codec.c,v 1.154 2006/06/11 15:09:47 valtri Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * routines for using w32 codecs
  * DirectShow support by Miguel Freitas (Nov/2001)
  * DMO support (Dez/2002)
- *
  */
                    
 #include <stdlib.h>
@@ -657,7 +654,7 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
   this->bufsize = VIDEOBUFSIZE;
   this->buf = malloc(this->bufsize);
 
-  this->stream->video_out->open (this->stream->video_out, this->stream);
+  (this->stream->video_out->open) (this->stream->video_out, this->stream);
 
   this->outfmt = outfmt;
   this->decoder_ok = 1;
@@ -752,7 +749,7 @@ static void w32v_init_ds_dmo_codec (w32v_decoder_t *this, int buf_type) {
   this->bufsize = VIDEOBUFSIZE;
   this->buf = malloc(this->bufsize);
   
-  this->stream->video_out->open (this->stream->video_out, this->stream);
+  (this->stream->video_out->open) (this->stream->video_out, this->stream);
 
   this->outfmt = outfmt;
   this->decoder_ok = 1;
@@ -1181,7 +1178,7 @@ static int w32a_init_audio (w32a_decoder_t *this,
   if (this->output_open)
     this->stream->audio_out->close (this->stream->audio_out, this->stream);
 
-  this->output_open = this->stream->audio_out->open( this->stream->audio_out, this->stream,
+  this->output_open = (this->stream->audio_out->open) ( this->stream->audio_out, this->stream,
 					      16, in_fmt->nSamplesPerSec, 
 					      _x_ao_channels2mode(in_fmt->nChannels));
   if (!this->output_open) {
@@ -1553,7 +1550,7 @@ static video_decoder_t *open_video_decoder_plugin (video_decoder_class_t *class_
 
   w32v_decoder_t *this ;
 
-  this = (w32v_decoder_t *) xine_xmalloc (sizeof (w32v_decoder_t));
+  this = (w32v_decoder_t *) calloc(1, sizeof(w32v_decoder_t));
 
   this->video_decoder.decode_data         = w32v_decode_data;
   this->video_decoder.flush               = w32v_flush;
@@ -1596,7 +1593,7 @@ static void *init_video_decoder_class (xine_t *xine, void *data) {
   cfg = xine->config;
   if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
 
-  this = (w32v_class_t *) xine_xmalloc (sizeof (w32v_class_t));
+  this = (w32v_class_t *) calloc(1, sizeof(w32v_class_t));
 
   this->decoder_class.open_plugin     = open_video_decoder_plugin;
   this->decoder_class.get_identifier  = get_video_identifier;
@@ -1617,7 +1614,7 @@ static audio_decoder_t *open_audio_decoder_plugin (audio_decoder_class_t *class_
 
   w32a_decoder_t *this ;
 
-  this = (w32a_decoder_t *) xine_xmalloc (sizeof (w32a_decoder_t));
+  this = (w32a_decoder_t *) calloc(1, sizeof(w32a_decoder_t));
 
   this->audio_decoder.decode_data         = w32a_decode_data;
   this->audio_decoder.reset               = w32a_reset;
@@ -1658,7 +1655,7 @@ static void *init_audio_decoder_class (xine_t *xine, void *data) {
   cfg = xine->config;
   if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
 
-  this = (w32a_class_t *) xine_xmalloc (sizeof (w32a_class_t));
+  this = (w32a_class_t *) calloc(1, sizeof(w32a_class_t));
 
   this->decoder_class.open_plugin     = open_audio_decoder_plugin;
   this->decoder_class.get_identifier  = get_identifier;
@@ -1704,7 +1701,7 @@ static const decoder_info_t dec_info_audio = {
   1                    /* priority        */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 18, "win32v", XINE_VERSION_CODE, &dec_info_video, init_video_decoder_class },
   { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 15, "win32a", XINE_VERSION_CODE, &dec_info_audio, init_audio_decoder_class },
