@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: audio_filters.c,v 1.3 2005/07/16 16:36:59 miguelfreitas Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * catalog for audio filter plugins
  */
@@ -30,15 +28,17 @@
 #include "audio_filters.h"
 
 
-post_info_t upmix_special_info      = { XINE_POST_TYPE_AUDIO_FILTER };
-post_info_t upmix_mono_special_info = { XINE_POST_TYPE_AUDIO_FILTER };
-post_info_t stretch_special_info    = { XINE_POST_TYPE_AUDIO_FILTER };
+static const post_info_t upmix_special_info      = { XINE_POST_TYPE_AUDIO_FILTER };
+static const post_info_t upmix_mono_special_info = { XINE_POST_TYPE_AUDIO_FILTER };
+static const post_info_t stretch_special_info    = { XINE_POST_TYPE_AUDIO_FILTER };
+static const post_info_t volnorm_special_info    = { XINE_POST_TYPE_AUDIO_FILTER };
 
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_POST, 9, "upmix",      XINE_VERSION_CODE, &upmix_special_info,      &upmix_init_plugin },
   { PLUGIN_POST, 9, "upmix_mono", XINE_VERSION_CODE, &upmix_mono_special_info, &upmix_mono_init_plugin },
   { PLUGIN_POST, 9, "stretch",    XINE_VERSION_CODE, &stretch_special_info,    &stretch_init_plugin },
+  { PLUGIN_POST, 9, "volnorm",    XINE_VERSION_CODE, &volnorm_special_info,    &volnorm_init_plugin },
   { PLUGIN_NONE, 0, "",           0,                 NULL,                     NULL }
 };
