@@ -16,9 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xxmc.h,v 1.9 2005/09/25 00:44:04 miguelfreitas Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * video_out_xxmc.c, X11 decoding accelerated video extension interface for xine
  *
@@ -31,7 +29,7 @@
  *
  * overlay support by James Courtier-Dutton <James@superbug.demon.co.uk> - July 2001
  * X11 unscaled overlay support by Miguel Freitas - Nov 2003
- * XxMC implementation by Thomas Hellstrm - August 2004
+ * XxMC implementation by Thomas Hellström - August 2004
  */
 
 #ifndef _XXMC_H
@@ -205,6 +203,9 @@ struct xxmc_driver_s {
   xxmc_frame_t       *cur_frame;
   int                cur_field;
   int                bob;
+  int                disable_bob_for_progressive_frames;
+  int                disable_bob_for_scaled_osd;
+  int                scaled_osd_active;
   x11osd             *xoverlay;
   int                xv_xoverlay_type;
   int                xoverlay_type;
