@@ -21,9 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * $Id: yuv2rgb.c,v 1.51 2004/04/11 12:25:35 komadori Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #include "config.h"
@@ -1276,7 +1274,7 @@ static scale_line_func_t find_scale_line_func(int step) {
     {  1,  2, scale_line_1_2,   "2*zoom" },
     {  1,  1, scale_line_1_1,   "non-scaled" },
   };
-  int i;
+  size_t i;
 #ifdef	LOG
   /* to filter out multiple messages about the scale_line variant we're using */
   static int reported_for_step;
@@ -3146,7 +3144,7 @@ static void yuy22rgb_c_init (yuv2rgb_factory_t *this)
 
 static yuv2rgb_t *yuv2rgb_create_converter (yuv2rgb_factory_t *factory) {
 
-  yuv2rgb_t *this = xine_xmalloc (sizeof (yuv2rgb_t));
+  yuv2rgb_t *this = calloc(1, sizeof(yuv2rgb_t));
 
   this->swapped			 = factory->swapped;
   this->cmap                     = factory->cmap;
@@ -3222,7 +3220,7 @@ yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped,
 
 #ifdef LOG
     if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using MMXEXT for colorspace transform\n");
+      printf ("yuv2rgb: using MMXEXT for colour space transform\n");
 #endif
   }
 
@@ -3232,7 +3230,7 @@ yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped,
 
 #ifdef LOG
     if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using MMX for colorspace transform\n");
+      printf ("yuv2rgb: using MMX for colour space transform\n");
 #endif
   }
 #endif
@@ -3243,12 +3241,12 @@ yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped,
 
 #ifdef LOG
     if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using medialib for colorspace transform\n");
+      printf ("yuv2rgb: using medialib for colour space transform\n");
 #endif
   }
 #endif
   if (this->yuv2rgb_fun == NULL) {
-    lprintf ("no accelerated colorspace conversion found\n");
+    lprintf ("no accelerated colour space conversion found\n");
 
     yuv2rgb_c_init (this);
   }
