@@ -258,6 +258,7 @@ extern "C" {
 #define BUF_AUDIO_SMACKER	0x033B0000
 #define BUF_AUDIO_FLVADPCM	0x033C0000
 #define BUF_AUDIO_WAVPACK	0x033D0000
+#define BUF_AUDIO_MP3ADU	0x033E0000
 
 /* spu buffer types:    */
 
@@ -606,8 +607,8 @@ struct fifo_buffer_s
  * allocate num_buffers of buf_size bytes each
  */
 
-fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_PROTECTED;
-fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_PROTECTED;
+fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_MALLOC XINE_PROTECTED;
+fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_MALLOC XINE_PROTECTED;
 
 
 /* return BUF_VIDEO_xxx given the fourcc
@@ -626,7 +627,7 @@ uint32_t _x_formattag_to_buf_audio( uint32_t formattag ) XINE_PROTECTED;
 char * _x_buf_audio_name( uint32_t buf_type ) XINE_PROTECTED;
 
 
-#ifndef ATTRIBUTE_PACKED
+#ifndef SUPPORT_ATTRIBUTE_PACKED
 /* no attribute packed? let's try with pragma pack as a last resort */
 #pragma pack(2)
 #endif
@@ -635,7 +636,7 @@ char * _x_buf_audio_name( uint32_t buf_type ) XINE_PROTECTED;
  * - will always use machine endian format, so demuxers reading
  *   stuff from win32 formats must use the function below.
  */
-typedef struct __attribute__((__packed__)) {
+typedef struct XINE_PACKED {
     int32_t        biSize;
     int32_t        biWidth;
     int32_t        biHeight;
@@ -652,7 +653,7 @@ typedef struct __attribute__((__packed__)) {
 /* this is xine version of WAVEFORMATEX 
  * (the same comments from xine_bmiheader)
  */
-typedef struct __attribute__((__packed__)) {
+typedef struct XINE_PACKED {
   int16_t   wFormatTag;
   int16_t   nChannels;
   int32_t   nSamplesPerSec;
@@ -661,7 +662,7 @@ typedef struct __attribute__((__packed__)) {
   int16_t   wBitsPerSample;
   int16_t   cbSize;
 } xine_waveformatex;
-#ifndef ATTRIBUTE_PACKED
+#ifndef SUPPORT_ATTRIBUTE_PACKED
 #pragma pack()
 #endif
 
