@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * OSD stuff (text and graphic primitives)
  */
@@ -57,6 +57,9 @@
 #ifdef HAVE_FT2
 #include <ft2build.h>
 #include FT_FREETYPE_H
+# ifdef HAVE_FONTCONFIG
+#  include <fontconfig/fontconfig.h>
+# endif
 #endif
 
 #define FONT_VERSION  2
@@ -80,17 +83,6 @@
 #  define UCS2_ENCODING "UCS-2LE"
 #endif
 
-#ifdef MAX
-#undef MAX
-#endif
-#define MAX(a,b) ( (a) > (b) ) ? (a) : (b)
-
-#ifdef MIN
-#undef MIN
-#endif
-#define MIN(a,b) ( (a) < (b) ) ? (a) : (b)
-
-
 #if (FREETYPE_MAJOR > 2) || \
     (FREETYPE_MAJOR == 2 && FREETYPE_MINOR > 1) || \
     (FREETYPE_MAJOR == 2 && FREETYPE_MINOR == 1 && FREETYPE_PATCH >= 3)
@@ -99,27 +91,32 @@
 #  define KERNING_DEFAULT ft_kerning_default
 #endif
 
+#ifdef ENABLE_ANTIALIASING
+#  define FT_LOAD_FLAGS   FT_LOAD_DEFAULT
+#else
+#  define FT_LOAD_FLAGS  (FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING)
+#endif
+
 typedef struct osd_fontchar_s {
+  uint8_t *bmp;
   uint16_t code;
   uint16_t width;
   uint16_t height;
-  uint8_t *bmp;
 } osd_fontchar_t;
 
 struct osd_font_s {
   char             name[40];
   char            *filename;
+  osd_fontchar_t  *fontchar;
+  osd_font_t      *next;
   uint16_t         version;
   uint16_t         size;
   uint16_t         num_fontchars;
-  osd_fontchar_t  *fontchar;
-  osd_font_t      *next;
-  int              loaded;
+  uint16_t         loaded;
 }; 
 
 #ifdef HAVE_FT2
 struct osd_ft2context_s {
-  int        useme;
   FT_Library library;
   FT_Face    face;
   int        size;
@@ -135,13 +132,13 @@ struct osd_ft2context_s {
  * for the sake of simplicity)
  */
 
-static osd_object_t *osd_new_object (osd_renderer_t *this, int width, int height) {
+static osd_object_t *XINE_MALLOC osd_new_object (osd_renderer_t *this, int width, int height) {
      
   osd_object_t *osd;
   
   pthread_mutex_lock (&this->osd_mutex);  
   
-  osd = xine_xmalloc( sizeof(osd_object_t) );
+  osd = calloc(1, sizeof(osd_object_t));
   osd->renderer = this;
   osd->next = this->osds;
   this->osds = osd;
@@ -189,10 +186,10 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
   osd_renderer_t *this = osd->renderer;
   video_overlay_manager_t *ovl_manager;
   rle_elem_t rle, *rle_p=0;
-  int x, y, required;
+  int x, y;
   uint8_t *c;
 
-  lprintf("osd=%p vpts=%lld\n", osd, vpts);
+  lprintf("osd=%p vpts=%"PRId64"\n", osd, vpts);
   
   this->stream->xine->port_ticket->acquire(this->stream->xine->port_ticket, 1);
   
@@ -243,11 +240,11 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
     this->event.object.overlay->hili_right  = this->event.object.overlay->width;
    
     /* there will be at least that many rle objects (one for each row) */
-    required = osd->y2 - osd->y1;
     this->event.object.overlay->num_rle = 0;
-    this->event.object.overlay->data_size = 1024;
-    while (required > this->event.object.overlay->data_size)
-      this->event.object.overlay->data_size += 1024;
+    /* We will never need more rle objects than columns in any row
+       Rely on lazy page allocation to avoid us actually taking up
+       this much RAM */
+    this->event.object.overlay->data_size = osd->width * osd->height;
     rle_p = this->event.object.overlay->rle = 
        malloc(this->event.object.overlay->data_size * sizeof(rle_elem_t) );
     
@@ -264,14 +261,6 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
       /* loop over the remaining pixels in the row */
       for( x = osd->x1 + rle.len; x < osd->x2; x++, c++ ) {
         if( rle.color != *c ) {
-          if( (this->event.object.overlay->num_rle + required) > 
-              this->event.object.overlay->data_size ) {
-            this->event.object.overlay->data_size += 1024;
-            rle_p = this->event.object.overlay->rle = 
-              realloc( this->event.object.overlay->rle,
-                       this->event.object.overlay->data_size * sizeof(rle_elem_t) );
-            rle_p += this->event.object.overlay->num_rle;
-          }
 #ifdef DEBUG_RLE          
           lprintf("(%d, %d), ", rle.len, rle.color);
 #endif
@@ -289,8 +278,6 @@ static int _osd_show (osd_object_t *osd, int64_t vpts, int unscaled ) {
 #endif
       *rle_p++ = rle;
       this->event.object.overlay->num_rle++;
-      /* another row done */
-      required--;
     }
 #ifdef DEBUG_RLE
     lprintf("osd_show %p rle ends\n", osd);
@@ -339,7 +326,7 @@ static int _osd_hide (osd_object_t *osd, int64_t vpts) {
   osd_renderer_t *this = osd->renderer;
   video_overlay_manager_t *ovl_manager;
   
-  lprintf("osd=%p vpts=%lld\n",osd, vpts);
+  lprintf("osd=%p vpts=%"PRId64"\n",osd, vpts);
       
   if( osd->handle < 0 )
     return 0;
@@ -400,6 +387,11 @@ static void osd_point (osd_object_t *osd, int x, int y, int color) {
   
   lprintf("osd=%p (%d x %d)\n", osd, x, y);
   
+  if (x < 0 || x >= osd->width)
+    return;
+  if (y < 0 || y >= osd->height)
+    return;
+  
   /* update clipping area */
   osd->x1 = MIN(osd->x1, x);
   osd->x2 = MAX(osd->x2, (x + 1));
@@ -407,8 +399,7 @@ static void osd_point (osd_object_t *osd, int x, int y, int color) {
   osd->y2 = MAX(osd->y2, (y + 1));
 
   c = osd->area + y * osd->width + x;
-  if(c <= (osd->area + (osd->width * osd->height)))
-    *c = color;
+  *c = color;
 }
 
 /*
@@ -420,21 +411,66 @@ static void osd_line (osd_object_t *osd,
      
   uint8_t *c;
   int dx, dy, t, inc, d, inc1, inc2;
+  int swap_x = 0;
+  int swap_y = 0;
 
   lprintf("osd=%p (%d,%d)-(%d,%d)\n",osd, x1,y1, x2,y2 );
-     
-  /* update clipping area */
-  t = MIN( x1, x2 );
-  osd->x1 = MIN( osd->x1, t );
-  t = MAX( x1, x2 );
-  osd->x2 = MAX( osd->x2, t );
-  t = MIN( y1, y2 );
-  osd->y1 = MIN( osd->y1, t );
-  t = MAX( y1, y2 );
-  osd->y2 = MAX( osd->y2, t );
+  
+  /* sort line */
+  if (x2 < x1) {
+    t  = x1;
+    x1 = x2;
+    x2 = t;
+    swap_x = 1;
+  }
+  if (y2 < y1) {
+    t  = y1;
+    y1 = y2;
+    y2 = t;
+    swap_y = 1;
+  }
 
-  dx = abs(x1-x2);
-  dy = abs(y1-y2);
+  /* clip line */ 
+  if (x1 < 0) {
+    y1 = y1 + (y2-y1) * -x1 / (x2-x1);
+    x1 = 0;
+  }
+  if (y1 < 0) {
+    x1 = x1 + (x2-x1) * -y1 / (y2-y1);
+    y1 = 0;
+  }
+  if (x2 > osd->width) {
+    y2 = y1 + (y2-y1) * (osd->width-x1) / (x2-x1);
+    x2 = osd->width;
+  }
+  if (y2 > osd->height) {
+    x2 = x1 + (x2-x1) * (osd->height-y1) / (y2-y1);
+    y2 = osd->height;
+  }
+  
+  if (x1 >= osd->width || y1 >= osd->height)
+    return;
+ 
+  /* update clipping area */
+  osd->x1 = MIN( osd->x1, x1 );
+  osd->x2 = MAX( osd->x2, x2 );
+  osd->y1 = MIN( osd->y1, y1 );
+  osd->y2 = MAX( osd->y2, y2 );
+  
+  dx = x2 - x1;
+  dy = y2 - y1;
+  
+  /* unsort line */
+  if (swap_x) {
+    t  = x1;
+    x1 = x2;
+    x2 = t;
+  }
+  if (swap_y) {
+    t  = y1;
+    y1 = y2;
+    y2 = t;
+  }
 
   if( dx>=dy ) {
     if( x1>x2 )
@@ -453,11 +489,7 @@ static void osd_line (osd_object_t *osd,
     
     while(x1<x2)
     {
-
-      if(c <= (osd->area + (osd->width * osd->height)))
-	*c = color;
-
-      c++;
+      *c++ = color;
       
       x1++;
       if( d<0 ) {
@@ -483,9 +515,7 @@ static void osd_line (osd_object_t *osd,
     c = osd->area + y1 * osd->width + x1;
 
     while(y1<y2) {
-
-      if(c <= (osd->area + (osd->width * osd->height)))
-	*c = color;
+      *c = color; 
       
       c += osd->width;
       y1++;
@@ -511,15 +541,33 @@ static void osd_filled_rect (osd_object_t *osd,
   int x, y, dx, dy;
 
   lprintf("osd=%p (%d,%d)-(%d,%d)\n",osd, x1,y1, x2,y2 );
-
-  /* update clipping area */
-  x = MIN( x1, x2 );
-  osd->x1 = MIN( osd->x1, x );
+  
+  /* sort rectangle */
+  x  = MIN( x1, x2 );
   dx = MAX( x1, x2 );
-  osd->x2 = MAX( osd->x2, dx );
-  y = MIN( y1, y2 );
-  osd->y1 = MIN( osd->y1, y );
+  y  = MIN( y1, y2 );
   dy = MAX( y1, y2 );
+  
+  /* clip rectangle */
+  if (x >= osd->width || y >= osd->height)
+    return;
+ 
+  if (x < 0) {
+    dx += x;
+    x = 0;
+  }
+  if (y < 0) {
+    dy += y;
+    y = 0;
+  }
+
+  dx = MIN( dx, osd->width );
+  dy = MIN( dy, osd->height );
+   
+  /* update clipping area */
+  osd->x1 = MIN( osd->x1, x );
+  osd->x2 = MAX( osd->x2, dx );
+  osd->y1 = MIN( osd->y1, y );
   osd->y2 = MAX( osd->y2, dy );
 
   dx -= x;
@@ -611,7 +659,7 @@ static int osd_renderer_load_font(osd_renderer_t *this, char *filename) {
   /* fixme: check for all read errors... */
   if( (fp = gzopen(filename,"rb")) != NULL ) {
 
-    font = xine_xmalloc( sizeof(osd_font_t) );
+    font = calloc(1, sizeof(osd_font_t));
 
     gzread(fp, font->name, sizeof(font->name) );
     font->version = gzread_i16(fp);
@@ -756,97 +804,137 @@ static int osd_renderer_unload_font(osd_renderer_t *this, char *fontname ) {
   return ret;
 }
 
+#ifdef HAVE_FT2
+static int osd_set_font_freetype2( osd_object_t *osd, const char *fontname, int size ) {
+  if (!osd->ft2) {
+    osd->ft2 = calloc(1, sizeof(osd_ft2context_t));
+    if(FT_Init_FreeType( &osd->ft2->library )) {
+      xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG,
+	      _("osd: cannot initialize ft2 library\n"));
+      free(osd->ft2);
+      osd->ft2 = NULL;
+      return 0;
+    }
+  }
+   
+#ifdef HAVE_FONTCONFIG
+  do {
+    FcPattern *pat = NULL, *match = NULL;
+    FcFontSet *fs = FcFontSetCreate();
+    FcResult result;
+
+    pat = FcPatternBuild(NULL, FC_FAMILY, FcTypeString, fontname, FC_SIZE, FcTypeDouble, (double)size, NULL);
+    FcConfigSubstitute(NULL, pat, FcMatchPattern);
+    FcDefaultSubstitute(pat);
+
+    match = FcFontMatch(NULL, pat, &result);
+    FcPatternDestroy(pat);
+    
+    if ( ! match ) {
+      FcFontSetDestroy(fs);
+      xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG,
+	      _("osd: error matching font %s with FontConfig"), fontname);
+      break;
+    }
+    FcFontSetAdd(fs, match);
+
+    if ( fs->nfont != 0 ) {
+      FcChar8 *filename = NULL;
+      FcPatternGetString(fs->fonts[0], FC_FILE, 0, &filename);
+      if ( ! FT_New_Face(osd->ft2->library, (const char*)filename, 0, &osd->ft2->face) ) {
+	FcFontSetDestroy(fs);
+	goto end;
+      }
+
+      xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG,
+	      _("osd: error loading font %s with FontConfig"), fontname);
+    } else {
+      xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG,
+	      _("osd: error looking up font %s with FontConfig"), fontname);
+    }
+  } while(0);
+#endif
+  {
+    char pathname[1024];
+    /* try load font from current directory */
+    if ( !FT_New_Face(osd->ft2->library, fontname, 0, &osd->ft2->face) )
+      goto end;
+    
+    /* try load font from home directory */
+    snprintf(pathname, 1024, "%s/.xine/fonts/%s", xine_get_homedir(), fontname);
+    if ( !FT_New_Face(osd->ft2->library, pathname, 0, &osd->ft2->face) )
+      goto end;
+
+    /* try load font from xine font directory */
+    snprintf(pathname, 1024, "%s/%s", XINE_FONTDIR, fontname);
+    if ( !FT_New_Face(osd->ft2->library, pathname, 0, &osd->ft2->face) )
+      goto end;
+
+    xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG, 
+	    _("osd: error loading font %s with ft2\n"), fontname);
+  }
+
+  free(osd->ft2);
+  osd->ft2 = NULL;
+  return 0;
+
+ end:
+  if (FT_Set_Pixel_Sizes(osd->ft2->face, 0, size)) {
+    xprintf(osd->renderer->stream->xine, XINE_VERBOSITY_LOG,
+	    _("osd: error setting font size (no scalable font?)\n"));
+    free(osd->ft2);
+    osd->ft2 = NULL;
+    return 0;
+  }
+
+  osd->ft2->size = size;
+  return 1;
+}
+#endif
 
 /*
   set the font of osd object
 */
 
 static int osd_set_font( osd_object_t *osd, const char *fontname, int size) { 
-
-  osd_renderer_t *this = osd->renderer;
-  osd_font_t *font;
-  int best = 0;
-  int ret = 0;
-#ifdef HAVE_FT2
-  char pathname[1024];
-  int error_flag = 0;
-#endif
+  int ret = 1;
 
   lprintf("osd=%p font '%s'\n", osd, fontname);
  
-  pthread_mutex_lock (&this->osd_mutex);
+  pthread_mutex_lock (&osd->renderer->osd_mutex);
 
-  osd->font = NULL;
-
-  font = this->fonts;
-  while( font ) {
-
-    if( !strcasecmp(font->name, fontname) && (size>=font->size) 
-	&& (best<font->size)) {
-      ret = 1;
-      osd->font = font;
-      best = font->size;
-      lprintf ("best: font->name=%s, size=%d\n", font->name, font->size);
-
-    }
-    font = font->next;
-  }
-
-  if( ret ) {
-    /* load font if needed */
-    if( !osd->font->loaded )
-      ret = osd_renderer_load_font(this, osd->font->filename);
-    if(!ret)
-      osd->font = NULL;
-  }
-      
 #ifdef HAVE_FT2
-
-  if (osd->ft2) {
-    osd->ft2->useme = 0;
-  }
-
-  if (!ret) { /* trying to load a font file with ft2 */
-    if (!osd->ft2) {
-      osd->ft2 = xine_xmalloc(sizeof(osd_ft2context_t));
-      if(FT_Init_FreeType( &osd->ft2->library )) {
-        xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-		_("osd: cannot initialize ft2 library\n"));
-	free(osd->ft2);
-	osd->ft2 = NULL;
-      }
-    }
-    if (osd->ft2) {
-      /* try load font from current directory */
-      if (FT_New_Face(osd->ft2->library, fontname, 0, &osd->ft2->face)) {
-        /* try load font from home directory */
-        snprintf(pathname, 1024, "%s/.xine/fonts/%s", xine_get_homedir(), fontname);
-        if (FT_New_Face(osd->ft2->library, pathname, 0, &osd->ft2->face)) {
-          /* try load font from xine font directory */
-          snprintf(pathname, 1024, "%s/%s", XINE_FONTDIR, fontname);
-          if (FT_New_Face(osd->ft2->library, pathname, 0, &osd->ft2->face)) {
-            error_flag = 1;
-	    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
-		    _("osd: error loading font %s with ft2\n"), fontname);
-	  }
-        }
-      }
-      if (!error_flag) {
-	if (FT_Set_Pixel_Sizes(osd->ft2->face, 0, size)) {
-	  xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-		  _("osd: error setting font size (no scalable font?)\n"));
-	} else {
-	  ret = 1;
-	  osd->ft2->useme = 1;
-	  osd->ft2->size = size;
-	}
-      }
-    }	
-  }
-
+  if ( ! osd_set_font_freetype2(osd, fontname, size) )
 #endif
+    { /* If the FreeType2 loading failed */
+      osd_font_t *font;
+      int best = 0;
+      osd->font = NULL;
+      ret = 0;
 
-  pthread_mutex_unlock (&this->osd_mutex);
+      font = osd->renderer->fonts;
+      while( font ) {
+
+	if( !strcasecmp(font->name, fontname) && (size>=font->size) 
+	    && (best<font->size)) {
+	  ret = 1;
+	  osd->font = font;
+	  best = font->size;
+	  lprintf ("best: font->name=%s, size=%d\n", font->name, font->size);
+	}
+	font = font->next;
+      }
+
+      if( ret ) {
+	/* load font if needed */
+	if( !osd->font->loaded )
+	  ret = osd_renderer_load_font(osd->renderer, osd->font->filename);
+	if(!ret)
+	  osd->font = NULL;
+      }      
+    }
+
+  pthread_mutex_unlock (&osd->renderer->osd_mutex);
   return ret;
 }
 
@@ -896,7 +984,8 @@ static int osd_search(osd_fontchar_t *array, size_t n, uint16_t code) {
  * get next unicode value 
  */
 static uint16_t osd_iconv_getunicode(xine_t *xine, 
-				     iconv_t *cd, const char *encoding, char **inbuf, size_t *inbytesleft) {
+				     iconv_t cd, const char *encoding, ICONV_CONST char **inbuf,
+				     size_t *inbytesleft) {
   uint16_t unicode;
   char *outbuf = (char*)&unicode;
   size_t outbytesleft = 2;
@@ -967,7 +1056,7 @@ static int osd_set_encoding (osd_object_t *osd, const char *encoding) {
 	      _("osd: can't find out current locale character set\n"));
       return 0;
     }
-    lprintf("locale encoding='%s'\n", osd, enc);
+    lprintf("locale encoding='%s'\n", enc);
   } else
     enc = strdup(encoding);
 
@@ -1006,7 +1095,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
 
 #ifdef HAVE_FT2
   FT_UInt previous = 0;
-  FT_Bool use_kerning = osd->ft2 && osd->ft2->useme && FT_HAS_KERNING(osd->ft2->face);
+  FT_Bool use_kerning = osd->ft2 && FT_HAS_KERNING(osd->ft2->face);
   int first = 1;
 #endif
 
@@ -1025,7 +1114,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
 
     if ((font = osd->font)) proceed = 1;
 #ifdef HAVE_FT2
-    if (osd->ft2 && osd->ft2->useme) proceed = 1;
+    if (osd->ft2) proceed = 1;
 #endif
     
     if (proceed == 0) {
@@ -1044,7 +1133,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
   while( inbytesleft ) {
 #ifdef HAVE_ICONV
     unicode = osd_iconv_getunicode(this->stream->xine, osd->cd, osd->encoding, 
-                                   (char **)&inbuf, &inbytesleft);
+                                   (ICONV_CONST char **)&inbuf, &inbytesleft);
 #else
     unicode = inbuf[0];
     inbuf++;
@@ -1053,7 +1142,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
 
 
 #ifdef HAVE_FT2
-    if (osd->ft2 && osd->ft2->useme) {
+    if (osd->ft2) {
 
       FT_GlyphSlot slot = osd->ft2->face->glyph;
 
@@ -1067,7 +1156,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
       }
       previous = i;
 
-      if (FT_Load_Glyph(osd->ft2->face, i, FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING)) {
+      if (FT_Load_Glyph(osd->ft2->face, i, FT_LOAD_FLAGS)) {
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG, _("osd: error loading glyph\n"));
         continue;
       }
@@ -1155,7 +1244,7 @@ static int osd_render_text (osd_object_t *osd, int x1, int y1,
       }
     
 #ifdef HAVE_FT2
-    } /* !(osd->ft2 && osd->ft2->useme) */
+    } /* !(osd->ft2) */
 #endif
 
   }
@@ -1179,7 +1268,7 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
 
 #ifdef HAVE_FT2
   /* not all free type fonts provide kerning */
-  FT_Bool use_kerning = osd->ft2 && osd->ft2->useme && FT_HAS_KERNING(osd->ft2->face);
+  FT_Bool use_kerning = osd->ft2 && FT_HAS_KERNING(osd->ft2->face);
   FT_UInt previous = 0;
   int first_glyph = 1;
 #endif
@@ -1193,7 +1282,7 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
 
     if ((font = osd->font)) proceed = 1;
 #ifdef HAVE_FT2
-    if (osd->ft2 && osd->ft2->useme) proceed = 1;
+    if (osd->ft2) proceed = 1;
 #endif
     
     if (proceed == 0) {
@@ -1212,7 +1301,7 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
   while( inbytesleft ) {
 #ifdef HAVE_ICONV
     unicode = osd_iconv_getunicode(this->stream->xine, osd->cd, osd->encoding, 
-                                   (char **)&inbuf, &inbytesleft);
+                                   (ICONV_CONST char **)&inbuf, &inbytesleft);
 #else
     unicode = inbuf[0];
     inbuf++;
@@ -1220,7 +1309,7 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
 #endif
 
 #ifdef HAVE_FT2
-    if (osd->ft2 && osd->ft2->useme) {
+    if (osd->ft2) {
       FT_GlyphSlot  slot = osd->ft2->face->glyph;
 
       i = FT_Get_Char_Index( osd->ft2->face, unicode);
@@ -1260,12 +1349,12 @@ static int osd_get_text_size(osd_object_t *osd, const char *text, int *width, in
         *width += font->fontchar[i].width - (font->fontchar[i].width * FONT_OVERLAP);
       }
 #ifdef HAVE_FT2
-    } /* !(osd->ft2 && osd->ft2->useme) */
+    } /* !(osd->ft2) */
 #endif
   }
 
 #ifdef HAVE_FT2
-  if (osd->ft2 && osd->ft2->useme) {
+  if (osd->ft2) {
     /* if we have a true type font we need to do some corrections for the last
      * letter. As this one is still in the gylph slot we can still work with
      * it. For the last letter be must not use advance and width but the real
@@ -1310,10 +1399,9 @@ static void osd_preload_fonts (osd_renderer_t *this, char *path) {
 
         if( p ) {
 	  osd_font_t  *font;
-	  char        *pathname;
 
           *p++ = '\0';
-          font = xine_xmalloc( sizeof(osd_font_t) );
+          font = calloc(1, sizeof(osd_font_t) );
           
           strncpy(font->name, s, sizeof(font->name));
           font->size = atoi(p);
@@ -1321,9 +1409,7 @@ static void osd_preload_fonts (osd_renderer_t *this, char *path) {
           lprintf("font '%s' size %d is preloaded\n", 
                   font->name, font->size);
 
-          pathname = (char *) xine_xmalloc(strlen(path) + strlen(entry->d_name) + 2);
-          sprintf (pathname, "%s/%s", path, entry->d_name);
-          font->filename = pathname;
+          asprintf (&font->filename, "%s/%s", path, entry->d_name);
           
           font->next = this->fonts;
           this->fonts = font;
@@ -1371,7 +1457,16 @@ static void osd_free_object (osd_object_t *osd_to_close) {
   while( osd ) {
     if ( osd == osd_to_close ) {
       free( osd->area );
-      if( osd->ft2 ) free( osd->ft2 );
+
+#ifdef HAVE_FT2
+      if( osd->ft2 ) {
+	if ( osd->ft2->library )
+	  FT_Done_FreeType(osd->ft2->library);
+
+	free( osd->ft2 );
+      }
+#endif
+
       osd_free_encoding(osd);
       
       if( last )
@@ -1470,9 +1565,9 @@ osd_renderer_t *_x_osd_renderer_init( xine_stream_t *stream ) {
   osd_renderer_t *this;
   char str[1024];
 
-  this = xine_xmalloc(sizeof(osd_renderer_t)); 
+  this = calloc(1, sizeof(osd_renderer_t)); 
   this->stream = stream;
-  this->event.object.overlay = xine_xmalloc( sizeof(vo_overlay_t) );
+  this->event.object.overlay = calloc(1, sizeof(vo_overlay_t));
 
   pthread_mutex_init (&this->osd_mutex, NULL);
 
