@@ -15,13 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine_interface.c,v 1.100 2006/12/19 19:10:52 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * convenience/abstraction layer, functions to implement
  * libxine's public interface
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +32,7 @@
 #include <stdlib.h>
 #include <pthread.h>
 #include <stdarg.h>
-#if defined (__linux__)
+#if defined (__linux__) || defined (__GLIBC__)
 #include <endian.h>
 #elif defined (__FreeBSD__)
 #include <machine/endian.h>
@@ -341,6 +338,24 @@ int xine_port_send_gui_data (xine_video_port_t *vo,
 						  type, data);
 }
 
+static void send_audio_amp_event_internal(xine_stream_t *stream)
+{
+  xine_event_t            event;
+  xine_audio_level_data_t data;
+
+  data.left
+    = data.right
+    = stream->audio_out->get_property (stream->audio_out, AO_PROP_AMP);
+  data.mute
+    = stream->audio_out->get_property (stream->audio_out, AO_PROP_AMP_MUTE);
+
+  event.type        = XINE_EVENT_AUDIO_AMP_LEVEL;
+  event.data        = &data;
+  event.data_length = sizeof (data);
+
+  xine_event_send(stream, &event);
+}
+
 void xine_set_param (xine_stream_t *stream, int param, int value) {
   /* Avoid crashing */
   if ( ! stream ) {
@@ -412,15 +427,21 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
     
   case XINE_PARAM_AUDIO_AMP_LEVEL:
     stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
-    if (stream->audio_out)
-      stream->audio_out->set_property (stream->audio_out, AO_PROP_AMP, value);
+    if (stream->audio_out) {
+      int old_value = stream->audio_out->get_property (stream->audio_out, AO_PROP_AMP);
+      if (old_value != stream->audio_out->set_property (stream->audio_out, AO_PROP_AMP, value))
+        send_audio_amp_event_internal(stream);
+    }
     stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
     break;
 
   case XINE_PARAM_AUDIO_AMP_MUTE:
     stream->xine->port_ticket->acquire(stream->xine->port_ticket, 0);
-    if (stream->audio_out)
-      stream->audio_out->set_property (stream->audio_out, AO_PROP_AMP_MUTE, value);
+    if (stream->audio_out) {
+      int old_value = stream->audio_out->get_property (stream->audio_out, AO_PROP_AMP_MUTE);
+      if (old_value != stream->audio_out->set_property (stream->audio_out, AO_PROP_AMP_MUTE, value))
+        send_audio_amp_event_internal(stream);
+    }
     stream->xine->port_ticket->release(stream->xine->port_ticket, 0);
     break;
 
@@ -497,7 +518,7 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
     break;
   
   case XINE_PARAM_EARLY_FINISHED_EVENT:
-    stream->early_finish_event = value;
+    stream->early_finish_event = !!value;
     break;
   
   case XINE_PARAM_DELAY_FINISHED_EVENT:
@@ -505,7 +526,7 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
     break;
   
   case XINE_PARAM_GAPLESS_SWITCH:
-    stream->gapless_switch = value;
+    stream->gapless_switch = !!value;
     break;
     
   default:
@@ -923,13 +944,13 @@ xine_post_out_t * xine_get_audio_source(xine_stream_t *stream) {
 int _x_message(xine_stream_t *stream, int type, ...) {
   xine_ui_message_data_t *data;
   xine_event_t            event;
-  char                   *explanation;
-  int                     size;
+  const char              *explanation;
+  size_t                  size;
   int                     n;
   va_list                 ap;
   char                   *s, *params;
   char                   *args[1025];
-  static char            *std_explanation[] = {
+  static const char      *std_explanation[] = {
     "",
     N_("Warning:"),
     N_("Unknown host:"),
@@ -969,7 +990,7 @@ int _x_message(xine_stream_t *stream, int type, ...) {
   args[n] = NULL;
   
   size += sizeof(xine_ui_message_data_t) + 1;
-  data = xine_xmalloc( size );
+  data = calloc(1, size );
 
   strcpy(data->compatibility.str, "Upgrade your frontend to see the error messages");
   data->type           = type;
