@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2006 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -15,10 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xineutils.h,v 1.96 2004/12/12 13:51:30 valtri Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 #ifndef XINEUTILS_H
 #define XINEUTILS_H
@@ -43,6 +40,9 @@ extern "C" {
 #  include "xmlparser.h"
 #  include "xine_buffer.h"
 #  include "configfile.h"
+#  include "list.h"
+#  include "array.h"
+#  include "sorted_array.h"
 #else
 #  ifdef WIN32
 #    include <winsock.h>
@@ -55,13 +55,21 @@ extern "C" {
 #  include <xine/xmlparser.h>
 #  include <xine/xine_buffer.h>
 #  include <xine/configfile.h>
+#  include <xine/list.h>
+#  include <xine/array.h>
+#  include <xine/sorted_array.h>
 #endif
 
 #include <stdio.h>
 #include <string.h>
-/* for alloca under MinGW */
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
+
+/* 
+ * Mark exported data symbols for link engine library clients with older
+ * Win32 compilers
+ */
+#if defined(WIN32) && !defined(XINE_LIBRARY_COMPILE)
+#  define DL_IMPORT __declspec(dllimport)
+#  define extern DL_IMPORT extern
 #endif
 
   /*
@@ -75,11 +83,11 @@ extern "C" {
   } xine_mutex_t;
 
   int xine_mutex_init    (xine_mutex_t *mutex, const pthread_mutexattr_t *mutexattr,
-			  char *id);
+			  const char *id) XINE_PROTECTED;
 
-  int xine_mutex_lock    (xine_mutex_t *mutex, char *who);
-  int xine_mutex_unlock  (xine_mutex_t *mutex, char *who);
-  int xine_mutex_destroy (xine_mutex_t *mutex);
+  int xine_mutex_lock    (xine_mutex_t *mutex, const char *who) XINE_PROTECTED;
+  int xine_mutex_unlock  (xine_mutex_t *mutex, const char *who) XINE_PROTECTED;
+  int xine_mutex_destroy (xine_mutex_t *mutex) XINE_PROTECTED;
 
 
 
@@ -117,7 +125,7 @@ extern "C" {
 #define MM_SSE                  MM_ACCEL_X86_SSE
 #define MM_SSE2                 MM_ACCEL_X86_SSE2
 
-uint32_t xine_mm_accel (void);
+uint32_t xine_mm_accel (void) XINE_CONST XINE_PROTECTED;
 
 #if defined(ARCH_X86) || defined(ARCH_X86_64)
 
@@ -589,22 +597,11 @@ typedef	union {
 
 		     /* Optimized/fast memcpy */
 
-/*
-   TODO : fix dll linkage problem for xine_fast_memcpy on win32
-
-   xine_fast_memcpy dll linkage is screwy here.
-   declaring as dllimport seems to fix the problem
-   but causes compiler warning with libxineutils
-*/
-#ifdef _MSC_VER
-__declspec( dllimport ) extern void *(* xine_fast_memcpy)(void *to, const void *from, size_t len);
-#else
-extern void *(* xine_fast_memcpy)(void *to, const void *from, size_t len);
-#endif
+extern void *(* xine_fast_memcpy)(void *to, const void *from, size_t len) XINE_PROTECTED;
 
 #ifdef HAVE_XINE_INTERNAL_H
 /* Benchmark available memcpy methods */
-void xine_probe_fast_memcpy(xine_t *xine);
+void xine_probe_fast_memcpy(xine_t *xine) XINE_PROTECTED;
 #endif
 
 
@@ -614,53 +611,50 @@ void xine_probe_fast_memcpy(xine_t *xine);
 /*
  * profiling (unworkable in non DEBUG isn't defined)
  */
-void xine_profiler_init (void);
-int xine_profiler_allocate_slot (char *label);
-void xine_profiler_start_count (int id);
-void xine_profiler_stop_count (int id);
-void xine_profiler_print_results (void);
+void xine_profiler_init (void) XINE_PROTECTED;
+int xine_profiler_allocate_slot (const char *label) XINE_PROTECTED;
+void xine_profiler_start_count (int id) XINE_PROTECTED;
+void xine_profiler_stop_count (int id) XINE_PROTECTED;
+void xine_profiler_print_results (void) XINE_PROTECTED;
 
 /*
  * Allocate and clean memory size_t 'size', then return the pointer
  * to the allocated memory.
  */
-#if !defined(__GNUC__) || __GNUC__ < 3
-void *xine_xmalloc(size_t size);
-#else
-void *xine_xmalloc(size_t size) __attribute__ ((__malloc__));
-#endif
+void *xine_xmalloc(size_t size) XINE_MALLOC XINE_DEPRECATED XINE_PROTECTED;
 
 /*
  * Same as above, but memory is aligned to 'alignement'.
  * **base is used to return pointer to un-aligned memory, use
  * this to free the mem chunk
  */
-void *xine_xmalloc_aligned(size_t alignment, size_t size, void **base);
+void *xine_xmalloc_aligned(size_t alignment, size_t size, void **base) XINE_PROTECTED;
 
 /*
  * Get user home directory.
  */
-const char *xine_get_homedir(void);
+const char *xine_get_homedir(void) XINE_PROTECTED;
 
 #if defined(WIN32) || defined(__CYGWIN__)
 /*
  * Get other xine directories.
  */
-const char *xine_get_plugindir(void);
-const char *xine_get_fontdir(void);
-const char *xine_get_localedir(void);
+const char *xine_get_pluginroot(void) XINE_PROTECTED;
+const char *xine_get_plugindir(void) XINE_PROTECTED;
+const char *xine_get_fontdir(void) XINE_PROTECTED;
+const char *xine_get_localedir(void) XINE_PROTECTED;
 #endif
 
 /*
  * Clean a string (remove spaces and '=' at the begin,
  * and '\n', '\r' and spaces at the end.
  */
-char *xine_chomp (char *str);
+char *xine_chomp (char *str) XINE_PROTECTED;
 
 /*
  * A thread-safe usecond sleep
  */
-void xine_usec_sleep(unsigned usec);
+void xine_usec_sleep(unsigned usec) XINE_PROTECTED;
 
 
   /*
@@ -668,7 +662,7 @@ void xine_usec_sleep(unsigned usec);
    */
 
 
-void xine_strdupa(char *dest, char *src);
+void xine_strdupa(char *dest, char *src) XINE_PROTECTED XINE_DEPRECATED;
 #define xine_strdupa(d, s) do {                                             \
                                 (d) = NULL;                                 \
                                 if((s) != NULL) {                           \
@@ -682,6 +676,13 @@ void xine_strdupa(char *dest, char *src);
 #define xine_strsep(STRINGP, DELIM) strsep((STRINGP), (DELIM))
 #define xine_setenv(NAME, VAL, XX) setenv((NAME), (VAL), (XX))
 
+/**
+ * append to a string, reallocating
+ * normally, updates & returns *dest
+ * on error, *dest is unchanged & NULL is returned.
+ */
+char *xine_strcat_realloc (char **dest, char *append) XINE_PROTECTED;
+
 /*
  * Color Conversion Utility Functions
  * The following data structures and functions facilitate the conversion
@@ -700,34 +701,34 @@ typedef struct yuv_planes_s {
 
 } yuv_planes_t;
 
-void init_yuv_conversion(void);
-void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height);
-void free_yuv_planes(yuv_planes_t *yuv_planes);
+void init_yuv_conversion(void) XINE_PROTECTED;
+void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height) XINE_PROTECTED;
+void free_yuv_planes(yuv_planes_t *yuv_planes) XINE_PROTECTED;
 
 extern void (*yuv444_to_yuy2)
-  (yuv_planes_t *yuv_planes, unsigned char *yuy2_map, int pitch);
+  (const yuv_planes_t *yuv_planes, unsigned char *yuy2_map, int pitch) XINE_PROTECTED;
 extern void (*yuv9_to_yv12)
-  (unsigned char *y_src, int y_src_pitch, unsigned char *y_dest, int y_dest_pitch,
-   unsigned char *u_src, int u_src_pitch, unsigned char *u_dest, int u_dest_pitch,
-   unsigned char *v_src, int v_src_pitch, unsigned char *v_dest, int v_dest_pitch,
-   int width, int height);
+  (const unsigned char *y_src, int y_src_pitch, unsigned char *y_dest, int y_dest_pitch,
+   const unsigned char *u_src, int u_src_pitch, unsigned char *u_dest, int u_dest_pitch,
+   const unsigned char *v_src, int v_src_pitch, unsigned char *v_dest, int v_dest_pitch,
+   int width, int height) XINE_PROTECTED;
 extern void (*yuv411_to_yv12)
-  (unsigned char *y_src, int y_src_pitch, unsigned char *y_dest, int y_dest_pitch,
-   unsigned char *u_src, int u_src_pitch, unsigned char *u_dest, int u_dest_pitch,
-   unsigned char *v_src, int v_src_pitch, unsigned char *v_dest, int v_dest_pitch,
-   int width, int height);
+  (const unsigned char *y_src, int y_src_pitch, unsigned char *y_dest, int y_dest_pitch,
+   const unsigned char *u_src, int u_src_pitch, unsigned char *u_dest, int u_dest_pitch,
+   const unsigned char *v_src, int v_src_pitch, unsigned char *v_dest, int v_dest_pitch,
+   int width, int height) XINE_PROTECTED;
 extern void (*yv12_to_yuy2)
-  (unsigned char *y_src, int y_src_pitch,
-   unsigned char *u_src, int u_src_pitch,
-   unsigned char *v_src, int v_src_pitch,
+  (const unsigned char *y_src, int y_src_pitch,
+   const unsigned char *u_src, int u_src_pitch,
+   const unsigned char *v_src, int v_src_pitch,
    unsigned char *yuy2_map, int yuy2_pitch,
-   int width, int height, int progressive);
+   int width, int height, int progressive) XINE_PROTECTED;
 extern void (*yuy2_to_yv12)
-  (unsigned char *yuy2_map, int yuy2_pitch,
+  (const unsigned char *yuy2_map, int yuy2_pitch,
    unsigned char *y_dst, int y_dst_pitch,
    unsigned char *u_dst, int u_dst_pitch,
    unsigned char *v_dst, int v_dst_pitch,
-   int width, int height);
+   int width, int height) XINE_PROTECTED;
 
 #define SCALEFACTOR 65536
 #define CENTERSAMPLE 128
@@ -762,31 +763,31 @@ extern void (*yuy2_to_yv12)
   g = (packed_pixel & 0x07E0) >> 3; \
   b = (packed_pixel & 0x001F) << 3;
 
-extern int y_r_table[256];
-extern int y_g_table[256];
-extern int y_b_table[256];
+extern int y_r_table[256] XINE_PROTECTED;
+extern int y_g_table[256] XINE_PROTECTED;
+extern int y_b_table[256] XINE_PROTECTED;
 
-extern int u_r_table[256];
-extern int u_g_table[256];
-extern int u_b_table[256];
+extern int u_r_table[256] XINE_PROTECTED;
+extern int u_g_table[256] XINE_PROTECTED;
+extern int u_b_table[256] XINE_PROTECTED;
 
-extern int v_r_table[256];
-extern int v_g_table[256];
-extern int v_b_table[256];
+extern int v_r_table[256] XINE_PROTECTED;
+extern int v_g_table[256] XINE_PROTECTED;
+extern int v_b_table[256] XINE_PROTECTED;
 
 /* frame copying functions */
 extern void yv12_to_yv12
-  (unsigned char *y_src, int y_src_pitch, unsigned char *y_dst, int y_dst_pitch,
-   unsigned char *u_src, int u_src_pitch, unsigned char *u_dst, int u_dst_pitch,
-   unsigned char *v_src, int v_src_pitch, unsigned char *v_dst, int v_dst_pitch,
-   int width, int height);
+  (const unsigned char *y_src, int y_src_pitch, unsigned char *y_dst, int y_dst_pitch,
+   const unsigned char *u_src, int u_src_pitch, unsigned char *u_dst, int u_dst_pitch,
+   const unsigned char *v_src, int v_src_pitch, unsigned char *v_dst, int v_dst_pitch,
+   int width, int height) XINE_PROTECTED;
 extern void yuy2_to_yuy2
-  (unsigned char *src, int src_pitch,
+  (const unsigned char *src, int src_pitch,
    unsigned char *dst, int dst_pitch,
-   int width, int height);
+   int width, int height) XINE_PROTECTED;
 
 /* print a hexdump of the given data */
-void xine_hexdump (const char *buf, int length);
+void xine_hexdump (const char *buf, int length) XINE_PROTECTED;
 
 /*
  * Optimization macros for conditions
@@ -847,6 +848,7 @@ void xine_hexdump (const char *buf, int length);
       do {                                                                  \
         LONG_LOG_MODULE_STRING                                              \
         printf(fmt, ##args);                                                \
+        fflush(stdout);                                                     \
       } while(0)
   #else /* __GNUC__ */
     #ifdef _MSC_VER
@@ -854,12 +856,14 @@ void xine_hexdump (const char *buf, int length);
         do {                                                                \
           LONG_LOG_MODULE_STRING                                            \
           printf("%s", fmtargs);                                            \
+          fflush(stdout);                                                   \
         } while(0)
     #else /* _MSC_VER */
-      #define lprintf(fmt, ...)                                             \
+      #define lprintf(...)                                                  \
         do {                                                                \
           LONG_LOG_MODULE_STRING                                            \
           printf(__VA_ARGS__);                                              \
+          fflush(stdout);                                                   \
         } while(0)
     #endif  /* _MSC_VER */
   #endif /* __GNUC__ */
@@ -868,7 +872,7 @@ void xine_hexdump (const char *buf, int length);
     #define lprintf(fmt, args...)     do {} while(0)
   #else
   #ifdef _MSC_VER
-void __inline lprintf(const char * fmt, ...) {};
+void __inline lprintf(const char * fmt, ...) {}
   #else
     #define lprintf(...)              do {} while(0)
   #endif /* _MSC_VER */
@@ -956,99 +960,26 @@ void xine_xprintf(xine_t *xine, int verbose, const char *fmt, ...);
 #  define XINE_PROFILE_ACCUMULATE(function) function
 #endif /* DEBUG */
 
-
-/******** double chained lists with builtin iterator *******/
-
-typedef struct xine_node_s {
-
-  struct xine_node_s    *next, *prev;
-
-  void                  *content;
-
-  int                    priority;
-
-} xine_node_t;
-
-
-typedef struct {
-
-  xine_node_t    *first, *last, *cur;
-
-} xine_list_t;
-
-
-
-xine_list_t *xine_list_new (void);
-
-
-/**
- * dispose the whole list.
- * note: disposes _only_ the list structure, content must be free()d elsewhere
- */
-void xine_list_free(xine_list_t *l);
-
-
-/**
- * returns: Boolean
- */
-int xine_list_is_empty (xine_list_t *l);
-
-/**
- * return content of first entry in list.
- */
-void *xine_list_first_content (xine_list_t *l);
-
-/**
- * return next content in list.
- */
-void *xine_list_next_content (xine_list_t *l);
-
-/**
- * Return last content of list.
- */
-void *xine_list_last_content (xine_list_t *l);
-
-/**
- * Return previous content of list.
- */
-void *xine_list_prev_content (xine_list_t *l);
-
-/**
- * Append content to list, sorted by decreasing priority.
- */
-void xine_list_append_priority_content (xine_list_t *l, void *content, int priority);
-
-/**
- * Append content to list.
- */
-void xine_list_append_content (xine_list_t *l, void *content);
-
-/**
- * Insert content in list.
- */
-void xine_list_insert_content (xine_list_t *l, void *content);
-
-/**
- * Remove current content in list.
- * note: removes only the list entry; content must be free()d elsewhere.
- */
-void xine_list_delete_current (xine_list_t *l);
-
 /**
  * get encoding of current locale
  */
-char *xine_get_system_encoding(void);
+char *xine_get_system_encoding(void) XINE_MALLOC XINE_PROTECTED;
 
 /*
  * guess default encoding for the subtitles
  */
-const char *xine_guess_spu_encoding(void);
+const char *xine_guess_spu_encoding(void) XINE_PROTECTED;
 
 /*
  * use the best clock reference (API compatible with gettimeofday)
  * note: it will be a monotonic clock, if available.
  */
-int xine_monotonic_clock(struct timeval *tv, struct timezone *tz);      
+int xine_monotonic_clock(struct timeval *tv, struct timezone *tz) XINE_PROTECTED;
+
+/* don't harm following code */
+#ifdef extern
+#  undef extern
+#endif
 
 #ifdef __cplusplus
 }
